<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopgate_Helpers_Payment_Modules_Generic implements Shopgate_Interfaces_Payment, Shopgate_Interfaces_PaymentValid
{

    /** @var ShopgateCartBase | ShopgateOrder $order */
    protected $order;

    /**
     * Must rewrite this with module name
     * inside xtc3
     */
    const MODULE_NAME = '';

    /**
     * Rewrite if config key is different
     * than the module_name constant
     */
    const CONFIG_KEY = '';

    public function __construct(ShopgateCartBase $order)
    {
        $this->order = $order;
    }

    /**
     * @return ShopgateCartBase | ShopgateOrder
     */
    public function getOrder()
    {
        return $this->order;
    }

    /**
     * @inheritdoc
     */
    public function isValid()
    {
        return $this->isEnabled() && $this->isInstalled() && $this->isGenericValid();
    }

    /**
     * @inheritdoc
     */
    public function isEnabled()
    {
        $configKey = $this->getConstant('CONFIG_KEY');
        $moduleKey = $this->getConstant('MODULE_NAME');
        if (!empty($configKey)) {
            $moduleKey = $configKey;
        }

        if (empty($moduleKey)) {
            ShopgateLogger::getInstance()->log(sprintf('Bad keys provided "%s", "%s', $configKey, $moduleKey));
            throw new Shopgate_Exceptions_Payment('Module name or key must be provided in class: ' . get_class($this));
        }
        $dbHelper = new Shopgate_Helpers_Database();

        return $dbHelper->isPaymentModuleActive($moduleKey);
    }

    /**
     * @inheritdoc
     */
    public function isInstalled()
    {
        return true;
    }


    /**
     * @inheritdoc
     */
    public function isGenericValid()
    {
        return true;
    }

    /**
     * Added support for PHP version 5.2
     * $this::CONSTANT retrieval as static::
     * was supported at 5.3+
     *
     * @param string $input
     *
     * @return mixed
     */
    final protected function getConstant($input)
    {
        $configClass = new ReflectionClass($this);

        return $configClass->getConstant($input);
    }

    /**
     * ==========================================
     * =========== Payment Interface ============
     * ==========================================
     */

    /**
     * When rewriting make sure to inherit
     * this method as it uses special rewrites
     *
     * @inheritdoc
     */
    public function getModuleName()
    {
        return $this->getConstant('MODULE_NAME');
    }

    /**
     * @inheritdoc
     */
    public function getHistoryComments()
    {
        $comment = '';
        foreach ($this->getOrder()->getPaymentInfos() as $key => $value) {
            $comment .= $key . ': ' . $value . "\n";
        }

        return $comment;
    }

    /**
     * @inheritdoc
     */
    public function saveAdditionalInfo($orderId, array $data = null)
    {
    }
}
