<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopgate_Helpers_PluginInit
{
    /**
     * read all country data from the database by iso-2 code
     *
     * @param $country
     *
     * @return string
     */
    public function getCountryByIsoCode2($country)
    {
        $qry    = "SELECT * FROM " . TABLE_COUNTRIES . " WHERE UPPER(countries_iso_code_2) = UPPER('" . $country . "')";
        $result = xtc_db_query($qry);
        $qry    = xtc_db_fetch_array($result);

        return !empty($qry['countries_id']) ? $qry['countries_id'] : 'DE';
    }

    /**
     * read all the language data from the database by the language code
     *
     * @param $languageCode
     * @param $languageId
     * @param $language
     */
    public function getLanguageByCode($languageCode, &$languageId, &$language)
    {
        $qry        = "SELECT * FROM " . TABLE_LANGUAGES . " WHERE UPPER(code) = UPPER('" . $languageCode . "')";
        $result     = xtc_db_query($qry);
        $qry        = xtc_db_fetch_array($result);
        $languageId = !empty($qry['languages_id']) ? (int)$qry['languages_id'] : 2;
        if (!empty($qry['directory'])) {
            $language = $qry['directory'];
        }
    }

    /**
     * read all currency data from the database by the actual currency
     *
     * @param $currency
     * @param $exchangeRate
     * @param $currencyId
     * @param $currencyret
     */
    public function getCurrencyByCode($currency, &$exchangeRate, &$currencyId, &$currencyret)
    {
        $qry          = "SELECT * FROM " . TABLE_CURRENCIES . " WHERE UPPER(code) = UPPER('" . $currency . "')";
        $result       = xtc_db_query($qry);
        $qry          = xtc_db_fetch_array($result);
        $exchangeRate = !empty($qry['value']) ? $qry['value'] : 1;
        $currencyId   = !empty($qry['currencies_id']) ? $qry['currencies_id'] : 1;
        $currencyret  = !empty($qry)
            ? $qry
            : array(
                'code'            => 'EUR', 'symbol_left' => '',
                'symbol_right'    => ' EUR', 'decimal_point' => ',',
                'thousands_point' => '.', 'decimal_places' => '2',
                'value'           => 1.0
            );
    }

    /**
     * read all language data from database by the iso-2 code
     *
     * @param $isoCode
     *
     * @return mixed
     * @throws ShopgateLibraryException
     */
    public static function getLanguageIdByIsoCode($isoCode)
    {

        $isoCodeParts = explode('_', $isoCode);
        $isoCode      = isset($isoCodeParts[0]) ? $isoCodeParts[0] : $isoCode;
        $qry          = "SELECT * FROM `" . TABLE_LANGUAGES . "` WHERE UPPER(code) = UPPER('" . $isoCode . "')";
        $result       = xtc_db_query($qry);
        $resultItem   = xtc_db_fetch_array($result);

        if (!isset($resultItem['languages_id'])) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::UNKNOWN_ERROR_CODE, 'Invalid iso code given : ' . $isoCode
            );
        } else {
            return $resultItem['languages_id'];
        }
    }

    /**
     * read the path to the shop systems language files from the database by iso-2 code
     *
     * @param $isoCode
     *
     * @return mixed
     * @throws ShopgateLibraryException
     */
    public static function getLanguageDirectoryByIsoCode($isoCode)
    {

        $isoCodeParts = explode('_', $isoCode);
        $isoCode      = isset($isoCodeParts[0]) ? $isoCodeParts[0] : $isoCode;

        $qry        = "SELECT * FROM `" . TABLE_LANGUAGES . "` WHERE UPPER(code) = UPPER('" . $isoCode . "')";
        $result     = xtc_db_query($qry);
        $resultItem = xtc_db_fetch_array($result);

        if (!isset($resultItem['languages_id'])) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::UNKNOWN_ERROR_CODE, 'Invalid iso code given : ' . $isoCode
            );
        } else {
            return $resultItem['directory'];
        }
    }
}
