<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopgate_Models_Coupon
{
    const SG_COUPON_TYPE_GIFT          = 'G';
    const SG_COUPON_TYPE_PERCENTAGE    = 'P';
    const SG_COUPON_TYPE_FIX           = 'F';
    const SG_COUPON_TYPE_FREE_SHIPPING = 'S';
    const SG_COUPON_ACTIVE             = 'Y';

    private $currencyCode;
    private $config;
    private $languageId;
    private $language;
    private $countryId;

    /**
     * @param string            $currencyCode
     * @param ShopgateConfigXtc $config
     * @param int               $languageId
     * @param string            $language
     * @param int               $countryId
     */
    public function __construct($currencyCode, $config, $languageId, $language, $countryId)
    {
        $this->currencyCode = $currencyCode;
        $this->config       = $config;
        $this->languageId   = $languageId;
        $this->language     = $language;
        $this->countryId    = $countryId;
        $this->initializeCouponModule();
    }

    /**
     * Read the coupon data from the database by coupon code
     *
     * @param string $code
     *
     * @return array
     */
    public function getCouponByCode($code)
    {
        $code = xtc_db_prepare_input($code);
        $couponQuery
              = "SELECT * FROM `" . TABLE_COUPONS . "` AS c " .
            "LEFT JOIN `" . TABLE_COUPONS_DESCRIPTION . "` AS cd ON cd.coupon_id=c.coupon_id " .
            "WHERE c.coupon_code='{$code}' AND cd.language_id={$this->languageId}";

        $coupon = xtc_db_fetch_array(xtc_db_query($couponQuery));
        // check if coupon is an gift voucher
        if (empty($coupon)) {
            $couponQuery = "SELECT * FROM `" . TABLE_COUPONS . "` AS c WHERE c.coupon_code='{$code}'";
            $coupon      = xtc_db_fetch_array(xtc_db_query($couponQuery));
        }

        return $coupon;
    }

    /**
     * Check if a coupon is valid
     * there are different types of coupons
     * G = gift, this coupon will be created if a customer registered a new account in the shop
     * S = shipping, free shipping i guess
     * F = fixed, the coupon value is a fixed value, which needs to be subtracted from item(s)/ the whole cart,
     *     depending on the coupon's setting
     * P = Percentage, the coupon amount is a percentage value which needs to be used to calculate the amount to
     *     subtract for item(s)/ the whole cart, depending on the coupon's setting
     *
     * @param ShopgateCart                  $cart
     * @param Shopgate_Models_Item_CartItem $cartItemModel
     * @param array                         $coupon
     * @param float|string                  $orderAmount
     *
     * @return string
     */
    public function validateCoupon(
        ShopgateCart $cart,
        Shopgate_Models_Item_CartItem $cartItemModel,
        array $coupon,
        $orderAmount
    ) {
        $msg = "";
        if ($coupon['coupon_type'] != self::SG_COUPON_TYPE_GIFT) {

            if (empty($coupon) || $coupon['coupon_active'] !== self::SG_COUPON_ACTIVE) {
                $msg .= ShopgateLibraryException::COUPON_NOT_VALID . "\n";
            }

            $currentDate = date("Y-m-d H:i:s");
            if ($coupon['coupon_start_date'] >= $currentDate) {
                $msg .= ERROR_INVALID_STARTDATE_COUPON . "\n";
            }

            if ($coupon['coupon_expire_date'] <= $currentDate) {
                $msg .= ERROR_INVALID_FINISDATE_COUPON . "\n";
            }

            if ($coupon['coupon_minimum_order'] > $orderAmount) {
                $xtPrice   = new xtcPrice($this->currencyCode, "");
                $minAmount = $xtPrice->xtcFormat($coupon['coupon_minimum_order'], true);

                $msg .= (defined("ERROR_MINIMUM_ORDER_COUPON_1")
                        ? ERROR_MINIMUM_ORDER_COUPON_1 . '(' . $minAmount . ')'
                        : sprintf(TEXT_COUPON_HELP_MINORDER, $minAmount))
                    . "\n";
            }

            if (!$this->checkCouponRedeemAmount($coupon)) {
                $msg .= ERROR_INVALID_USES_COUPON . $coupon['uses_per_coupon'] . "\n";
            }

            if (!$this->checkCouponRedeemAmountToCustomer($coupon, $cart->getExternalCustomerId())) {
                $msg .= ERROR_INVALID_USES_USER_COUPON . $coupon['uses_per_user'] . TIMES . "\n";
            }

            if ($coupon['restrict_to_products']
                && !$this->cartHasRestrictedProduct($coupon, $cart->getItems(), $cartItemModel)
            ) {
                $msg .= TEXT_COUPON_HELP_RESTRICT . "\n";
            }

            if ($coupon['restrict_to_categories']
                && !$this->cartHasRestrictedProductToCategory($coupon, $cart->getItems(), $cartItemModel)
            ) {
                $msg .= TEXT_COUPON_HELP_RESTRICT . "\n";
            }
        } elseif ($coupon['coupon_type'] == self::SG_COUPON_TYPE_GIFT) {
            // Seems to be a bug in modified. It's allowed to use the voucher, but there is no
            // price reduction on order. As workaround we set the coupon as invalid.
            $msg .= ERROR_INVALID_STARTDATE_COUPON . "\n";
        }

        return $msg;
    }

    /**
     * Check redeem amount to a coupon
     *
     * @param array $coupon_result
     *
     * @return bool
     */
    private function checkCouponRedeemAmount($coupon_result)
    {
        $coupon_count = xtc_db_query(
            "SELECT coupon_id FROM " . TABLE_COUPON_REDEEM_TRACK . " WHERE coupon_id = '" . $coupon_result['coupon_id']
            . "'"
        );

        return (xtc_db_num_rows($coupon_count) >= $coupon_result['uses_per_coupon']
            && $coupon_result['uses_per_coupon'] > 0) ? false : true;
    }

    /**
     * Check redeem amount to customer for a coupon
     *
     * @param array $coupon_result
     * @param int   $customerId
     *
     * @return bool
     */
    private function checkCouponRedeemAmountToCustomer($coupon_result, $customerId)
    {
        $coupon_count_customer = xtc_db_query(
            "SELECT coupon_id FROM " . TABLE_COUPON_REDEEM_TRACK . " WHERE coupon_id = '" . $coupon_result['coupon_id']
            . "' AND customer_id = '" . (int)$customerId . "'"
        );

        return (xtc_db_num_rows($coupon_count_customer) >= $coupon_result['uses_per_user']
            && $coupon_result['uses_per_user'] > 0) ? false : true;
    }

    /**
     * Checks if restricted (white listed) product is in cart.
     * The logic dictates that if a restricted product is in cart,
     * then we need to allow discount application.
     *
     * @param array                         $coupon
     * @param ShopgateOrderItem[]           $items
     * @param Shopgate_Models_Item_CartItem $cartItemModel
     *
     * @return bool
     */
    private function cartHasRestrictedProduct(array $coupon, $items, Shopgate_Models_Item_CartItem $cartItemModel)
    {
        $ids = explode(",", $coupon['restrict_to_products']);
        foreach ($items as $cartItem) {
            $id = $cartItemModel->getProductIdFromCartItem($cartItem);
            if (in_array($id, $ids)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Checks if restricted (white listed) category is applied to a product
     * in the current cart. Logic dictates that if a restricted product is in cart,
     * then we need to allow discount application.
     *
     * @param array                         $coupon
     * @param ShopgateOrderItem[]           $items
     * @param Shopgate_Models_Item_CartItem $cartItemModel
     *
     * @return bool
     */
    private function cartHasRestrictedProductToCategory(
        array $coupon,
        $items,
        Shopgate_Models_Item_CartItem $cartItemModel
    ) {
        $categoryIds = explode(',', $coupon['restrict_to_categories']);
        foreach ($items as $item) {
            $id                      = $cartItemModel->getProductIdFromCartItem($item);
            $categoryPath            = xtc_get_product_path(xtc_get_prid($id));
            $productCategoryIdsArray = explode("_", $categoryPath);
            for ($ii = 0, $nn = count($categoryIds); $ii < $nn; $ii++) {
                if (in_array($categoryIds[$ii], $productCategoryIdsArray)) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Fill the ShopgateExternalCoupon object with data e.g. coupon amount.
     *
     * @param array                         $coupon      - the coupon database data
     * @param ShopgateExternalCoupon        $sgCoupon
     * @param ShopgateCart                  $cart
     * @param Shopgate_Models_Item_CartItem $cartItemModel
     * @param float|string                  $orderAmount - the whole order amount to calculate % discount off of
     */
    public function setCouponData(
        array $coupon,
        ShopgateExternalCoupon $sgCoupon,
        ShopgateCart $cart,
        Shopgate_Models_Item_CartItem $cartItemModel,
        $orderAmount
    ) {
        $creditAmount = 0;
        $realDiscount = $this->calculateCouponAmount($coupon['coupon_amount'], $coupon['coupon_type'], $orderAmount);
        switch ($coupon['coupon_type']) {
            case self::SG_COUPON_TYPE_FREE_SHIPPING:
                $sgCoupon->setIsFreeShipping(true);
                $creditAmount = $realDiscount;
                break;

            case self::SG_COUPON_TYPE_FIX :
            case self::SG_COUPON_TYPE_PERCENTAGE :
                if ($coupon['restrict_to_products']) {
                    if ($this->cartHasRestrictedProduct($coupon, $cart->getItems(), $cartItemModel)) {
                        $creditAmount = $realDiscount;
                    }
                } elseif ($coupon['restrict_to_categories']) {
                    if ($this->cartHasRestrictedProductToCategory($coupon, $cart->getItems(), $cartItemModel)) {
                        $creditAmount = $realDiscount;
                    }
                } else {
                    $creditAmount = $realDiscount;
                }

                $sgCoupon->setIsFreeShipping(false);
                break;

            // Nothing to do here. The coupon will be marked as "accepted" by modified but there is
            // no price reduction
            case self::SG_COUPON_TYPE_GIFT:
            default :
                break;
        }

        $sgCoupon->setAmountGross($creditAmount);
        $sgCoupon->setName($coupon['coupon_name']);
        $sgCoupon->setCode($coupon['coupon_code']);
        $sgCoupon->setCurrency($this->currencyCode);
        $sgCoupon->setDescription($coupon['coupon_description']);
    }

    /**
     * Calc
     *
     * @param float|string $couponAmount
     * @param string       $couponType
     * @param float|string $orderAmount
     *
     * @return float
     */
    private function calculateCouponAmount($couponAmount, $couponType, $orderAmount)
    {
        return $couponType === self::SG_COUPON_TYPE_PERCENTAGE
            ? $orderAmount * ($couponAmount / 100)
            : (float) $couponAmount;
    }

    /**
     * Redeem the coupon in the shop system
     *
     * @param ShopgateExternalCoupon $sgCoupon
     * @param int                    $customerId
     * @param int                    $orderId
     *
     * @return void
     */
    public function redeemCoupon(ShopgateExternalCoupon $sgCoupon, $customerId, $orderId)
    {
        $coupon = $this->getCouponByCode($sgCoupon->getCode());
        if (empty($coupon)) {
            return;
        }

        if ($coupon['coupon_type'] == self::SG_COUPON_TYPE_GIFT) {
            $this->proceedWelcomeVoucher($coupon, $customerId, $orderId);
        } else {
            $this->insertRedeemInformation($coupon['coupon_id'], $customerId, $orderId);
        }
    }

    /**
     * Insert the order total value for coupons
     *
     * @param int                    $orderId
     * @param ShopgateExternalCoupon $sgCoupon
     * @param int                    $sort
     *
     * @return float
     */
    public function insertOrderTotal($orderId, ShopgateExternalCoupon $sgCoupon, $sort)
    {
        $xtPrice      = new xtcPrice($this->currencyCode, "");
        $insertAmount = $sgCoupon->getAmountGross() * (-1);
        $printAmount  = $xtPrice->xtcFormat($insertAmount, true);

        $orderTotal = array(
            'orders_id'  => $orderId,
            'title'      => MODULE_ORDER_TOTAL_COUPON_TITLE . ':' . $sgCoupon->getCode() . ':',
            'text'       => '<strong><span style="color:#ff0000">' . $printAmount . '</span></strong>',
            'value'      => $insertAmount,
            'class'      => 'ot_coupon',
            'sort_order' => $sort,
		);

        xtc_db_perform(TABLE_ORDERS_TOTAL, $orderTotal);

        return $sgCoupon->getAmountGross();
    }

    /**
     * Include the language file to the shop module ot_coupon
     */
    private function initializeCouponModule()
    {
        if (!class_exists('ot_coupon')) {
            $couponModuleFile = DIR_FS_LANGUAGES . $this->language . '/modules/order_total/ot_coupon.php';
            if (file_exists($couponModuleFile)) {
                /** @noinspection PhpIncludeInspection */
                require_once($couponModuleFile);
            }
        }
    }

    /**
     * Insert redeem information into the database
     *
     * @param int $couponId
     * @param int $customerId
     * @param int $orderId
     */
    private function insertRedeemInformation($couponId, $customerId, $orderId)
    {
        global $REMOTE_ADDR;
        xtc_db_query(
            "INSERT INTO  " . TABLE_COUPON_REDEEM_TRACK
            . " (coupon_id, customer_id, redeem_date, redeem_ip, order_id) VALUES ('"
            . $couponId . "', '" . $customerId . "', now(),'" . $REMOTE_ADDR . "','" . $orderId . "')"
        );
    }

    /**
     * Proceed the welcome voucher. This voucher will be disabled after one use
     *
     * @param array $coupon
     * @param int   $customerId
     * @param int   $orderId
     */
    private function proceedWelcomeVoucher(array $coupon, $customerId, $orderId)
    {
        $couponAmount          = $coupon['coupon_amount'];
        $gvCustomerAmount      = $this->getCustomerGvAmount($customerId);
        $totalGvCustomerAmount = $couponAmount + $gvCustomerAmount;

        $this->setCouponInactive($coupon['coupon_id']);
        $this->insertRedeemInformation($coupon['coupon_id'], $customerId, $orderId);

        if ($gvCustomerAmount > 0) {
            // already has gv_amount so update
            $this->updateCustomerGvAmount($totalGvCustomerAmount, $customerId);
        } else {
            // no gv_amount so insert
            $this->insertCustomersGvAmount($totalGvCustomerAmount, $customerId);
        }
    }

    /**
     * Read existing customers gv amount from database by customer's id
     *
     * @param int $customerId
     *
     * @return array|bool|int|mixed
     */
    private function getCustomerGvAmount($customerId)
    {
        $customerAmountQuery = xtc_db_query(
            "SELECT amount FROM " . TABLE_COUPON_GV_CUSTOMER . " WHERE customer_id = '" . $customerId . "'"
        );

        if ($customerAmount = xtc_db_fetch_array($customerAmountQuery)) {
            $customerAmount = !empty($customerAmount['amount']) ? $customerAmount['amount'] : 0;
        }

        return $customerAmount;
    }

    /**
     * Set coupon as inactive in the database
     *
     * @param int $couponId
     */
    private function setCouponInactive($couponId)
    {
        xtc_db_query("UPDATE " . TABLE_COUPONS . " SET coupon_active = 'N' WHERE coupon_id = '" . $couponId . "'");
    }

    /**
     * Updates a customers gift value amount into the database by customer's id
     *
     * @param float|string $totalGvAmount
     * @param int          $customerId
     */
    private function updateCustomerGvAmount($totalGvAmount, $customerId)
    {
        xtc_db_query(
            "UPDATE " . TABLE_COUPON_GV_CUSTOMER . " SET amount = '" . $totalGvAmount
            . "' WHERE customer_id = '" . $customerId . "'"
        );
    }

    /**
     * Stores a customers gift value amount into the database by customer's id
     *
     * @param float|string $totalGvAmount
     * @param int          $customerId
     */
    private function insertCustomersGvAmount($totalGvAmount, $customerId)
    {
        xtc_db_query(
            "INSERT INTO " . TABLE_COUPON_GV_CUSTOMER . " (customer_id, amount) VALUES ('"
            . $customerId . "', '" . $totalGvAmount . "')"
        );
    }
}
