<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopgate_Models_Customer
{
    const TABLE_CUSTOMERS_SHOPGATE_CUSTOMER = 'customers_shopgate_customer';

    /**
     * @var ShopgateConfigXtc
     */
    protected $config;

    /**
     * @var int
     */
    protected $languageId;

    /**
     * @param ShopgateConfigXtc $config
     * @param int               $languageId
     */
    public function __construct(ShopgateConfigXtc $config, $languageId)
    {
        $this->config     = $config;
        $this->languageId = $languageId;
    }

    /**
     * generate the discount to, in the config set, customers group
     *
     * @return mixed
     */
    public function getCustomerDiscountGroupAttributes()
    {
        // get customer-group first
        $qry = "SELECT" . " status.customers_status_name,"
            . " status.customers_status_discount,"
            . " status.customers_status_discount_attributes" . " FROM "
            . TABLE_CUSTOMERS_STATUS . " AS status"
            . " WHERE status.customers_status_id = "
            . $this->config->getCustomerPriceGroup()
            . " AND status.language_id = " . $this->languageId . ";";

        // Check if the customer group exists (ignore if not)
        $queryResult = xtc_db_query($qry);
        if ($queryResult) {

            $customerGroupResult = xtc_db_fetch_array($queryResult);

            if (!empty($customerGroupResult) && isset($customerGroupResult['customers_status_discount'])) {
                $customerGroupMaxPriceDiscount = $customerGroupResult['customers_status_discount'];
            }

            if (!empty($customerGroupResult) && isset($customerGroupResult['customers_status_discount'])) {
                $customerGroupDiscountAttributes = $customerGroupResult['customers_status_discount_attributes']
                    ? true
                    : false;
            }
        }

        return $customerGroupDiscountAttributes;
    }

    /**
     * read customer databa from the database by uid
     *
     * @param $customerId
     *
     * @return mixed
     */
    public function getCustomerById($customerId)
    {
        $result = xtc_db_query("SELECT * FROM " . TABLE_CUSTOMERS . " WHERE customers_id = '{$customerId}'");

        return xtc_db_fetch_array($result);
    }

    /**
     * generate a ShopgateCustomer object using customer data from
     * database, regarding the customer group
     *
     * @param $customerId
     *
     * @return null|\ShopgateCartCustomer
     */
    public function getCartCustomer($customerId)
    {
        $customer = $this->getCustomerById($customerId);

        if (empty($customer)) {
            return null;
        }

        $customerGroup = new ShopgateCartCustomerGroup();
        $customerGroup->setId($customer["customers_status"]);

        $shopgateCustomer = new ShopgateCartCustomer();
        $shopgateCustomer->setCustomerGroups(array($customerGroup));

        return $shopgateCustomer;
    }

    /**
     * read the address data from the database by a customers uid
     *
     * @param $customersId
     * @param $customersDefaultAddressId
     *
     * @return mixed
     */
    public function getAddressByCustomersId($customersId, $customersDefaultAddressId)
    {
        // get customers address
        $qry       = " SELECT	* FROM " . TABLE_ADDRESS_BOOK . " AS ab	WHERE ab.customers_id = '{$customersId}'"
            . (!empty($customersDefaultAddressId)
                ? (" AND ab.address_book_id = '{$customersDefaultAddressId}'")
                : ""
            ) . ";";
        $qryResult = xtc_db_query($qry);

        return xtc_db_fetch_array($qryResult);
    }


    /**
     * check if the current customers status exist in the
     * database, regarding the language
     *
     * @param $shopCustomer
     *
     * @return mixed
     * @throws \ShopgateLibraryException
     */
    public function customerStatusExist($shopCustomer)
    {
        $result          = xtc_db_query(
            "SELECT * FROM " . TABLE_CUSTOMERS_STATUS .
            " WHERE language_id = '{$this->languageId}' AND customers_status_id = '{$shopCustomer["customers_status"]}'"
        );
        $customersStatus = xtc_db_fetch_array($result);
        if (empty($customersStatus)) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_NO_CUSTOMER_GROUP_FOUND,
                print_r($shopCustomer, true)
            );
        }

        return $customersStatus;
    }

    /**
     * return an array with all customer groups
     *
     * @return array
     */
    public function getCustomerGroups()
    {
        $customerGroups = array();

        $query = "SELECT 
                        cs.customers_status_name AS name,
                        cs.customers_status_id AS id,
                        0 AS 'is_default'
                    FROM " . TABLE_CUSTOMERS_STATUS . " AS cs
                    WHERE cs.language_id = {$this->languageId}
                    ORDER BY cs.customers_status_id";

        $result = xtc_db_query($query);

        while ($customerGroup = xtc_db_fetch_array($result)) {
            foreach ($customerGroup AS &$value) {
                $value = utf8_encode($value);
            }
            if ($customerGroup['id'] == DEFAULT_CUSTOMERS_STATUS_ID_GUEST) {
                $customerGroup['is_default'] = 1;
            }
            $customerGroup['customer_tax_class_key'] = 'default';
            $customerGroups[]                        = $customerGroup;
        }

        return $customerGroups;
    }

    /**
     * read a customers token from the database by customers uid
     *
     * @param string $internalCustomerId
     *
     * @return  bool | string
     */
    public function getCustomerToken($internalCustomerId)
    {
        $query  = "SELECT customer_token
                FROM " . Shopgate_Models_Customer::TABLE_CUSTOMERS_SHOPGATE_CUSTOMER . "
                WHERE customer_id = {$internalCustomerId}";
        $res    = xtc_db_query($query);
        $result = xtc_db_fetch_array($res);

        if (is_array($result)) {
            return $result['customer_token'];
        }

        return false;
    }

    /**
     * check if a toke exist to a customer
     *
     * @param $internalCustomerId
     *
     * @return bool
     */
    public function hasCustomerToken($internalCustomerId)
    {
        return $this->getCustomerToken($internalCustomerId) ? true : false;
    }

    /**
     * store the token to a customer in the database
     *
     * @param $internalCustomerId
     * @param $eMailAddress
     *
     * @return string
     */
    public function insertToken($internalCustomerId, $eMailAddress)
    {
        $token = md5($internalCustomerId . $eMailAddress . microtime());

        xtc_db_query(
            "INSERT INTO `" . Shopgate_Models_Customer::TABLE_CUSTOMERS_SHOPGATE_CUSTOMER . "` " .
            "(`customer_id`, `customer_token`) VALUES " .
            "(" . xtc_db_input($internalCustomerId) . ", '" . xtc_db_input($token) . "')"
        );

        return $token;
    }

    /**
     * read the customers uid from the database by its token
     *
     * @param string $token
     *
     * @return bool | int
     */
    public function getCustomerIdByToken($token)
    {
        $query  = "SELECT customer_id
                FROM " . Shopgate_Models_Customer::TABLE_CUSTOMERS_SHOPGATE_CUSTOMER . "
                WHERE customer_token = '" . $token . "'";
        $res    = xtc_db_query($query);
        $result = xtc_db_fetch_array($res);

        return isset($result['customer_id']) ? $result['customer_id'] : false;
    }
}
