<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopgate_Models_Customer_Order
{
    const DEFAULT_CUSTOMER_ADDRESS_IDENTIFIER_DELIVERY = 'delivery';
    const DEFAULT_CUSTOMER_ADDRESS_IDENTIFIER_INVOICE  = 'invoice';

    /**
     * @var mixed
     */
    protected $_customerId = false;

    /**
     * @param string            $customerToken
     * @param ShopgateConfigXtc $config
     * @param int               $languageId
     *
     * @throws ShopgateLibraryException
     */
    public function __construct($customerToken, ShopgateConfigXtc $config, $languageId)
    {
        $shopgateCustomerModel = new Shopgate_Models_Customer($config, $languageId);
        $this->_customerId     = $shopgateCustomerModel->getCustomerIdByToken($customerToken);

        if (!$this->_customerId) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_WRONG_USERNAME_OR_PASSWORD,
                'Username or password is incorrect'
            );
        }
    }

    /**
     * generate order data to all shopgate orders
     *
     * @param        $customerLanguage
     * @param int    $limit
     * @param int    $offset
     * @param string $orderDateFrom
     * @param string $sortOrder
     *
     * @return array
     */
    public function getOrders(
        $customerLanguage,
        $limit = 10,
        $offset = 0,
        $orderDateFrom = '',
        $sortOrder = 'created_desc'
    ) {

        $result = array();

        if (!class_exists('order')) {
            require_once(DIR_WS_INCLUDES . '/classes/order.php');
        }

        if (!class_exists('payment')) {
            require_once(DIR_WS_INCLUDES . '/classes/payment.php');
        }

        $getOrdersResource = $this->_getOrdersResource(
            Shopgate_Helpers_PluginInit::getLanguageIdByIsoCode($customerLanguage),
            $limit,
            $offset,
            $orderDateFrom,
            $sortOrder
        );

        while ($order = xtc_db_fetch_array($getOrdersResource)) {
            /** @var order $orderDetail */
            $orderDetail         = new order($order['orders_id']);
            $shopgateOrder       = new ShopgateExternalOrder();
            $orderTotalsResource = $this->_getOrderTotalsResource($order['orders_id']);
            $orderTotals         = array();

            while ($_orderTotal = xtc_db_fetch_array($orderTotalsResource)) {
                $orderTotals[$_orderTotal['class']][] = $_orderTotal;
            }

            $shopgateOrder->setOrderNumber($this->_getShopgateOrderId($order['orders_id']));
            $shopgateOrder->setExternalOrderNumber($order['orders_id']);
            $shopgateOrder->setExternalOrderId($order['orders_id']);

            $shopgateOrder->setStatusName($order['orders_status_name']);

            $shopgateOrder->setCreatedTime($order['date_purchased']);
            $shopgateOrder->setMail($order['customers_email_address']);
            $shopgateOrder->setPhone($order['customers_telephone']);
            $shopgateOrder->setCurrency($order['currency']);
            $shopgateOrder->setPaymentMethod(
                $this->_getPaymentMethodTitle($order['payment_method'], $customerLanguage)
            );

            $totals = $orderDetail->getTotalData($order['orders_id']);

            $shopgateOrder->setAmountComplete(round($totals['total'], 2));
            $shopgateOrder = $this->_setShippingInfo($shopgateOrder, $customerLanguage);
            $shopgateOrder->setExtraCosts($this->_getExtraCosts($orderTotals));
            $shopgateOrder->setExternalCoupons($this->_getExternalCoupons($orderTotals, $shopgateOrder->getCurrency()));
            $shopgateOrder->setDeliveryAddress(
                $this->_getAddress(
                    $orderDetail->delivery,
                    $order,
                    Shopgate_Models_Customer_Order::DEFAULT_CUSTOMER_ADDRESS_IDENTIFIER_DELIVERY
                )
            );
            $shopgateOrder->setInvoiceAddress(
                $this->_getAddress(
                    $orderDetail->billing,
                    $order,
                    Shopgate_Models_Customer_Order::DEFAULT_CUSTOMER_ADDRESS_IDENTIFIER_INVOICE
                )
            );

            $shopgateOrder->setOrderTaxes($this->_getOrderTaxes($orderTotals));
            $orderItems = array();

            foreach ($orderDetail->products as $product) {
                $orderItem = new ShopgateExternalOrderItem();
                $orderItem->setItemNumber($product['orders_products_id']);
                $orderItem->setQuantity((int)$product['qty']);
                $orderItem->setName($product['name']);
                $orderItem->setUnitAmount(round($product['price'] / ($product['tax'] / 100 + 1), 2));
                $orderItem->setUnitAmountWithTax(round($product['price'], 2));
                $orderItem->setTaxPercent((float)$product['tax']);
                $orderItem->setCurrency($order['currency']);

                if (isset($product['attributes']) && count($product['attributes']) > 0) {
                    $attributes = array();
                    foreach ($product['attributes'] as $attribute) {
                        array_push($attributes, $attribute['option'] . ': ' . $attribute['value']);
                    }
                    $orderItem->setName($orderItem->getName() . ' - ' . implode(', ', $attributes));
                }

                array_push($orderItems, $orderItem);
            }

            $shopgateOrder->setItems($orderItems);

            array_push($result, $shopgateOrder);
        }

        return $result;
    }

    /**
     * generate payment method title by payment- and language code
     *
     * @param $paymentCode
     * @param $languageCode
     *
     * @return mixed
     */
    protected function _getPaymentMethodTitle($paymentCode, $languageCode)
    {
        $paymentDir         = DIR_WS_MODULES . 'payment/' . $paymentCode . '.php';
        $paymentTranslation =
            DIR_FS_CATALOG . 'lang/' . Shopgate_Helpers_PluginInit::getLanguageDirectoryByIsoCode($languageCode)
            . '/modules/payment/' . $paymentCode . '.php';

        /**
         * sorry - xtcommerce nativ
         */
        if (file_exists($paymentTranslation) && $paymentDir) {
            require_once($paymentDir);
            require_once($paymentTranslation);

            $class   = $paymentCode;
            $payment = new $class();

            return $payment->title;

        }

        return $paymentCode;

    }

    /**
     * calculate the extra shipping costs
     *
     * @param array $orderTotals
     *
     * @return array
     */
    protected function _getExtraCosts($orderTotals)
    {
        $result = array();
        if (isset($orderTotals['ot_shipping']) && is_array($orderTotals['ot_shipping'])) {
            foreach ($orderTotals['ot_shipping'] as $shippingItem) {
                if ($shippingItem['value'] > 0) {
                    $extraCostItem = new ShopgateExternalOrderExtraCost();
                    $extraCostItem->setType('shipping');
                    $extraCostItem->setAmount(round($shippingItem['value'], 2));
                    $extraCostItem->setLabel($shippingItem['title']);
                    array_push($result, $extraCostItem);
                }
            }
        }

        return $result;
    }

    /**
     * generates a list of coupons to a order by totals and currency
     *
     * @param array  $orderTotals
     * @param string $currency
     *
     * @return array
     */
    protected function _getExternalCoupons($orderTotals, $currency)
    {
        $result = array();

        if (array_key_exists('ot_coupon', $orderTotals) && is_array($orderTotals['ot_coupon'])) {
            foreach ($orderTotals['ot_coupon'] as $couponItem) {

                /**
                 * prepare data
                 */
                $couponCodePartials = explode(':', $couponItem['title']);

                if (array_key_exists(1, $couponCodePartials)) {
                    $shopgateCoupon = new ShopgateExternalCoupon();
                    $shopgateCoupon->setOrderIndex($couponItem['sort_order']);
                    $shopgateCoupon->setCode(trim($couponCodePartials[1]));
                    $shopgateCoupon->setAmount(round(abs($couponItem['value']), 2));
                    $shopgateCoupon->setCurrency($currency);
                    $shopgateCoupon->setName(trim($couponCodePartials[0]));

                    array_push($result, $shopgateCoupon);
                }
            }
        }

        return $result;
    }

    /**
     * generate an fill a ShopgateAddress object with a customers address data
     *
     * @param        $address
     * @param        $order
     * @param string $addressType
     *
     * @return ShopgateAddress
     */
    protected function _getAddress(
        $address,
        $order,
        $addressType = Shopgate_Models_Customer_Order::DEFAULT_CUSTOMER_ADDRESS_IDENTIFIER_DELIVERY
    ) {
        $resultAddress = new ShopgateAddress();

        switch ($addressType) {
            case Shopgate_Models_Customer_Order::DEFAULT_CUSTOMER_ADDRESS_IDENTIFIER_DELIVERY :
                $resultAddress->setIsDeliveryAddress(true);
                $resultAddress->setCountry($order['delivery_country_iso_code_2']);
                break;
            case Shopgate_Models_Customer_Order::DEFAULT_CUSTOMER_ADDRESS_IDENTIFIER_INVOICE :
                $resultAddress->setIsInvoiceAddress(true);
                $resultAddress->setCountry($order['billing_country_iso_code_2']);
                break;
        }

        $resultAddress->setFirstName($address['firstname']);
        $resultAddress->setLastName($address['lastname']);
        $resultAddress->setCompany($address['company']);
        $resultAddress->setStreet1($address['street_address']);
        $resultAddress->setZipcode($address['postcode']);
        $resultAddress->setCity($address['city']);

        return $resultAddress;

    }

    /**
     * get the taxes to a order
     *
     * @param array $orderTaxes
     *
     * @return array
     */
    protected function _getOrderTaxes($orderTaxes)
    {
        $result = array();

        if (isset($orderTaxes['ot_tax']) && is_array($orderTaxes['ot_tax'])) {
            foreach ($orderTaxes['ot_tax'] as $orderTax) {
                $orderTaxItem = new ShopgateExternalOrderTax();

                $orderTaxItem->setLabel($orderTax['title']);
                $orderTaxItem->setAmount(round($orderTax['value'], 2));
                array_push($result, $orderTaxItem);
            }
        }

        return $result;
    }

    /**
     * read order data from the database
     *
     * @param        $customerLanguageId
     * @param int    $limit
     * @param int    $offset
     * @param string $orderDateFrom
     * @param string $sortOrder
     *
     * @return bool|resource
     */
    protected function _getOrdersResource(
        $customerLanguageId,
        $limit = 10,
        $offset = 0,
        $orderDateFrom = '',
        $sortOrder = 'created_desc'
    ) {
        switch ($sortOrder) {
            case 'created_desc' :
                $sortBy = 'date_purchased DESC';
                break;
            case 'created_asc' :
                $sortBy = 'date_purchased ASC';
                break;
            default :
                $sortBy = 'orders_id DESC';
        }

        $dateSelector = $orderDateFrom != '' ? " AND o.date_purchased >= '" . $orderDateFrom . "'" : '';

        $query
            = "SELECT
                        o.*,
                        s.*
                    FROM
                        " . TABLE_ORDERS . " o,
                        " . TABLE_ORDERS_TOTAL . " ot,
                        " . TABLE_ORDERS_STATUS . " s,
                        " . TABLE_CUSTOMERS_INFO . " ci
                    WHERE
                        o.customers_id = '" . $this->_customerId . "' AND
                        o.orders_id = ot.orders_id AND
                        ot.class = 'ot_total' AND
                        o.orders_status = s.orders_status_id AND
                        s.language_id = '" . $customerLanguageId . "' AND
                        o.customers_id = ci.customers_info_id AND
                        o.date_purchased >= ci.customers_info_date_account_created
                        " . $dateSelector . "
                    ORDER BY " . $sortBy . "
                    LIMIT " . $limit . " OFFSET " . $offset . "
                    ";

        return xtc_db_query($query);
    }

    /**
     * read order total data from the database by the orders uid
     *
     * @param $orderId
     *
     * @return bool|resource
     */
    protected function _getOrderTotalsResource($orderId)
    {
        /** @var AccountContentView $coo_account_history_view */
        $query
            = "SELECT *
                FROM " . TABLE_ORDERS_TOTAL . "
                WHERE orders_id = " . $orderId . "
                ORDER BY orders_id DESC";

        return xtc_db_query($query);
    }

    /**
     * read the shopgate order number from the database
     * by the shop systems order id
     *
     * @param $orderId
     *
     * @return null
     */
    protected function _getShopgateOrderId($orderId)
    {
        $qry = xtc_db_query("SELECT * FROM `" . TABLE_ORDERS_SHOPGATE_ORDER . "`");

        while ($row = xtc_db_fetch_array($qry)) {
            if ($row['orders_id'] == $orderId) {
                return $row['shopgate_order_number'];
            }
        }

        return null;
    }

    /**
     * fill a ShopgateOrder object with shipping data
     *
     * @param ShopgateOrder | ShopgateExternalOrder $shopgateOrder
     * @param string        $langIsoCode
     *
     * @return mixed
     * @throws ShopgateLibraryException
     */
    protected function _setShippingInfo($shopgateOrder, $langIsoCode)
    {
        $isoCodeParts = explode('_', $langIsoCode);
        $langId       = isset($isoCodeParts[0]) ? $isoCodeParts[0] : $langIsoCode;

        $config = new ShopgateConfigXtc();
        $config->loadByLanguage($langId);
        $shippedStatusId = $config->getOrderStatusShipped();

        $query = "SELECT * FROM " . TABLE_ORDERS_STATUS_HISTORY . "
                  WHERE orders_id = " . $shopgateOrder->getExternalOrderNumber() . "
                  AND orders_status_id = " . $shippedStatusId;
        $res   = xtc_db_query($query);
        while ($item = xtc_db_fetch_array($res)) {
            $shopgateOrder->setIsShippingCompleted(true);
            $shopgateOrder->setShippingCompletedTime($item['date_added']);
            break;
        }

        return $shopgateOrder;
    }
}
