<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopgate_Models_Item extends Shopgate_Model_Catalog_Product
{

    protected $config;
    protected $languageId;
    protected $exchangeRate;

    /** @var array $shopCurrency */
    protected $shopCurrency;

    /** @var Shopgate_Helper_Pricing */
    protected $priceHelper;

    /**
     * @param ShopgateConfigXtc $config
     * @param int               $languageId
     * @param int               $exchangeRate
     */
    public function __construct($config, $languageId, $exchangeRate = 1)
    {
        $this->languageId   = $languageId;
        $this->config       = $config;
        $this->exchangeRate = $exchangeRate;
    }

    /**
     * @param Shopgate_Helper_Pricing $priceHelper
     */
    public function setPriceHelper(Shopgate_Helper_Pricing $priceHelper)
    {
        $this->priceHelper = $priceHelper;
    }

    /**
     * set currency data for export
     *
     * @param string $currency
     */
    public function setShopCurrency($currency)
    {
        $this->shopCurrency = $currency;
    }

    public function getProductsQuantity()
    {

    }

    /**
     * read the product data from the database by the products uid
     *
     * @param $productId
     *
     * @return mixed
     */
    public function getProductById($productId)
    {
        $productQuery    = "select p.products_tax_class_id,
                              p.products_id,
                              pd.products_name,
                              p.products_price,
                              sp.specials_quantity,
                              sp.specials_new_products_price,
                              sp.expires_date 
                            from products p
                                LEFT JOIN products_description pd ON p.products_id = pd.products_id 
                                          AND pd.language_id = {$this->languageId}
                                LEFT JOIN specials sp ON  (sp.products_id = p.products_id 
                                AND sp.status = 1 
                                AND (sp.expires_date > now() 
                                OR sp.expires_date = '0000-00-00 00:00:00' OR sp.expires_date IS NULL)) 
                            where p.products_id = {$productId}";
        $dbProductResult = xtc_db_query($productQuery);
        $dbProduct       = xtc_db_fetch_array($dbProductResult);

        return $dbProduct;
    }

    /**
     * read the whole product data from the database
     *
     * @param       $categoryBlacklist
     * @param null  $offset
     * @param null  $limit
     * @param array $uids
     *
     * @return mixed
     */
    public function getProductsFromDb(
        $categoryBlacklist, $offset = null, $limit = null, array $uids = array()
    ) {
        $qry = "
            SELECT DISTINCT
                p.products_id,
                p.products_model,
                p.products_ean,
                p.products_quantity,
                p.products_image,
                p.products_price,
                DATE_FORMAT(p.products_last_modified, '%Y-%m-%d') as products_last_modified,
                p.products_weight,
                p.products_status,
                sp.specials_new_products_price,
                sp.specials_quantity,
                pdsc.products_keywords,
                pdsc.products_name,
                pdsc.products_description,
                pdsc.products_short_description,
                shst.shipping_status_name,
                mf.manufacturers_name,
                p.products_tax_class_id,
                p.products_fsk18,
                p.products_vpe_status,
                p.products_vpe_value,
                vpe.products_vpe_name,
                p.products_sort,
                p.products_startpage,
                p.products_startpage_sort,
                p.products_discount_allowed,
                p.products_date_available
            FROM " . TABLE_PRODUCTS . " p
            LEFT JOIN " . TABLE_PRODUCTS_DESCRIPTION
            . " pdsc ON (p.products_id = pdsc.products_id AND pdsc.language_id = '"
            . $this->languageId . "')
            LEFT JOIN " . TABLE_SHIPPING_STATUS
            . " shst ON (p.products_shippingtime = shst.shipping_status_id AND shst.language_id = '"
            . $this->languageId . "')
            LEFT JOIN " . TABLE_MANUFACTURERS . " mf ON (mf.manufacturers_id = p.manufacturers_id)
            LEFT JOIN " . TABLE_SPECIALS . " sp ON (sp.products_id = p.products_id AND sp.status = 1 AND (sp.expires_date > now() OR sp.expires_date = '0000-00-00 00:00:00' OR sp.expires_date IS NULL))
            LEFT JOIN " . TABLE_PRODUCTS_VPE . " vpe ON (vpe.products_vpe_id = p.products_vpe AND vpe.language_id = pdsc.language_id)
             ";

        if (count($categoryBlacklist) > 0) {
            $qry .= "RIGHT JOIN products_to_categories as ptoc ON (ptoc.products_id = p.products_id) WHERE ";
            $i   = 1;
            foreach ($categoryBlacklist as $categoryId) {
                $qry .= (($i == 1) ? "" : " AND ") . " (ptoc.categories_id != "
                    . $categoryId . ") ";
                $i++;
            }
            $qry .= " AND p.products_status = 1";
        } else {
            $qry .= " WHERE p.products_status = 1";
        }

        foreach ($this->config->getExcludeItemIds() as $itemId) {
            $qry .= " AND p.products_id != " . $itemId . " ";
        }

        // sp.specials_quantity > 0 deleted, to handle the check in the loop

        if (STOCK_CHECK == "true" && STOCK_ALLOW_CHECKOUT == 'false') {
            $qry .= " AND p.products_quantity > 0 ";
        }

        // Code for enabling to download specific products (for debugging purposes only, at this time)
        if (!empty($uids)
            && is_array($uids)
        ) {
            $qry .= " AND p.products_id IN ('" . implode(
                    "', '", $uids
                ) . "') ";
        }

        // Ahorn24 fix. 10 products were not found without sorting.
        $qry .= ' ORDER BY p.products_id ASC ';

        if (!is_null($offset) && !is_null($limit)) {
            $qry .= " LIMIT {$offset}, {$limit}";
        }

        return xtc_db_query($qry);
    }

    /**
     * read the highest products_sort value from database
     *
     * @return int
     */
    public function getProductsDbMaxSortValue()
    {
        $result  = xtc_db_query("SELECT MAX(products_sort) max_sort FROM " . TABLE_PRODUCTS);
        $maxSort = xtc_db_fetch_array($result);

        return $maxSort["max_sort"];
    }

    /**
     * read the minimum and maximum sort order value to products from the database
     *
     * @param $maxOrder
     * @param $addToOrderIndex
     */
    public function getProductsSortOrder(&$maxOrder, &$addToOrderIndex)
    {
        // order_index for the products
        $result          = xtc_db_query(
            "SELECT MIN(products_sort) AS 'min_order', MAX(products_sort) AS 'max_order' FROM "
            . TABLE_PRODUCTS
        );
        $orderIndices    = xtc_db_fetch_array($result);
        $maxOrder        = $orderIndices["max_order"] + 1;
        $minOrder        = $orderIndices["min_order"];
        $addToOrderIndex = 0;

        if ($minOrder < 0) {
            // make the sort_order positive
            $addToOrderIndex += abs($minOrder);
        }
    }

    /**
     * get all products to the virtual category "new products"
     *
     * @return array
     */
    public function getNewProducts()
    {

        $group_check              = '';
        $fsk_lock                 = '';
        $_SESSION['languages_id'] = $this->languageId;
        //logic taken from file products_new.php in dir /
        $date_new_products = date(
            "Y.m.d", mktime(
                1, 1, 1, date("m"), date("d") - MAX_DISPLAY_NEW_PRODUCTS_DAYS,
                date("Y")
            )
        );
        $days              =
            " and p.products_date_added > '" . $date_new_products . "' ";

        $productsNewQuery = "select distinct
                                    p.products_id,
                                    p.products_fsk18,
                                    pd.products_name,
                                    pd.products_short_description,
                                    p.products_image,
                                    p.products_price,
                                    p.products_vpe,
                                    p.products_vpe_status,
                                    p.products_vpe_value,
                                    p.products_tax_class_id,
                                    p.products_shippingtime,
                                    p.products_date_added,
                                    m.manufacturers_name
                           from " . TABLE_PRODUCTS . " p
                           left join " . TABLE_MANUFACTURERS . " m on p.manufacturers_id = m.manufacturers_id
                           left join " . TABLE_PRODUCTS_DESCRIPTION . " pd on p.products_id = pd.products_id,
                                     " . TABLE_CATEGORIES . " c,
                                     " . TABLE_PRODUCTS_TO_CATEGORIES . " p2c 
                           where pd.language_id = '" . (int)$_SESSION['languages_id'] . "'
                           and c.categories_status = 1
                           and p.products_id = p2c.products_id
                           and c.categories_id = p2c.categories_id
                           and products_status = '1'
                                " . $group_check . "
                                " . $fsk_lock . "                                    
                                " . $days . "
                           order by p.products_date_added DESC ";

        $result            = array();
        $newProductsResult = xtc_db_query($productsNewQuery);

        while ($products_new = xtc_db_fetch_array($newProductsResult)) {
            $result[] = $products_new;
        }

        return $result;
    }

    /**
     * generates the description to a product depending on the shop system settings
     *
     * @return mixed
     */
    public function getProductDescription()
    {
        // create the description, based on the settings
        $desc        = $this->item["products_description"];
        $shortDesc   = $this->item["products_short_description"];
        $description = '';
        switch ($this->config->getExportDescriptionType()) {
            case SHOPGATE_SETTING_EXPORT_DESCRIPTION:
                $description = $desc;
                break;
            case SHOPGATE_SETTING_EXPORT_SHORTDESCRIPTION:
                $description = $shortDesc;
                break;
            case SHOPGATE_SETTING_EXPORT_DESCRIPTION_SHORTDESCRIPTION:
                $description = $desc . "<br/><br/>" . $shortDesc;
                break;
            case SHOPGATE_SETTING_EXPORT_SHORTDESCRIPTION_DESCRIPTION:
                $description = $shortDesc . "<br/><br/>" . $desc;
                break;
        }

        return preg_replace("/\n|\r/", "", $description);
    }

    /**
     * fetches and returns deeplink for given item
     *
     * @return string
     */
    public function generateDeeplink()
    {
        // get deeplink
        $deeplink = xtc_href_link(
            'product_info.php',
            xtc_product_link($this->item['products_id'], $this->item['products_name']),
            'NONSSL', false
        );

        return $deeplink;
    }

    /**
     * return cleared item name
     *
     * @param array $this ->item
     *
     * @return string
     */
    public function generateItemName()
    {
        return trim(preg_replace('/<[^>]+>/', ' ', $this->item['products_name']));
    }

    /**
     * return cleared ean
     *
     * @param array $this ->item
     *
     * @return string
     */
    public function generateEan()
    {
        return preg_replace("/\s+/i", '', $this->item['products_ean']);
    }

    /**
     * returns weight
     *
     * @param array $this ->item
     *
     * @return mixed
     */
    public function generateWeight()
    {
        return $this->item['products_weight'] * 1000;
    }

    /**
     * Generates an available text based on the date available field
     *
     * @param string $defaultStatusName
     *
     * @return string
     */
    public function generateAvailableText($defaultStatusName = '')
    {
        if (empty($this->item)
            || empty($this->item['shipping_status_name']) && empty($defaultStatusName)
        ) {
            return '';
        }

        if (!empty($defaultStatusName)) {
            $availableText = (string)$defaultStatusName;
        } else {
            $availableText = (string)$this->item['shipping_status_name'];
        }

        // Check if the product is available in the future
        if (!empty($this->item['products_date_available'])) {
            // Check if the date is in the future
            $availableOnTimestamp = strtotime(
                substr($this->item['products_date_available'], 0, 10) . ' 00:00:00'
            ); // Take the date beginning at 00:00:00 o' clock
            // Set the "available on" text only if it is at least one day in the future
            if ($availableOnTimestamp - time() > 60 * 60 * 24
            ) { // 60sec * 60min * 24h == count seconds in 1 day
                switch (strtolower($this->config->getLanguage())) {
                    case 'de':
                        $dateAvailableFormatted =
                            date('d.m.Y', $availableOnTimestamp);
                        break;
                    case 'en':
                    default:
                        $dateAvailableFormatted =
                            date('m/d/Y', $availableOnTimestamp);
                        break;
                }
                $availableText = str_replace(
                    '#DATE#', $dateAvailableFormatted,
                    SHOPGATE_PLUGIN_FIELD_AVAILABLE_TEXT_AVAILABLE_ON_DATE
                );
            }
        }

        $availableText = $availableText == 'Unbekannt'
            ? 0
            : $availableText;

        return $availableText;
    }

    /**
     * export all product images
     *
     * @return array
     */
    public function generateProductsImages()
    {
        $this->log(
            "execute generateProductsImages() ...", ShopgateLogger::LOGTYPE_DEBUG
        );

        $qry = "
            SELECT *
            FROM " . TABLE_PRODUCTS_IMAGES . "
            WHERE products_id = '" . $this->item["products_id"] . "'
            ORDER BY image_nr
        ";

        $images = array();

        if (!empty($this->item['products_image'])) {
            if (file_exists(
                DIR_FS_CATALOG . DIR_WS_ORIGINAL_IMAGES
                . $this->item['products_image']
            )) {
                $images[] =
                    HTTP_SERVER . DIR_WS_CATALOG . DIR_WS_ORIGINAL_IMAGES
                    . $this->item['products_image'];
            } elseif (file_exists(
                DIR_FS_CATALOG . DIR_WS_POPUP_IMAGES
                . $this->item['products_image']
            )) {
                $images[] = HTTP_SERVER . DIR_WS_CATALOG . DIR_WS_POPUP_IMAGES
                    . $this->item['products_image'];
            }
        }

        $query = xtc_db_query($qry);
        while ($image = xtc_db_fetch_array($query)) {
            if (file_exists(
                DIR_FS_CATALOG . DIR_WS_ORIGINAL_IMAGES . $image['image_name']
            )) {
                $images[] =
                    HTTP_SERVER . DIR_WS_CATALOG . DIR_WS_ORIGINAL_IMAGES
                    . $image['image_name'];
            } elseif (file_exists(
                DIR_FS_CATALOG . DIR_WS_POPUP_IMAGES . $image['image_name']
            )) {
                $images[] = HTTP_SERVER . DIR_WS_CATALOG . DIR_WS_POPUP_IMAGES
                    . $image['image_name'];
            }
        }

        return $images;
    }

    /**
     * fetches all categories and return array with id and sorting index
     *
     * @return array
     */
    public function fetchAllCategories()
    {
        $maxOrder        = 0;
        $addToOrderIndex = 0;

        $specialProducts = $this->config->getExportNewProductsCategory()
            ? $this->getNewProducts()
            : array();

        $maxSort = $this->getProductsDbMaxSortValue();
        $this->getProductsSortOrder($maxOrder, $addToOrderIndex);

        $categoryModel = new Shopgate_Models_Catalog_Category($this->config);
        $categories    = $categoryModel->getProductCategoryNumbers($this->item, $maxSort, $addToOrderIndex);

        if ($this->config->getExportNewProductsCategory()) {
            foreach ($specialProducts as $specialProduct) {
                if ($specialProduct['products_id'] == $this->item['products_id']) {
                    $category_numbers[] = array(
                        'id'   => $this->config->getExportNewProductsCategoryId(),
                        'sort' => 0
                    );
                }
            }
        }

        return $categories;
    }

    /**
     * check if the shop settings regard or ignore the products stock
     *
     * @return int
     */
    public function generateUseStock()
    {
        return (STOCK_ALLOW_CHECKOUT == 'true' || STOCK_CHECK != 'true')
            ? 0
            : 1;
    }

    /**
     * read block price data from the database
     *
     * @return string
     */
    public function getBlockPrices()
    {
        $initHelper = new Shopgate_Helpers_PluginInit();
        $countryId  = $initHelper->getCountryByIsoCode2($this->config->getCountry());
        $zoneId     = $this->config->getTaxZoneId();
        $taxRate    = xtc_get_tax_rate(
            $this->item["products_tax_class_id"], $countryId, $zoneId
        );

        $customerStatusId = $this->config->getCustomerPriceGroup();
        if ($customerStatusId > 0) {
            return '';
        }

        $qry = "
            SELECT *
            FROM " . TABLE_PERSONAL_OFFERS_BY . "$customerStatusId
            WHERE products_id = '" . $this->item["products_id"] . "'
              AND quantity > 1
            ORDER BY quantity
        ";

        $specialOffers  = array();
        $_specialOffers = xtc_db_query($qry);

        while ($specialOffer = xtc_db_fetch_array($_specialOffers)) {
            $specialOffers[] = implode(
                "=>", array(
                    "qty"            => $specialOffer["quantity"],
                    "personal_offer" => round(
                        $specialOffer["personal_offer"] * (1
                            + ($taxRate / 100)), 2
                    ),
                )
            );
        }

        return implode("||", $specialOffers);
    }

    /**
     * generate the age rating to an product
     *
     * @return string
     */
    public function generateAgeRating()
    {
        return $this->item['products_fsk18'] == 1
            ? '18'
            : '';
    }

    /**
     * read data about relations between products from the database
     *
     * @param int $productsId
     *
     * @return string
     */
    public function fetchRelatedShopItems($productsId)
    {
        $this->log(
            "execute _getRelatedShopItems() ...", ShopgateLogger::LOGTYPE_DEBUG
        );
        $qry = "
            SELECT px.xsell_id
            FROM " . TABLE_PRODUCTS_XSELL . " px
            INNER JOIN " . TABLE_PRODUCTS . " p ON (px.products_id = p.products_id)
            WHERE p.products_id = '$productsId'
                AND (p.products_date_available < NOW() OR p.products_date_available IS NULL)
            ORDER BY px.sort_order
        ";

        $xSellIds = array();

        $query = xtc_db_query($qry);
        for ($i = 0; $i < xtc_db_num_rows($query); $i++) {
            $array      = xtc_db_fetch_array($query);
            $xSellIds[] = $array["xsell_id"];
        }

        return $xSellIds;
    }

    /**
     * calculates the base price to an xt_price
     *
     * @param $price
     *
     * @return string
     */
    public function generateBasePrice($price)
    {
        $vpe = "";

        if (!empty($this->item["products_vpe_value"])
            && !empty($this->item["products_vpe_name"])
            && $this->item["products_vpe_value"] != 0.0000
        ) {

            if ($this->item["products_vpe_status"] == 1) {

                $factor = 1;
                switch (strtolower($this->item["products_vpe_name"])) {
                    case "ml":
                    case "mg":
                        $factor =
                            $this->item["products_vpe_value"] < 250 ? 100 : 1000;
                        break;
                }

                $_price = ($price / $this->item["products_vpe_value"]) * $factor;

                $vpe = $this->shopCurrency["symbol_left"];

                $vpe .= $this->priceHelper->formatPriceNumber(
                    $_price, $this->shopCurrency["decimal_places"],
                    $this->shopCurrency["decimal_point"],
                    $this->shopCurrency["thousands_point"]
                );

                $vpe .= " " . trim($this->shopCurrency["symbol_right"]);
                $vpe .= ' pro ' . (($factor == 1) ? '' : $factor . ' ');
                $vpe .= $this->item["products_vpe_name"];
            }
        }

        return $vpe;
    }

    /**
     * calculate stock for given product
     *
     * @return mixed
     */
    public function generateStockQuantity()
    {
        $qty = $this->item['products_quantity'];
        if (!empty($this->item['specials_new_products_price'])
            && (STOCK_CHECK == 'true' && STOCK_ALLOW_CHECKOUT == 'false')
            && ($this->item['specials_quantity'] > 0)
        ) {
            $qty = $this->item['specials_quantity'] > $this->item['products_quantity']
                ? $this->item['products_quantity']
                : $this->item['specials_quantity'];
        }

        return $qty;
    }

    /**
     * generate internal order infos for product
     *
     * @return array
     */
    public function generateInternalOrderInfo()
    {
        $orderInfo = array(
            'base_item_number' => $this->item['products_id']
        );
        if (!empty($this->item['specials_new_products_price'])
            && (STOCK_CHECK == 'true' && STOCK_ALLOW_CHECKOUT == 'false')
            && ($this->item['specials_quantity'] > 0)
        ) {
            $orderInfos['is_special_price'] = 1;
        }

        return $orderInfo;
    }

    /**
     * read input field data of a product from the database
     *
     * @param $productId
     *
     * @return array
     */
    public function getInputFields($productId)
    {
        $this->log(
            "execute _getInputFields() ...", ShopgateLogger::LOGTYPE_DEBUG
        );
        $optionsAsTextfieldIds = $this->config->getExportOptionAsInputField();

        $qry = "
            SELECT
                pa.products_attributes_id,
                po.products_options_id,
                pov.products_options_values_id,
                po.products_options_name,
                pov.products_options_values_name,
                pa.attributes_model,
                pa.options_values_price,
                pa.price_prefix,
                pa.options_values_weight,
                pa.attributes_stock,
                pa.weight_prefix
            FROM " . TABLE_PRODUCTS_ATTRIBUTES . " pa
            INNER JOIN " . TABLE_PRODUCTS_OPTIONS . " po ON pa.options_id = po.products_options_id
            INNER JOIN " . TABLE_PRODUCTS_OPTIONS_VALUES
            . " pov ON (pa.options_values_id = pov.products_options_values_id AND pov.language_id = $this->languageId)
            WHERE pa.products_id = '$productId'
                AND (pov.products_options_values_name = 'TEXTFELD' "
            . ((!empty($optionsAsTextfieldIds)) ? " OR pa.options_id IN (" . mysql_real_escape_string(
                    $optionsAsTextfieldIds
                ) . "))" : ")")
            . "ORDER BY pa.sortorder ASC";

        $query          = xtc_db_query($qry);
        $inputFieldsAll = array();

        while ($inputFields = xtc_db_fetch_array($query)) {
            $inputFieldsAll[$inputFields['products_options_id']][] = $inputFields;
        }

        return $inputFieldsAll;
    }

    /**
     * generate properties to an product
     *
     * @return array
     */
    public function generateProperties()
    {
        $properties = array();

        if (!empty($this->item["products_fsk18"])
            && $this->item["products_fsk18"] == 1
        ) {
            $properties[] = array(
                'label' => 'Altersbeschränkung',
                'value' => '18 Jahre'
            );
        }

        return $properties;
    }
}
