<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopgate_Models_Item_CartItem extends ShopgateObject
{
    /**
     * @var int
     */
    private $languageId;

    /**
     * @param int $languageId
     */
    public function __construct($languageId)
    {
        $this->languageId = $languageId;
    }

    /**
     * Calculate the weight to an product regarding the weight of options
     *
     * @param ShopgateOrderItem[] $products
     * @return float
     * @throws ShopgateLibraryException
     */
    public function getProductsWeight($products)
    {
        $weight     = 0;
        $options    = array();
        $attributes = array();
        foreach ($products as $product) {
            $productId = $this->getProductIdFromCartItem($product);
            foreach ($product->getOptions() as $option) {
                $options[$option->getOptionNumber()] = $option->getValueNumber();
            }

            $internalOrderInfo = json_decode($product->getInternalOrderInfo(), true);
            for ($i = 1; $i <= 10; ++$i) {
                if (empty($internalOrderInfo['attribute_' . $i])) {
                    break;
                }
                $attribute                            = current($internalOrderInfo['attribute_' . $i]);
                $attributes[$attribute['options_id']] = $attribute['options_values_id'];
            }

            if (count($options) != 0 || count($attributes) != 0) {
                // calculate the additional attribute/option weight
                $dbHelper      = new Shopgate_Helpers_Database();
                $prefixColName = $dbHelper->getWeightPrefixColumnName();
                $weightColName = $dbHelper->getWeightColumnName();
                if (!$prefixColName || !$weightColName) {
                    $this->log("Wrong column names, prefix: {$prefixColName} OR weight: {$weightColName}");
                    throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_DATABASE_ERROR);
                }
                $query = "SELECT SUM(CONCAT({$prefixColName}, {$weightColName})) AS weight FROM "
                         . TABLE_PRODUCTS_ATTRIBUTES . " AS pa WHERE ";

                $conditions = array();
                $variants   = $options + $attributes;
                foreach ($variants as $productOptionId => $productOptionValueId) {
                    $conditions[] =
                        " (pa.products_id = {$productId} " .
                        "AND pa.options_id = {$productOptionId} " .
                        "AND pa.options_values_id = {$productOptionValueId}) ";
                }

                $query .= implode(' OR ', $conditions);
                $result = xtc_db_fetch_array(xtc_db_query($query));
                $weight += $result['weight'] * $product->getQuantity();
            }

            if (!empty($productId)) {
                // calculate the "base" product weight
                /** @noinspection SqlResolve */
                $result = xtc_db_fetch_array(
                    xtc_db_query(
                        "SELECT products_weight FROM " . TABLE_PRODUCTS . " AS p WHERE p.products_id = {$productId}"
                    )
                );

                $weight += $result['products_weight'] * $product->getQuantity();
            }
        }

        return $weight;
    }

    /**
     * If the current order item (product) is an child product the item number is
     * generated in the schema <productId>_<attributeId>
     *
     * this function returns the id, the product has in the shop system
     *
     * @param ShopgateOrderItem $product
     *
     * @return string
     */
    public function getProductIdFromCartItem(ShopgateOrderItem $product)
    {
        $parentId = $product->getParentItemNumber();
        if (empty($parentId)) {
            $productId = $product->getItemNumber();
            if (strpos($productId, '_') !== false) {
                $productIdArr = explode('_', $productId);

                return $productIdArr[0];
            }

            return $productId;
        }

        return $parentId;
    }

    /**
     * Read the whole attribute data to an product from the database
     *
     * @param ShopgateOrderItem $item
     * @param ShopgateCartItem  $sgProduct
     *
     * @return mixed
     */
    public function getAttributesToProduct($item, $sgProduct)
    {
        $attributes   = $sgProduct->getAttributes();
        $dbAttributes = array();
        foreach ($attributes as $attribute) {
            $query  = "SELECT
                    po.products_options_name AS `name`,
                    pov.products_options_values_name AS `value`
                    FROM products AS p
                    LEFT JOIN products_attributes AS pa ON p.products_id = pa.products_id
                    LEFT JOIN products_options AS po ON (pa.options_id = po.products_options_id AND po.language_id = 1)
                    LEFT JOIN products_options_values AS pov ON
                      (pa.options_values_id = pov.products_options_values_id AND po.language_id = pov.language_id)
                    WHERE pa.products_id = {$this->getProductIdFromCartItem($item)}
                      AND po.products_options_name = '{$attribute->getName()}'
                      AND pov.products_options_values_name = '{$attribute->getValue()}'";
            $result = xtc_db_fetch_array($query);

            while ($dbProductAttributes =
                xtc_db_fetch_array($result)) {
                $sgAttribute = new ShopgateOrderItemAttribute();
                $sgAttribute->setName($dbProductAttributes['option_name']);
                $sgAttribute->setValue($dbProductAttributes['value']);
                $dbAttributes[] = $sgAttribute;
            }
        }
        $sgProduct->setAttributes($dbAttributes);

        return $sgProduct;
    }

    /**
     * Read all option data to an product from database and
     * create an ShopgateOrderItemOption object with the data
     *
     * @param string | int $productId
     * @param array        $attributeIds
     * @param float        $taxRate
     *
     * @return array
     */
    public function getOptionsToProduct($productId, $attributeIds, $taxRate)
    {
        $resultAttributes = array();
        foreach ($attributeIds as $attributeId) {
            $query = "SELECT
                        o.products_options_id AS `options_id`,
                        ov.products_options_values_id AS `values_id`,
                        ov.products_options_values_name AS `values_name`,
                        pa.price_prefix AS `prefix`,
                        pa.options_values_price AS `price`,
                        o.products_options_name AS `name`
                    FROM products AS p
                        LEFT JOIN products_attributes AS pa ON p.products_id = pa.products_id
                        LEFT JOIN products_options AS o  ON o.products_options_id = pa.options_id
                          AND o.language_id = {$this->languageId}
                        LEFT JOIN products_options_values AS ov ON (ov.products_options_values_id = pa.options_values_id
                          AND o.language_id = ov.language_id)
                    WHERE p.products_id = {$productId} AND
                          pa.products_attributes_id = {$attributeId["products_attributes_id"]} AND
                          o.products_options_id = {$attributeId["options_id"]} AND
                          ov.products_options_values_id = {$attributeId["options_values_id"]}";

            $result       = xtc_db_query($query);
            $optionResult = xtc_db_fetch_array($result);
            $sgOption     = new ShopgateOrderItemOption();
            $sgOption->setName($this->stringToUtf8($optionResult['name']));
            $sgOption->setOptionNumber($optionResult['options_id']);
            $sgOption->setValue($this->stringToUtf8($optionResult['values_name']));
            $sgOption->setValueNumber($optionResult['values_id']);

            if (!empty($optionResult['prefix'])) {
                $price =
                    ($optionResult['prefix'] == '-') ? ($optionResult['price'] * (-1)) : $optionResult['price'];
            } else {
                $price = $optionResult['price'];
            }

            $sgOption->setAdditionalAmountWithTax($price * (1 + ($taxRate / 100)));
            $resultAttributes[] = $sgOption;
        }

        return $resultAttributes;
    }

    /**
     * Calculate the complete amount of all items a cart object has
     *
     * @param ShopgateCart $cart
     *
     * @return float|int
     */
    public function getCompleteAmount(ShopgateCart $cart)
    {
        $completeAmount = 0;
        foreach ($cart->getItems() as $item) {
            $itemAmount = ($item->getTaxPercent() > 0)
                ? $item->getUnitAmount() * (1 + ($item->getTaxPercent() / 100))
                : $item->getUnitAmountWithTax();
            $completeAmount += $itemAmount * $item->getQuantity();
        }

        return $completeAmount;
    }
}
