<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopgate_Models_Item_Csv extends Shopgate_Models_Item
{
    /**
     * @var array|null
     */
    protected $defaultRow = null;

    /**
     * @var array|null
     */
    protected $actionCache = null;

    /**
     * @param $defaultRow
     *
     * @return Shopgate_Models_Item_Csv
     */
    public function setDefaultRow($defaultRow)
    {
        $this->defaultRow = $defaultRow;

        return $this;
    }

    /**
     * Dynamically call functions to set product data
     *
     * @return array
     */
    public function generateData()
    {
        foreach (array_keys($this->defaultRow) as $key) {
            /* clear values */
            $this->defaultRow[$key] = null;

            $action = 'setItem' . str_replace(" ", "", ucwords(str_replace('_', " ", $key)));
            if (empty($this->actionCache[$action])) {
                $this->actionCache[$action] = true;
            }
        }

        foreach (array_keys($this->actionCache) as $_action) {
            if (method_exists($this, $_action)) {
                $this->{$_action}();
            }
        }

        // generate complex data
        $this->generatePriceRelatedData();
        $this->generateInputs();

        return $this->defaultRow;
    }

    /**
     * Get item number / id
     */
    protected function setItemItemNumber()
    {
        $this->defaultRow['item_number'] = $this->item['products_id'];
    }

    /**
     * Get item number public
     */
    protected function setItemItemNumberPublic()
    {
        $this->defaultRow['item_number_public'] = $this->item['products_model'];
    }

    /**
     * Get item deep link
     */
    protected function setItemUrlDeeplink()
    {
        $this->defaultRow['url_deeplink'] = $this->generateDeeplink();
    }

    /**
     * Set manufacturer
     */
    protected function setItemManufacturer()
    {
        $this->defaultRow['manufacturer'] = $this->item['manufacturers_name'];
    }

    /**
     * Set description
     */
    protected function setItemDescription()
    {
        $stringHelper                    = $this->getHelper(ShopgateObject::HELPER_STRING);
        $this->defaultRow['description'] = $stringHelper->removeTagsFromString($this->getProductDescription());
    }

    /**
     * Set active status
     */
    protected function setItemActiveStatus()
    {
        $this->defaultRow['active_status'] = ShopgatePlugin::PRODUCT_STATUS_STOCK;
    }

    /**
     * Set if item is available
     */
    protected function setItemIsAvailable()
    {
        $this->defaultRow['is_available'] = $this->item['products_status'];
    }

    /**
     * Set available text
     */
    protected function setItemAvailableText()
    {
        $this->defaultRow['available_text'] = $this->generateAvailableText();
    }

    /**
     * Set currency code
     */
    protected function setItemCurrency()
    {
        $this->defaultRow['currency'] = $this->shopCurrency['code'];
    }

    /**
     * Set item images
     */
    protected function setItemUrlsImages()
    {
        $images                          = $this->generateProductsImages();
        $this->defaultRow['urls_images'] = implode("||", $images);;
    }


    /**
     * Set item name
     */
    protected function setItemItemName()
    {
        $this->defaultRow['item_name'] = $this->generateItemName();
    }

    /**
     * Set last update time
     */
    protected function setItemLastUpdate()
    {
        $this->defaultRow['last_update'] = $this->item['products_last_modified'];
    }

    /**
     * Set ean attribute if predefined
     */
    protected function setItemEan()
    {
        $this->defaultRow['ean'] = $this->generateEan();
    }

    /**
     * Set product weight
     */
    protected function setItemWeight()
    {
        $this->defaultRow['weight'] = $this->generateWeight();
    }

    /**
     * Set product tags
     */
    protected function setItemTags()
    {
        $this->defaultRow['tags'] = trim($this->item['products_keywords']);
    }

    /**
     * Set product categories
     */
    protected function setItemCategoryNumbers()
    {
        $categories    = $this->fetchAllCategories();
        $categoryArray = array();
        foreach ($categories as $category) {
            $categoryArray[] = $category['id'] . '=>' . $category['sort'];
        }

        $this->defaultRow['category_numbers'] = implode('||', $categoryArray);
    }

    /**
     * Set use stock flag
     */
    protected function setItemUseStock()
    {
        $this->defaultRow['use_stock'] = $this->generateUseStock();
    }

    /**
     * Set stock quantity
     */
    protected function setItemStockQuantity()
    {
        $this->defaultRow['stock_quantity'] = $this->generateStockQuantity();
    }

    /**
     * Set tax percent
     */
    protected function setItemTaxPercent()
    {
        $this->defaultRow['tax_percent'] = $this->item['products_tax_class_id'];
    }

    /**
     * Set block prices
     */
    protected function setItemBlockPricing()
    {
        $this->defaultRow['block_pricing'] = $this->getBlockPrices();
    }

    /**
     * Set the age rating for the products
     */
    protected function setItemAgeRating()
    {
        $this->defaultRow['age_rating'] = $this->generateAgeRating();
    }

    /**
     * Set the related items
     */
    protected function setItemRelatedShopItemNumbers()
    {
        $relatedShopItems                              = $this->fetchRelatedShopItems($this->item['products_id']);
        $this->defaultRow['related_shop_item_numbers'] = implode('||', $relatedShopItems);
    }

    /**
     * Set if product should be highlight
     */
    protected function setItemIsHighlight()
    {
        $this->defaultRow['is_highlight'] = $this->item['products_startpage'];
    }

    /**
     * Set the order index for highlighted products
     */
    protected function setItemHighlightOrderIndex()
    {
        $this->defaultRow['highlight_order_index'] = $this->item['products_startpage_sort'];
    }

    /**
     * Set the properties
     */
    protected function setItemProperties()
    {
        $properties = $this->generateProperties();
        $result     = array();

        foreach ($properties as $data) {
            $result[] = $data['label'] . '=>' . $data['value'];
        }

        $this->defaultRow['properties'] = implode('||', $result);
    }

    /**
     * Set the intern order info
     */
    protected function setItemInternalOrderInfo()
    {
        $orderInfos = $this->generateInternalOrderInfo();
        if (!empty($orderInfos)) {
            $this->defaultRow['internal_order_info'] = $this->jsonEncode($orderInfos);
        }
    }

    /**
     * Set all data concerning prices
     */
    protected function generatePriceRelatedData()
    {
        $currencyID = 0;
        $initHelper = new Shopgate_Helpers_PluginInit();
        $initHelper->getCurrencyByCode(
            $this->config->getCurrency(),
            $this->exchangeRate,
            $currencyID,
            $this->shopCurrency
        );
        $countryId = $initHelper->getCountryByIsoCode2($this->config->getCountry());
        $zoneId    = $this->config->getTaxZoneId();
        $taxRate   = xtc_get_tax_rate($this->item['products_tax_class_id'], $countryId, $zoneId);
        $xtPrice   = new xtcPrice($this->shopCurrency['code'], $this->config->getCustomerPriceGroup());
        $price     = $xtPrice->xtcGetPrice(
            $this->item['products_id'],
            false,
            1,
            $this->item['products_tax_class_id'],
            $this->item['products_price'],
            1
        );
        $oldPrice  = ($price < $this->item['products_price'])
            ? $this->priceHelper->formatPriceNumber(
                ($this->item['products_price'] * $this->exchangeRate) * (1 + ($taxRate / 100))
            )
            : '';

        $this->defaultRow['tax_percent']     = $taxRate;
        $this->defaultRow['unit_amount']     = $this->priceHelper->formatPriceNumber($price);
        $this->defaultRow['basic_price']     = $this->generateBasePrice($price);
        $this->defaultRow['old_unit_amount'] =
            (!empty($oldPrice) && $this->priceHelper->formatPriceNumber($oldPrice) > 0)
                ? $oldPrice
                : '';
    }

    /**
     * Build input fields for export
     */
    protected function generateInputs()
    {
        $rawInputs      = $this->getInputFields($this->item['products_id']);
        $preparedInputs = $this->buildInputFields($rawInputs);

        if (!empty($preparedInputs)) {
            $this->defaultRow['has_input_fields'] = '1';
            $this->defaultRow                     = array_merge($this->defaultRow, $preparedInputs);
        }
    }

    /**
     * Bring the input field data in the data structure as needed
     *
     * @param $inputFieldsAll
     *
     * @return array
     */
    private function buildInputFields($inputFieldsAll)
    {
        $sgProductVar = array();
        $i            = 0;
        foreach ($inputFieldsAll as $inputField) {
            $i++;

            $sgProductVar["input_field_" . $i . "_number"]     = $inputField[0]["products_options_id"];
            $sgProductVar["input_field_" . $i . "_type"]       = 'text';
            $sgProductVar["input_field_" . $i . "_label"]      =
                strip_tags($inputField[0]["products_options_name"]);
            $sgProductVar["input_field_" . $i . "_add_amount"] =
                ($inputField["options_values_price"] != 0) ?
                    "=>" . $inputField["price_prefix"] . round(
                        $inputField["options_values_price"], 2
                    ) : "";
            // keine Angabe möglich
            $sgProductVar["input_field_" . $i . "_infotext"] = '';
            $sgProductVar["input_field_" . $i . "_required"] = 0;
        }

        return $sgProductVar;
    }

    /**
     * Generate variation data as needed
     *
     * @return array
     */
    public function generateVariations()
    {
        $rawVariations        = $this->_buildVariations();
        $aggregatedVariations = array();

        if (!empty($rawVariations)) {
            if ($rawVariations['has_options']) {
                $aggregatedVariations = array_merge($this->defaultRow, $rawVariations);
            } else {
                $this->defaultRow['has_children'] = 1;
                if (isset($rawVariations['has_options'])) {
                    unset($rawVariations['has_options']);
                }
                $itemNumber       = $this->defaultRow['item_number'];
                $basePrice        = round($this->defaultRow['unit_amount'], 2);
                $baseOldPrice     = round($this->defaultRow['old_unit_amount'], 2);
                $baseWeight       = $this->defaultRow['weight'];
                $parentItemNumber = $this->defaultRow['item_number'];
                $isFirst          = true;
                $orderInfos       = json_decode($this->defaultRow['internal_order_info'], 1);
                $itemArr          = $this->defaultRow;

                $initHelper = new Shopgate_Helpers_PluginInit();
                $countryId  = $initHelper->getCountryByIsoCode2($this->config->getCountry());
                $zoneId     = $this->config->getTaxZoneId();
                $taxRate    = xtc_get_tax_rate($this->item['products_tax_class_id'], $countryId, $zoneId);

                $customerModel                   = new Shopgate_Models_Customer($this->config, $this->languageId);
                $customerGroupDiscountAttributes = $customerModel->getCustomerDiscountGroupAttributes();

                foreach ($rawVariations as $key => $variation) {
                    $price  = 0;
                    $weight = 0;
                    // Offset amount including tax without discounts (but with exchange rate, of set)
                    $originalOffsetAmount = 0;
                    if (!empty($variation["offset_amount"])) {
                        if (!empty($this->exchangeRate)) {
                            $variation["offset_amount"] *= $this->exchangeRate;
                            $variations[$key]["offset_amount"] =
                                $variation["offset_amount"];
                        }
                        $originalOffsetAmount = $variation["offset_amount"] * (1 + ($taxRate / 100));

                        // Variations also need to be discounted if products discount is set
                        if (!empty($productDiscount) && round($productDiscount, 2) > 0) {
                            if ($customerGroupDiscountAttributes) { // Seems to be buggy in gambio so it is ignored here
                                $variation["offset_amount"]        =
                                    $this->_getDiscountPrice(
                                        $variation["offset_amount"],
                                        $productDiscount
                                    );
                                $variations[$key]["offset_amount"] =
                                    $variation["offset_amount"];
                            }
                        }
                        $price = $variation["offset_amount"] * (1 + ($taxRate / 100));
                    }

                    if (isset($variation["offset_weight"])) {
                        $weight = $variation["offset_weight"] * 1000;
                    }

                    $hash = "";

                    for (
                        $i = 1;
                        $i < 10 && isset($variation["attribute_$i"]); $i++
                    ) {
                        $hash .= $variation["attribute_$i"];
                        $itemArr["attribute_$i"] = htmlentities(
                            $variation["attribute_$i"],
                            ENT_NOQUOTES,
                            $this->config->getEncoding()
                        );
                    }

                    $hash = md5($hash);
                    $hash = substr($hash, 0, 5);
                    if (empty($variation)) {
                        $variation = array("order_info" => array());
                    }

                    // Set Order Info from parent product
                    if (!empty($variation['order_info'])
                        && is_array($variation['order_info'])
                    ) {
                        $variation["order_info"] = !is_null($orderInfos)
                            ? array_merge($orderInfos, $variation['order_info'])
                            : $variation['order_info'];
                    } else {
                        $variation["order_info"] = $orderInfos;
                    }

                    $itemArr['internal_order_info'] =
                        $this->jsonEncode($variation["order_info"]);
                    $itemArr["item_number"]         =
                        $itemNumber . ($isFirst ? "" : "_" . $hash);
                    if (!empty($variation["item_number"])) {
                        $itemArr["item_number_public"] =
                            $variation["item_number"];
                    } else {
                        $itemArr['item_number_public'] =
                            $this->item['products_model'];
                    }

                    $itemArr["unit_amount"] =
                        $this->priceHelper->formatPriceNumber($basePrice + $price);
                    if (!empty($baseOldPrice)
                        && round($baseOldPrice, 2) > 0
                    ) {
                        $itemArr["old_unit_amount"] =
                            $this->priceHelper->formatPriceNumber(
                                $baseOldPrice + $originalOffsetAmount
                            );
                    } else {
                        $itemArr["old_unit_amount"] = '';
                    }
                    $itemArr["weight"] = $baseWeight + $weight;

                    if (!empty($variation["ean"])) {
                        $itemArr["ean"] = $variation["ean"];
                    }

                    if ($isFirst == false) {
                        $itemArr["use_stock"] =
                            (STOCK_ALLOW_CHECKOUT == 'true' || ATTRIBUTE_STOCK_CHECK != 'true') ? 0 : 1;
                    }

                    // Overwrite stock only if its set up in the configuration
                    if (ATTRIBUTE_STOCK_CHECK == 'true' && $isFirst == false) {
                        if (!empty($this->item["specials_new_products_price"])
                            && $this->item["specials_quantity"] > 0
                        ) {
                            $itemArr["stock_quantity"] =
                                $variation["stock_quantity"]
                                > $this->item["specials_quantity"]
                                    ? $this->item["specials_quantity"]
                                    : $variation["stock_quantity"];
                        } else {
                            $itemArr["stock_quantity"] =
                                $variation["stock_quantity"];
                        }
                    }

                    $aggregatedVariations[] = $itemArr;

                    $isFirst                       = false;
                    $itemArr['has_children']       = 0;
                    $itemArr['parent_item_number'] = $parentItemNumber;
                }
            }
        } else {
            $this->defaultRow['has_children'] = 0;
            $this->defaultRow['has_options']  = 0;
        }

        return $aggregatedVariations;
    }

    /**
     * Returns a array with all Variations of the Product
     *
     * @return array
     */
    private function _buildVariations()
    {
        $this->log('execute generateVariations() ...', ShopgateLogger::LOGTYPE_DEBUG);

        $productId  = $this->item['products_id'];
        $initHelper = new Shopgate_Helpers_PluginInit();
        $countryId  = $initHelper->getCountryByIsoCode2($this->config->getCountry());
        $zoneId     = $this->config->getTaxZoneId();
        $taxRate    = xtc_get_tax_rate($this->item['products_tax_class_id'], $countryId, $zoneId);

        $optionsAsTextfieldIds = $this->config->getExportOptionAsInputField();

        $sg_prod_var = array();
        $eanColumn = $this->config->getAttributeEanColumn();
        if (!empty($eanColumn)) {
            $eanColumn = ", pa.$eanColumn";
        }

        $qry = "
            SELECT
                pa.products_attributes_id,
                po.products_options_id,
                pov.products_options_values_id,
                po.products_options_name,
                pov.products_options_values_name,
                pa.attributes_model,
                pa.options_values_price,
                pa.price_prefix,
                pa.options_values_weight,
                pa.attributes_stock,
                pa.weight_prefix
                $eanColumn
            FROM " . TABLE_PRODUCTS_ATTRIBUTES . " pa
            INNER JOIN " . TABLE_PRODUCTS_OPTIONS
            . " po ON (pa.options_id = po.products_options_id AND po.language_id = $this->languageId)
            INNER JOIN " . TABLE_PRODUCTS_OPTIONS_VALUES
            . " pov ON (pa.options_values_id = pov.products_options_values_id AND pov.language_id = $this->languageId)
            WHERE pa.products_id = '" . $productId . "'
                AND pov.products_options_values_name != 'TEXTFELD'";

        $qry .= (!empty($optionsAsTextfieldIds)) ? " AND pa.options_id NOT IN (" . mysql_real_escape_string(
                $optionsAsTextfieldIds
            ) . ")" : "";
        $qry .= " ORDER BY pa.sortorder ASC";

        $query   = xtc_db_query($qry);
        $options = array();

        while ($variation = xtc_db_fetch_array($query)) {
            $options[$variation["products_options_id"]][] = $variation;
        }

        /**
         * reindex
         */
        $options = array_values(array_filter($options));

        if (empty($options)) {
            return array();
        }

        // Find and rename duplicate option-value names
        foreach ($options as $optionIndex => $singleOption) {
            // Check all option-value names for duplicate names
            foreach ($singleOption as $key => $optionVariation) {
                if (!empty($optionVariation)) {
                    // Compare with following entries
                    $indexNumber = 1;
                    for ($i = $key + 1; $i < count($singleOption); $i++) {
                        if (trim(
                                $singleOption[$i]['products_options_values_name']
                            ) == trim(
                                $optionVariation['products_options_values_name']
                            )
                        ) {
                            $indexNumber++;
                            $options[$optionIndex][$i]['products_options_values_name'] .= " $indexNumber";
                        }
                    }
                    // Add index 1 to the actual name if duplicate name-entries found
                    if ($indexNumber > 1) {
                        $options[$optionIndex][$key]['products_options_values_name'] .= " 1";

                        // Refresh the working variable for further operation
                        $singleOption = $options[$optionIndex];
                    }
                }
            }
        }

        $countVariations = 1;
        foreach ($options as $option) {
            $countVariations *= count($option);
        }

        if ($countVariations > $this->config->getMaxAttributes()) {
            $this->_buildOptions($sg_prod_var, $options, $taxRate);
            $sg_prod_var["has_options"] = 1;
        } else {
            $this->_buildAttributes($sg_prod_var, $options);
            $sg_prod_var["has_options"] = 0;
        }

        return $sg_prod_var;
    }

    /**
     * Build the Productvariations as options
     *
     * @param &array $sg_prod_var
     * @param array $variations
     * @param float $tax_rate
     */
    private function _buildOptions(&$sg_prod_var, $variations, $tax_rate)
    {
        $this->log('execute _buildOptions() ...', ShopgateLogger::LOGTYPE_DEBUG);

        $tmp = array();
        $i   = 0;
        foreach ($variations as $_variation) {
            $i++;
            $tmp["option_$i"] =
                $_variation[0]["products_options_id"] . '=' . strip_tags(
                    $_variation[0]["products_options_name"]
                );

            $options = array();
            foreach ($_variation as $option) {
                // Currency and tax must be included here because the data is directly used for the item
                $optionOffsetPrice =
                    $option["options_values_price"] * $this->exchangeRate * (1
                        + ($tax_rate / 100)); // Include Tax
                $optionOffsetPrice =
                    round($optionOffsetPrice * 100, 0); // get euro-cent

                $field = trim(strip_tags($option["products_options_values_id"]))
                    . "=" . trim(
                        strip_tags($option["products_options_values_name"])
                    );
                $field .= ($option["options_values_price"] != 0) ?
                    "=>" . trim($option["price_prefix"]) . $optionOffsetPrice
                    : '';

                $options[] = $field;
            }
            $tmp["option_" . $i . "_values"] = implode("||", $options);
        }

        $sg_prod_var = $tmp;
    }

    /**
     * Build the Product variations recursively
     *
     * @param array $sg_prod_var
     * @param array $variations
     * @param int   $index
     * @param array $baseVar
     */
    private function _buildAttributes(
        &$sg_prod_var,
        $variations,
        $index = 0,
        $baseVar = array()
    ) {
        $this->log('execute _buildAttributes() ...', ShopgateLogger::LOGTYPE_DEBUG);

        if ($index == 0) {
            // Index 0 sind die Überschriften. Diese müssen als erstes hinzugefügt werden
            for ($i = 0; $i < count($variations); $i++) {
                $sg_prod_var[0]['attribute_' . ($i + 1)] =
                    $variations[$i][0]['products_options_name'];
            }
        }

        foreach ($variations[$index] as $variation) {
            $tmpNewVariation = array();

            // copy all prvious attributes (inclusive the order info)
            if (!empty($baseVar)) {
                for ($i = 1; $i <= 10; $i++) {
                    $keyName = 'attribute_' . $i;
                    if (array_key_exists($keyName, $baseVar)) {
                        $tmpNewVariation[$keyName]               =
                            $baseVar[$keyName];
                        $tmpNewVariation['order_info'][$keyName] =
                            $baseVar['order_info'][$keyName];
                    } else {
                        break;
                    }
                }
            }

            if (count($variations) == 1) {
                // only if 1 dimension
                $tmpNewVariation['item_number'] =
                    $variation['attributes_model'];
            }

            $tmpNewVariation['attribute_' . ($index + 1)]               =
                $variation['products_options_values_name'];
            $tmpNewVariation['order_info']['attribute_' . ($index + 1)] =
                array(
                    $variation['products_attributes_id'] => array(
                        'options_id'        => $variation['products_options_id'],
                        'options_values_id' => $variation['products_options_values_id'],
                    ),
                );

            $tmpNewVariation['stock_quantity'] = $variation['attributes_stock'];
            if (isset($baseVar['stock_quantity'])
                && $baseVar['stock_quantity'] < $variation['attributes_stock']
            ) {
                $tmpNewVariation['stock_quantity'] = $baseVar['stock_quantity'];
            }

            // Kalkuliere den Preisunterschied (Steuern und Währung werden noch nicht hier berücksichtigt)
            $price = $variation['options_values_price'];
            if ($variation['price_prefix'] == '-') {
                $price = -1 * $price;
            }
            if (empty($baseVar['offset_amount'])) {
                $baseVar['offset_amount'] = 0;
            }
            $tmpNewVariation['offset_amount'] =
                $baseVar['offset_amount'] + $price;

            // Kalkuliere den Gewichtsunterschied
            $weight = (float)$variation['options_values_weight'];
            if ($variation['weight_prefix'] == '-') {
                $weight = -1 * $weight;
            }
            if (empty($baseVar['offset_weight'])) {
                $baseVar['offset_weight'] = 0;
            }
            $tmpNewVariation['offset_weight'] =
                $baseVar['offset_weight'] + (double)$weight;

            $eanColumn = $this->config->getAttributeEanColumn();
            if (!empty($eanColumn) && !empty($variation[$eanColumn]) && empty($tmpNewVariation['ean'])) {
                $tmpNewVariation['ean'] = $variation[$eanColumn];
            }

            if ($index < (count($variations) - 1)) {
                // Fahre mit nächstem Attribute fort (mit aktuellem Zwischenattribut als Basis für die Gewicht, Stock und Preisberechnung)
                // Das aktuelle Zwischenattribut enthält das Gesamtgewicht, den Gesamtpreis und den max-Stock, der für weitere Berechnungen notwendig ist
                $this->_buildAttributes(
                    $sg_prod_var,
                    $variations,
                    $index + 1,
                    $tmpNewVariation
                );
            } else {
                // Wenn kein Attribut mehr existiert, dieses auf den Stack legen
                $sg_prod_var[] = $tmpNewVariation;
            }
        }
    }
}
