<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopgate_Models_Item_Xml extends Shopgate_Models_Item
{
    /**
     * @var null|Shopgate_Models_Item_Xml
     */
    protected $parent = null;
    
    /**
     * @var array
     */
    protected $variation = array();
    
    /**
     * @var xtcPrice[]
     */
    private $xtPricesByCustomerGroups;
    
    /**
     * @param Shopgate_Models_Item_Xml $parent
     */
    public function setParent($parent)
    {
        $this->parent = $parent;
    }
    
    /**
     * @param xtcPrice[] $xtPrices
     */
    public function setXtPricesByCustomerGroups(array $xtPrices)
    {
        $this->xtPricesByCustomerGroups = $xtPrices;
    }
    
    /**
     * @param array $variation
     */
    public function setVariation($variation)
    {
        $this->variation = $variation;
    }
    
    /**
     * fire methods used for child product generation
     */
    public function setFireMethodsForChildren()
    {
        $this->fireMethods = array(
            'setUid',
            'setAttributes',
            'setPrice',
            'setWeight',
            'setInternalOrderInfo',
            'setStock',
            'setDisplayType'
        );
    }
    
    public function setUid()
    {
        if ($this->getIsChild()) {
            $hash = '';
            foreach ($this->variation as $attribute) {
                $hash .= $attribute['products_options_values_name'];
            }
            $hash = md5($hash);
            $hash = substr($hash, 0, 5);
            parent::setUid($this->item['products_id'] . '_' . $hash);
        } else {
            parent::setUid($this->item['products_id']);
        }
    }
    
    public function setLastUpdate()
    {
        parent::setLastUpdate(date(DateTime::ISO8601, strtotime($this->item['products_last_modified'])));
    }
    
    public function setName()
    {
        parent::setName($this->generateItemName());
    }
    
    public function setTaxPercent()
    {
        $initHelper = new Shopgate_Helpers_PluginInit();
        $countryId  = $initHelper->getCountryByIsoCode2($this->config->getCountry());
        $zoneId     = $this->config->getTaxZoneId();
        $taxRate    = xtc_get_tax_rate($this->item['products_tax_class_id'], $countryId, $zoneId);
        parent::setTaxPercent($taxRate);
    }
    
    public function setTaxClass()
    {
        parent::setTaxClass($this->item['products_tax_class_id']);
    }
    
    public function setCurrency()
    {
        parent::setCurrency($this->shopCurrency['code']);
    }
    
    public function setDescription()
    {
        $stringHelper = $this->getHelper(ShopgateObject::HELPER_STRING);
        $description  = $stringHelper->removeTagsFromString($this->getProductDescription());
        parent::setDescription($description);
    }
    
    public function setPrice()
    {
        $xtPrice    = new xtcPrice($this->shopCurrency['code'], $this->config->getCustomerPriceGroup());
        $finalPrice = $xtPrice->xtcGetPrice(
            $this->item['products_id'],
            false,
            1,
            $this->item['products_tax_class_id'],
            $this->item['products_price'],
            1
        );
        
        $taxRate         = $this->getTaxRate();
        $regPriceWithTax = $xtPrice->xtcAddTax($this->item['products_price'], $taxRate);
        $oldPrice        = $finalPrice < $regPriceWithTax ? $regPriceWithTax : '';
        
        if ($this->getIsChild()) {
            $additionalPrice = $xtPrice->xtcAddTax($this->calculateVariationsAddAmount(), $taxRate);
            
            $finalPrice += $additionalPrice;
            if ($oldPrice > 0) {
                $oldPrice += $additionalPrice;
            }
        }

        // By default use gross prices
        $priceType = Shopgate_Model_Catalog_Price::DEFAULT_PRICE_TYPE_GROSS;

        if (defined('PRICE_IS_BRUTTO') && PRICE_IS_BRUTTO == 'false') {
            $priceType = Shopgate_Model_Catalog_Price::DEFAULT_PRICE_TYPE_NET;
        }

        $priceModel = new Shopgate_Model_Catalog_Price();
        $priceModel->setBasePrice($this->generateBasePrice($finalPrice));
        $priceModel->setSalePrice($this->priceHelper->formatPriceNumber($finalPrice));
        if ($oldPrice > 0) {
            $priceModel->setPrice($this->priceHelper->formatPriceNumber($oldPrice));
        }
        $priceModel->setType($priceType);
        
        $this->addTierPricesTo($priceModel);
        
        parent::setPrice($priceModel);
    }
    
    public function setDeeplink()
    {
        parent::setDeeplink($this->generateDeeplink());
    }
    
    public function setIdentifiers()
    {
        $result = array();
        
        $identifierItemObject = new Shopgate_Model_Catalog_Identifier();
        $identifierItemObject->setType('SKU');
        $identifierItemObject->setValue($this->item['products_model']);
        $result[] = $identifierItemObject;
        
        $identifierItemObject = new Shopgate_Model_Catalog_Identifier();
        $identifierItemObject->setType('EAN');
        $identifierItemObject->setValue($this->generateEan());
        $result[] = $identifierItemObject;
        
        parent::setIdentifiers($result);
    }
    
    public function setWeight()
    {
        $weight = $this->generateWeight();
        
        if ($this->getIsChild()
            && !empty($this->variation)
        ) {
            foreach ($this->variation as $variation) {
                
                $variationWeight = (float)$variation['options_values_weight'];
                if ($variation['weight_prefix'] == '-') {
                    $weight -= $variationWeight;
                } else {
                    $weight += $variationWeight;
                }
            }
        }
        
        parent::setWeight($weight);
    }
    
    public function setShipping()
    {
        // not supported by xt:commerce3
        $shipping = new Shopgate_Model_Catalog_Shipping();
        $shipping->setAdditionalCostsPerUnit(null);
        $shipping->setCostsPerOrder(null);
        $shipping->setIsFree(false);
        
        parent::setShipping($shipping);
    }
    
    public function setManufacturer()
    {
        $title             = $this->item['manufacturers_name'];
        $manufacturerModel = new Shopgate_Model_Catalog_Manufacturer();
        $manufacturerModel->setUid(bin2hex($title));
        $manufacturerModel->setTitle($title);
        $manufacturerModel->setItemNumber(false);
        
        parent::setManufacturer($manufacturerModel);
    }
    
    public function setVisibility()
    {
        $visibility = new Shopgate_Model_Catalog_Visibility();
        $visibility->setLevel(Shopgate_Model_Catalog_Visibility::DEFAULT_VISIBILITY_CATALOG_AND_SEARCH);
        $visibility->setMarketplace(true);
        
        parent::setVisibility($visibility);
    }
    
    public function setStock()
    {
        $useStock = $this->generateUseStock();
        $stockQty = $this->generateStockQuantity();
        
        if ($this->getIsChild()
            && (defined('ATTRIBUTE_STOCK_CHECK') && ATTRIBUTE_STOCK_CHECK == 'true')
        ) {
            foreach ($this->variation as $variation) {
                if ($variation['attributes_stock'] < $stockQty) {
                    $stockQty = $variation['attributes_stock'];
                }
            }
        }
        
        $stockModel = new Shopgate_Model_Catalog_Stock();
        $stockModel->setAvailabilityText($this->generateAvailableText());
        $stockModel->setStockQuantity($stockQty);
        if ($useStock) {
            $isSalable = $stockQty > 0
                ? 1
                : 0;
            $stockModel->setIsSaleable($isSalable);
            $stockModel->setUseStock(1);
        } else {
            $stockModel->setIsSaleable(1);
            $stockModel->setUseStock(0);
        }
        
        parent::setStock($stockModel);
    }
    
    public function setTags()
    {
        $result = array();
        $tags   = explode(',', trim($this->item['products_keywords']));
        
        foreach ($tags as $tag) {
            if (!ctype_space($tag) && !empty($tag)) {
                $tagItemObject = new Shopgate_Model_Catalog_Tag();
                $tagItemObject->setValue(trim($tag));
                $result[] = $tagItemObject;
            }
        }
        
        parent::setTags($result);
    }
    
    public function setCategoryPaths()
    {
        $result     = array();
        $categories = $this->fetchAllCategories();
        
        foreach ($categories as $category) {
            $categoryItemObject = new Shopgate_Model_Catalog_CategoryPath();
            $categoryItemObject->setUid($category['id']);
            $categoryItemObject->setSortOrder($category['sort']);
            
            $result[$category['id']] = $categoryItemObject;
        }
        
        parent::setCategoryPaths($result);
    }
    
    public function setInternalOrderInfo()
    {
        $orderInfos = $this->generateInternalOrderInfo();
        
        if ($this->getIsChild()
            && !empty($this->variation)
        ) {
            $i = 1;
            foreach ($this->variation as $variation) {
                $orderInfos['attribute_' . $i] = array(
                    $variation['products_attributes_id'] => array(
                        'options_id'        => $variation['products_options_id'],
                        'options_values_id' => $variation['products_options_values_id']
                    )
                );
                $i++;
            }
        }
        
        parent::setInternalOrderInfo(json_encode($orderInfos));
    }
    
    public function setAgeRating()
    {
        parent::setAgeRating($this->generateAgeRating());
    }
    
    public function setRelations()
    {
        $result       = array();
        $crossSellIds = $this->fetchRelatedShopItems($this->item['products_id']);
        
        if (!empty($crossSellIds)) {
            $crossSellRelation = new Shopgate_Model_Catalog_Relation();
            $crossSellRelation->setType(Shopgate_Model_Catalog_Relation::DEFAULT_RELATION_TYPE_UPSELL);
            $crossSellRelation->setValues($crossSellIds);
            $result[] = $crossSellRelation;
        }
        
        parent::setRelations($result);
    }
    
    public function setProperties()
    {
        $result     = array();
        $properties = $this->generateProperties();
        
        foreach ($properties as $data) {
            if (is_array($data)
                && !empty($data['label'])
                && !empty($data['value'])
            ) {
                $label = $data['label'];
                $value = $data['value'];
                
                $propertyItemObject = new Shopgate_Model_Catalog_Property();
                $propertyItemObject->setUid(bin2hex($label));
                $propertyItemObject->setLabel($label);
                $propertyItemObject->setValue($value);
                $result[] = $propertyItemObject;
            }
        }
        
        parent::setProperties($result);
    }
    
    public function setImages()
    {
        $result = array();
        $images = $this->generateProductsImages();
        
        foreach ($images as $imageUrl) {
            $imagesItemObject = new Shopgate_Model_Media_Image();
            $imagesItemObject->setUrl($imageUrl);
            $result[] = $imagesItemObject;
        }
        
        parent::setImages($result);
    }
    
    
    public function setDisplayType()
    {
        $children = $this->getData('children');
        if (!empty($children)) {
            parent::setDisplayType(Shopgate_Model_Catalog_Product::DISPLAY_TYPE_SELECT);
        } else {
            parent::setDisplayType(Shopgate_Model_Catalog_Product::DISPLAY_TYPE_SIMPLE);
        }
    }
    
    public function setAttributes()
    {
        $result          = array();
        $parentAttGroups = $this->parent != null
            ? $this->parent->getAttributeGroups()
            : array();
        
        foreach ($this->variation as $attribute) {
            $attributeId   = $attribute['products_options_id'];
            $attributeItem = new Shopgate_Model_Catalog_Attribute();
            $attributeItem->setGroupUid($attributeId);
            $attributeItem->setLabel($attribute['products_options_values_name']);
            $result[] = $attributeItem;
            
            if (!isset($parentAttGroups[$attributeId])) {
                /* @var $attribute Shopgate_Model_Catalog_AttributeGroup */
                $attributeGroupItem = new Shopgate_Model_Catalog_AttributeGroup();
                $attributeGroupItem->setUid($attributeId);
                $attributeGroupItem->setLabel($attribute['products_options_name']);
                $parentAttGroups[$attributeId] = $attributeGroupItem;
            }
        }
        
        if ($this->parent != null) {
            $this->parent->setAttributeGroups($parentAttGroups);
        }
        
        parent::setAttributes($result);
    }
    
    public function setChildren()
    {
        $children   = array();
        $variations = $this->buildVariations();
        $helper     = $this->getHelper(self::HELPER_DATASTRUCTURE);
        
        if ($this->combinationCount($variations) <= $this->config->getMaxAttributes()) {
            $childList = $helper->arrayCross($variations);
            foreach ($childList as $variation) {
                $childModel = clone $this;
                $childModel->setIsChild(true);
                $childModel->setParent($this);
                $childModel->setVariation($variation);
                $childModel->setFireMethodsForChildren();
                $children[] = $childModel->generateData();
            }
        } else {
            $this->buildChildInputs($variations);
        }
        
        parent::setChildren($children);
    }
    
    /**
     * prepare variation data and fill an instance of Shopgate_Model_Catalog_Input with the present data
     *
     * @param $variations
     */
    protected function buildChildInputs($variations)
    {
        $result = array();
        foreach ($variations as $variation) {
            $first = true;
            foreach ($variation as $attrValue) {
                // already filtered by query
                $inputType = Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT;
                
                $selectionPrice = !empty($attrValue['options_values_price'])
                    ? $attrValue['options_values_price'] == '-'
                        ? '-' . $attrValue['options_values_price']
                        : $attrValue['options_values_price']
                    : 0;
                
                if ($first) {
                    $first     = false;
                    $inputItem = new Shopgate_Model_Catalog_Input();
                    $inputItem->setUid($attrValue['products_options_id']);
                    $inputItem->setType($inputType);
                    $inputItem->setLabel(strip_tags($attrValue['products_options_name']));
                    $inputItem->setRequired(0);
                    if ($inputType != Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT) {
                        $inputItem->setAdditionalPrice($this->priceHelper->formatPriceNumber($selectionPrice));
                    }
                    $result[$inputItem->getUid()] = $inputItem;
                }
                
                /** @var Shopgate_Model_Catalog_Input $input */
                $input   = $result[$attrValue['products_options_id']];
                $options = $input->getOptions();
                
                $option = new Shopgate_Model_Catalog_Option();
                $option->setUid($attrValue['products_options_values_id']);
                $option->setLabel($attrValue['products_options_values_name']);
                $option->setAdditionalPrice($this->priceHelper->formatPriceNumber($selectionPrice));
                
                $options[$option->getUid()] = $option;
                $input->setOptions($options);
            }
        }
        
        parent::setInputs($result);
    }
    
    private function combinationCount($variations)
    {
        if (empty($variations) || !is_array($variations)) {
            return 0;
        }
        
        $sum = 1;
        foreach ($variations as $variation) {
            $sum = $sum * count($variation);
        }
        
        return $sum;
    }
    
    public function setInputs()
    {
        $result    = array();
        $rawInputs = $this->getInputFields($this->item['products_id']);
        foreach ($rawInputs as $input) {
            $price = !empty($input['options_values_price'])
                ? $input['options_values_price'] == '-'
                    ? '-' . $input['options_values_price']
                    : $input['options_values_price']
                : 0;
            
            $inputItem = new Shopgate_Model_Catalog_Input();
            $inputItem->setUid($input['products_options_id']);
            $inputItem->setType(Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_TEXT);
            $inputItem->setLabel(strip_tags($input['products_options_name']));
            $inputItem->setRequired(0);
            $inputItem->setAdditionalPrice($this->priceHelper->formatPriceNumber($price));
            
            $result[] = $inputItem;
        }
        parent::setInputs($result);
    }
    
    /**
     * Returns a array with all Variations of the Product
     *
     * @return array
     */
    private function buildVariations()
    {
        $this->log('execute _buildVariations() ...', ShopgateLogger::LOGTYPE_DEBUG);
        
        $productId             = $this->item['products_id'];
        $optionsAsTextfieldIds = $this->config->getExportOptionAsInputField();
        $eanColumn             = $this->config->getAttributeEanColumn();
        
        if (!empty($eanColumn)) {
            $eanColumn = ", pa.$eanColumn";
        }
        
        /** @noinspection SqlResolve */
        $qry = "
            SELECT
                pa.products_attributes_id,
                po.products_options_id,
                pov.products_options_values_id,
                po.products_options_name,
                pov.products_options_values_name,
                pa.attributes_model,
                pa.options_values_price,
                pa.price_prefix,
                pa.options_values_weight,
                pa.attributes_stock,
                pa.weight_prefix
                $eanColumn
            FROM " . TABLE_PRODUCTS_ATTRIBUTES . " AS pa
            INNER JOIN " . TABLE_PRODUCTS_OPTIONS
            . " AS po ON (pa.options_id = po.products_options_id AND po.language_id = $this->languageId)
            INNER JOIN " . TABLE_PRODUCTS_OPTIONS_VALUES
            . " AS pov ON (pa.options_values_id = pov.products_options_values_id AND pov.language_id = $this->languageId)
            WHERE pa.products_id = '" . $productId . "'
                AND pov.products_options_values_name != 'TEXTFELD'";
        
        $qry .= (!empty($optionsAsTextfieldIds)) ? " AND pa.options_id NOT IN (" . mysql_real_escape_string(
                $optionsAsTextfieldIds
            ) . ")" : "";
        $qry .= " ORDER BY pa.sortorder ASC";
        
        $query      = xtc_db_query($qry);
        $variations = array();
        
        while ($variation = xtc_db_fetch_array($query)) {
            $variations[$variation['products_options_id']][] = $variation;
        }
        
        /**
         * reindex
         */
        $variations = array_values(array_filter($variations));
        
        if (empty($variations)) {
            return array();
        }
        
        // Find and rename duplicate option-value names
        foreach ($variations as $optionIndex => $singleOption) {
            // Check all option-value names for duplicate names
            foreach ($singleOption as $key => $optionVariation) {
                if (!empty($optionVariation)) {
                    // Compare with following entries
                    $indexNumber = 1;
                    for ($i = $key + 1; $i < count($singleOption); $i++) {
                        if (trim(
                                $singleOption[$i]['products_options_values_name']
                            ) == trim(
                                $optionVariation['products_options_values_name']
                            )
                        ) {
                            $indexNumber++;
                            $variations[$optionIndex][$i]['products_options_values_name'] .= " $indexNumber";
                        }
                    }
                    // Add index 1 to the actual name if duplicate name-entries found
                    if ($indexNumber > 1) {
                        $variations[$optionIndex][$key]['products_options_values_name'] .= ' 1';
                        
                        // Refresh the working variable for further operation
                        $singleOption = $variations[$optionIndex];
                    }
                }
            }
        }
        
        return $variations;
    }
    
    /**
     * @param Shopgate_Model_Catalog_Price $priceModel
     */
    protected function addTierPricesTo(Shopgate_Model_Catalog_Price $priceModel)
    {
        /**
         * @var int      $customerGroupId
         * @var xtcPrice $xtPrice
         */
        foreach ($this->xtPricesByCustomerGroups as $customerGroupId => $xtPrice) {
            if ($xtPrice->cStatus['customers_status_show_price'] == '0') {
                continue;
            }
            
            $getDiscountsOnly = ($xtPrice->cStatus['customers_status_graduated_prices'] == '1')
                ? ''
                : ' AND `quantity` = 1 ';
            
            $quantitiesQuery = xtc_db_query(
                'SELECT `quantity` ' .
                'FROM `personal_offers_by_customers_status_' . ((int)$customerGroupId) . '` ' .
                'WHERE `products_id` = ' . ((int)$this->item['products_id']) . ' ' . $getDiscountsOnly .
                'ORDER BY `quantity`;'
            );
            
            while ($quantity = xtc_db_fetch_array($quantitiesQuery)) {
                $addAmount = $this->getIsChild()
                    ? $this->calculateVariationsAddAmount()
                    : 0;
                
                $graduatedPrice = $xtPrice->xtcGetPrice(
                    $this->item['products_id'],
                    false,
                    $quantity['quantity'],
                    $this->item['products_tax_class_id'],
                    $this->item['products_price'],
                    1
                );
                
                if ($this->getIsChild()) {
                    $addAmountGross =
                        $addAmount *
                        $this->exchangeRate *
                        (1 + ($xtPrice->TAX[$this->item['products_tax_class_id']] / 100));
                    $graduatedPrice += $addAmountGross - (($xtPrice->cStatus['customers_status_discount_attributes'])
                            ? ($addAmountGross * ($xtPrice->xtcCheckDiscount($this->item['products_id']) / 100))
                            : 0
                        );
                }
                
                $reduction = $priceModel->getSalePrice() - $graduatedPrice;
                
                $tierPriceModel = new Shopgate_Model_Catalog_TierPrice();
                $tierPriceModel->setFromQuantity($quantity['quantity']);
                $tierPriceModel->setReduction($this->priceHelper->formatPriceNumber($reduction, 6));
                $tierPriceModel->setReductionType(Shopgate_Model_Catalog_TierPrice::DEFAULT_TIER_PRICE_TYPE_FIXED);
                $tierPriceModel->setAggregateChildren(true); // tier prices are always aggregated in
                $tierPriceModel->setCustomerGroupUid($customerGroupId);
                
                if (round($reduction, 6) > 0) {
                    $priceModel->addTierPriceGroup($tierPriceModel);
                }
            }
        }
    }
    
    /**
     * @return int
     */
    protected function calculateVariationsAddAmount()
    {
        $additionalPrice = 0;
        foreach ($this->variation as $variation) {
            $additionalVariantPrice = $variation['options_values_price'];
            if ($variation['price_prefix'] == '-') {
                $additionalPrice -= $additionalVariantPrice;
            } else {
                $additionalPrice += $additionalVariantPrice;
            }
        }
        
        return $additionalPrice;
    }
    
    /**
     * Retrieves current item's tax rate
     *
     * @return float|int
     */
    private function getTaxRate()
    {
        $currencyID = 0;
        $initHelper = new Shopgate_Helpers_PluginInit();
        $initHelper->getCurrencyByCode(
            $this->config->getCurrency(),
            $this->exchangeRate,
            $currencyID,
            $this->shopCurrency
        );
        $countryId = $initHelper->getCountryByIsoCode2($this->config->getCountry());
        $zoneId    = $this->config->getTaxZoneId();
        
        return xtc_get_tax_rate($this->item['products_tax_class_id'], $countryId, $zoneId);
    }
}
