<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Xtcommerce3\Tracking\Entity;

use Shopgate\Xtcommerce3\Tracking\ReadModel\CartDiff;
use Shopgate\Xtcommerce3\Tracking\ReadModel\CartItem;

class Cart
{
    /** @var CartItem[] */
    private $items;

    function __construct()
    {
        $this->items = array();
    }

    /**
     * @param \shoppingCart $xtc3Cart
     *
     * @return CartDiff[]
     */
    public function getDifferences(\shoppingCart $xtc3Cart)
    {
        /** @var CartDiff[] $cartDiffs */
        $cartDiffs = array();
        $products  = $xtc3Cart->get_products();
        if (!is_array($products)) {
            $products = array();
        }

        $itemsNew = $this->getCartItemsFromNativeProducts($products);
        $itemIds  = array_merge(array_keys($itemsNew), array_keys($this->items));

        foreach ($itemIds as $itemId) {
            if (isset($this->items[$itemId])) {
                $qtyOld    = $this->items[$itemId]->quantity;
                $amountOld = $this->items[$itemId]->totalAmount;
            } else {
                $qtyOld    = 0;
                $amountOld = 0;
            }
            if (isset($itemsNew[$itemId])) {
                $qtyNew    = $itemsNew[$itemId]->quantity;
                $amountNew = $itemsNew[$itemId]->totalAmount;
            } else {
                $qtyNew    = 0;
                $amountNew = 0;
            }
            $quantityDiff = $qtyNew - $qtyOld;
            $amountDiff   = $amountNew - $amountOld;

            if (!empty($quantityDiff)) {
                $cartDiffs[$itemId] = new CartDiff($itemId, $quantityDiff, $amountDiff);
            }
        }

        return $cartDiffs;
    }

    /**
     * @param \shoppingCart $xtc3Cart
     */
    public function update(\shoppingCart $xtc3Cart)
    {
        $products = $xtc3Cart->get_products();
        if (!is_array($products)) {
            $products = array();
        }

        $this->items = $this->getCartItemsFromNativeProducts($products);
    }

    /**
     * @return CartItem[]
     */
    public function getItems()
    {
        return $this->items;
    }

    /**
     * @param int $itemId
     *
     * @return CartItem|null
     */
    public function getItem($itemId)
    {
        return isset($this->items[$itemId]) ? $this->items[$itemId] : null;
    }

    /**
     * @return int
     */
    public function getAmountItems()
    {
        $amount = 0;
        foreach ($this->items as $item) {
            $amount += $item->totalAmount;
        }

        return $amount;
    }

    /**
     * @param array $products
     *
     * @return CartItem[]
     */
    protected function getCartItemsFromNativeProducts(array $products)
    {
        /** @var CartItem[] $cartItems */
        $cartItems = array();

        foreach ($products as $product) {
            $id = $product['id'];
            if (isset($cartItems[$id])) {
                $cartItems[$id]->quantity    += $product['quantity'];
                $cartItems[$id]->totalAmount += round($product['quantity'] * $product['price'] * 100);
            } else {
                $name           = $product['name'];
                $quantity       = $product['quantity'];
                $totalAmount    = round($product['quantity'] * $product['price'] * 100);
                $cartItems[$id] = new CartItem($id, $name, $quantity, $totalAmount);
            }
        }

        return $cartItems;
    }
}