<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Xtcommerce3\Tracking\Repository\Globals\Datasource;

class Xtcommerce3
{
    /** @var \product */
    private $product;

    /** @var array */
    private $session;

    /** @var array */
    private $server;

    /** @var string|null */
    private $title;

    /**
     * @param \product $product
     * @param array    $session
     * @param array    $server
     */
    public function __construct(\product $product, array &$session = array(), array &$server = array())
    {
        $this->product = $product;
        $this->session = &$session;
        $this->server  = &$server;

        $this->title = defined('TITLE')
            ? TITLE
            : null;

    }

    /**
     * @return string|null
     */
    public function getScriptName()
    {
        return @$this->server['SCRIPT_FILENAME'];
    }

    /**
     * @return string|null
     */
    public function getCurrencyCode()
    {
        return @$this->session['currency'];
    }

    /**
     * @return \shoppingCart|null
     */
    public function getNativeCart()
    {
        return @$this->session['cart'];
    }

    /**
     * @return \shoppingCart
     */
    public function createNativeCart()
    {
        $this->session['cart'] = new \shoppingCart();

        return $this->getNativeCart();
    }

    /**
     * @return null|string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @return string|null
     */
    public function getProductMetaTitle()
    {
        return $this->getProductData('products_meta_title');
    }

    /**
     * @return string|null
     */
    public function getProductName()
    {
        return $this->getProductData('products_name');
    }

    /**
     * @return string|null
     */
    public function getProductSku()
    {
        return $this->getProductData('products_model');
    }

    /**
     * @return int|null
     */
    public function getProductId()
    {
        return $this->product->isProduct()
            ? $this->product->pID
            : null;
    }

    /**
     * @return float|null
     */
    public function getProductPrice()
    {
        return $this->getProductData('products_price');
    }

    /**
     * @param string $fieldName
     *
     * @return mixed|null
     */
    private function getProductData($fieldName)
    {
        return !$this->product->isProduct()
            ? null
            : @$this->product->data[$fieldName];
    }
}