<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

use Shopgate\FacebookPixelLibrary\Base\Repository\Settings;

define('SHOPGATE_SETTING_EXPORT_DESCRIPTION', 0);
define('SHOPGATE_SETTING_EXPORT_SHORTDESCRIPTION', 1);
define('SHOPGATE_SETTING_EXPORT_DESCRIPTION_SHORTDESCRIPTION', 2);
define('SHOPGATE_SETTING_EXPORT_SHORTDESCRIPTION_DESCRIPTION', 3);

class ShopgateConfigXtc extends ShopgateConfig implements Settings
{
    /**
     * @var array
     */
    protected $redirect_languages;

    /**
     * @var
     */
    protected $shipping;

    /**
     * @var int
     */
    protected $tax_zone_id;

    /**
     * @var int
     */
    protected $customers_status_id;

    /**
     * @var int
     */
    protected $customer_price_group;

    /**
     * @var int
     */
    protected $order_status_open;

    /**
     * @var int
     */
    protected $order_status_shipped;

    /**
     * @var int
     */
    protected $order_status_shipping_blocked;

    /**
     * @var int
     */
    protected $order_status_cancled;

    /**
     * @var
     */
    protected $always_cancel_shipping;

    /**
     * @var
     */
    protected $reverse_categories_sort_order;

    /**
     * @var
     */
    protected $reverse_items_sort_order;

    /**
     * @var
     */
    protected $export_description_type;

    /**
     * @var
     */
    protected $shopgate_table_version;

    /**
     * @var
     */
    protected $maximum_category_export_depth;

    /**
     * @var array
     */
    protected $category_blacklist_ids;

    /**
     * @var
     */
    protected $send_order_confirmation_mail;

    /**
     * @var
     */
    protected $send_shopgate_shipping_confirmation_mail;

    /**
     * @var
     */
    protected $export_new_products_category;

    /**
     * @var int
     */
    protected $export_new_products_category_id;

    /**
     * @var
     */
    protected $export_option_as_input_field;

    /**
     * @var string
     */
    protected $attribute_ean_column;

    /**
     * @var string
     */
    protected $payment_name_mapping;

    /**
     * @var int
     */
    protected $facebook_pixel_id;

    /**
     * @var bool
     */
    protected $facebook_pixel_add_payment_info_event_active;

    /**
     * @var bool
     */
    protected $facebook_pixel_add_to_cart_event_active;

    /**
     * @var bool
     */
    protected $facebook_pixel_add_to_wishlist_event_active;

    /**
     * @var bool
     */
    protected $facebook_pixel_complete_registration_event_active;

    /**
     * @var bool
     */
    protected $facebook_pixel_initiate_checkout_event_active;

    /**
     * @var bool
     */
    protected $facebook_pixel_lead_event_active;

    /**
     * @var bool
     */
    protected $facebook_pixel_purchase_event_active;

    /**
     * @var bool
     */
    protected $facebook_pixel_search_event_active;

    /**
     * @var bool
     */
    protected $facebook_pixel_view_content_event_active;

    public function startup()
    {
        // overwrite some library defaults
        $this->plugin_name = 'xt:Commerce 3';

        $this->enable_redirect_keyword_update = 24;
        $this->enable_ping                    = 1;
        $this->enable_add_order               = 1;
        $this->enable_update_order            = 1;
        $this->enable_get_orders              = 1;
        $this->enable_get_customer            = 1;
        $this->enable_get_items_csv           = 1;
        $this->enable_get_items               = 1;
        $this->enable_get_reviews             = 1;
        $this->enable_get_categories          = 1;
        $this->enable_get_media_csv           = 1;
        $this->enable_get_categories_csv      = 1;
        $this->enable_get_reviews_csv         = 1;
        $this->enable_get_log_file            = 1;
        $this->enable_register_customer       = 1;
        $this->enable_mobile_website          = 1;
        $this->enable_cron                    = 1;
        $this->enable_clear_log_file          = 1;
        $this->enable_clear_cache             = 1;
        $this->enable_get_settings            = 1;
        $this->enable_check_cart              = 1;
        $this->enable_check_stock             = 1;

        $this->shop_is_active = 1;

        $this->encoding = 'ISO-8859-15';

        // the path to cache, logs and temporary export files changed with plugin version 2.9.33
        $this->export_folder_path = $this->getDataFolderPath() . '/temp';
        $this->log_folder_path    = $this->getDataFolderPath() . '/log';
        $this->cache_folder_path  = $this->getDataFolderPath() . '/cache';
        
        // set path to configuration to the new path (starting with plugin version 2.9.33) just to be safe
        /** @noinspection PhpDeprecationInspection */
        $this->config_folder_path = $this->getDataFolderPath() . '/configuration';

        // default filenames if no language was selected
        $this->items_csv_filename      = 'items-undefined.csv';
        $this->categories_csv_filename = 'categories-undefined.csv';
        $this->reviews_csv_filename    = 'reviews-undefined.csv';

        $this->access_log_filename  = 'access-undefined.log';
        $this->request_log_filename = 'request-undefined.log';
        $this->error_log_filename   = 'error-undefined.log';
        $this->debug_log_filename   = 'debug-undefined.log';

        $this->redirect_keyword_cache_filename      =
            'redirect_keywords-undefined.txt';
        $this->redirect_skip_keyword_cache_filename =
            'skip_redirect_keywords-undefined.txt';

        // initialize plugin specific stuff
        $this->redirect_languages = array();

        $this->shipping = '';

        $this->tax_zone_id = 5;

        $this->customers_status_id  = 1;
        $this->customer_price_group = 0;

        $this->order_status_open             = 1;
        $this->order_status_shipped          = 3;
        $this->order_status_shipping_blocked = 1;

        $this->order_status_cancled   = 0;
        $this->always_cancel_shipping = 0;

        $this->reverse_categories_sort_order = false;
        $this->reverse_items_sort_order      = false;
        $this->export_description_type       =
            SHOPGATE_SETTING_EXPORT_DESCRIPTION;

        $this->shopgate_table_version = '';

        $this->maximum_category_export_depth            = '';
        $this->category_blacklist_ids                   = array();
        $this->send_order_confirmation_mail             = 0;
        $this->send_shopgate_shipping_confirmation_mail = 0;
        $this->export_new_products_category             = 0;
        $this->export_new_products_category_id          = 999;

        $this->supported_fields_check_cart
            = array(
            'shipping_methods',
            'currency',
            'items',
            'external_coupons',
        );

        $this->export_option_as_input_field = "";
        $this->attribute_ean_column         = '';
        $this->payment_name_mapping         = "";
    }

    public function buildConfigFilePath($fileName = self::DEFAULT_CONFIGURATION_FILE_NAME)
    {
        // if a configuration file under the default path does not exist
        // assume it in the former (pre-composer) default path
        return file_exists($this->getDefaultConfigFilePath($fileName))
            ? $this->getDefaultConfigFilePath($fileName)
            : $this->getLegacyConfigFilePath($fileName);
    }

    public function saveFile(array $fieldList, $path = null, $validate = true)
    {
        $fileName = null === $path
            ? self::DEFAULT_CONFIGURATION_FILE_NAME
            : basename($path);

        // always try writing to the default path
        $path = $this->getDefaultConfigFilePath($fileName);

        parent::saveFile($fieldList, $path, $validate);
    }

    /**
     * @param string $fileName
     *
     * @return string
     */
    private function getDefaultConfigFilePath($fileName = self::DEFAULT_CONFIGURATION_FILE_NAME)
    {
        return "{$this->getDataFolderPath()}/configuration/{$fileName}";
    }

    /**
     * @param string $fileName
     *
     * @return string
     */
    private function getLegacyConfigFilePath($fileName = self::DEFAULT_CONFIGURATION_FILE_NAME)
    {
        return __DIR__ . '/../shopgate_library/config/' . $fileName;
    }

    /**
     * @return string
     */
    private function getDataFolderPath()
    {
        return __DIR__ . '/../data';
    }

    /**
     * @return mixed
     */
    public function getExportOptionAsInputField()
    {
        return $this->export_option_as_input_field;
    }

    /**
     * @param string $value
     */
    public function setExportOptionAsInputField($value)
    {
        $this->export_option_as_input_field = $value;
    }

    /**
     * @param array $fieldList
     *
     * @return array
     */
    protected function validateCustom(array $fieldList = array())
    {
        $failedFields = array();

        foreach ($fieldList as $field) {
            switch ($field) {
                case 'redirect_languages':
                    // at least one redirect language must be selected
                    if (empty($this->redirect_languages)) {
                        $failedFields[] = $field;
                    }
                    break;
            }
        }

        return $failedFields;
    }

    /**
     * @return mixed
     */
    public function getRedirectLanguages()
    {
        return $this->redirect_languages;
    }

    /**
     * @param $value
     */
    public function setRedirectLanguages($value)
    {
        $this->redirect_languages = $value;
    }

    /**
     * @return mixed
     */
    public function getShipping()
    {
        return $this->shipping;
    }

    /**
     * @param $value
     */
    public function setShipping($value)
    {
        $this->shipping = $value;
    }

    /**
     * @return mixed
     */
    public function getTaxZoneId()
    {
        return $this->tax_zone_id;
    }

    /**
     * @param $value
     */
    public function setTaxZoneId($value)
    {
        $this->tax_zone_id = $value;
    }

    /**
     * @return mixed
     */
    public function getCustomersStatusId()
    {
        return $this->customers_status_id;
    }

    /**
     * @param $value
     */
    public function setCustomersStatusId($value)
    {
        $this->customers_status_id = $value;
    }

    /**
     * @return mixed
     */
    public function getCustomerPriceGroup()
    {
        return $this->customer_price_group;
    }

    /**
     * @param $value
     */
    public function setCustomerPriceGroup($value)
    {
        $this->customer_price_group = $value;
    }

    /**
     * @return mixed
     */
    public function getOrderStatusOpen()
    {
        return $this->order_status_open;
    }

    /**
     * @param $value
     */
    public function setOrderStatusOpen($value)
    {
        $this->order_status_open = $value;
    }

    /**
     * @return mixed
     */
    public function getOrderStatusShipped()
    {
        return $this->order_status_shipped;
    }

    /**
     * @param $value
     */
    public function setOrderStatusShipped($value)
    {
        $this->order_status_shipped = $value;
    }

    /**
     * @return mixed
     */
    public function getOrderStatusShippingBlocked()
    {
        return $this->order_status_shipping_blocked;
    }

    /**
     * @param $value
     */
    public function setOrderStatusShippingBlocked($value)
    {
        $this->order_status_shipping_blocked = $value;
    }

    /**
     * @return mixed
     */
    public function getOrderStatusCancled()
    {
        return $this->order_status_cancled;
    }

    /**
     * @param $value
     */
    public function setOrderStatusCancled($value)
    {
        $this->order_status_cancled = $value;
    }

    /**
     * @return mixed
     */
    public function getAlwaysCancelShipping()
    {
        return $this->always_cancel_shipping;
    }

    /**
     * @param $value
     */
    public function setAlwaysCancelShipping($value)
    {
        $this->always_cancel_shipping = $value;
    }

    /**
     * @return mixed
     */
    public function getReverseCategoriesSortOrder()
    {
        return $this->reverse_categories_sort_order;
    }

    /**
     * @param $value
     */
    public function setReverseCategoriesSortOrder($value)
    {
        $this->reverse_categories_sort_order = $value;
    }

    /**
     * @return mixed
     */
    public function getReverseItemsSortOrder()
    {
        return $this->reverse_items_sort_order;
    }

    /**
     * @param $value
     */
    public function setReverseItemsSortOrder($value)
    {
        $this->reverse_items_sort_order = $value;
    }

    /**
     * @return mixed
     */
    public function getExportDescriptionType()
    {
        return $this->export_description_type;
    }

    /**
     * @param $value
     */
    public function setExportDescriptionType($value)
    {
        $this->export_description_type = $value;
    }

    /**
     * @return mixed
     */
    public function getShopgateTableVersion()
    {
        return $this->shopgate_table_version;
    }

    /**
     * @param $value
     */
    public function setShopgateTableVersion($value)
    {
        $this->shopgate_table_version = $value;
    }

    /**
     * @return mixed
     */
    public function getMaximumCategoryExportDepth()
    {
        return $this->maximum_category_export_depth;
    }

    /**
     * @param $value
     */
    public function setMaximumCategoryExportDepth($value)
    {
        $this->maximum_category_export_depth = $value;
    }

    /**
     * @return mixed
     */
    public function getCategoryBlacklistIds()
    {
        return $this->category_blacklist_ids;
    }

    /**
     * @param $value
     */
    public function setCategoryBlacklistIds($value)
    {
        $this->category_blacklist_ids = $value;
    }

    /**
     * @return mixed
     */
    public function getSendOrderConfirmationMail()
    {
        return $this->send_order_confirmation_mail;
    }

    /**
     * @param $value
     */
    public function setSendOrderConfirmationMail($value)
    {
        $this->send_order_confirmation_mail = $value;
    }

    /**
     * @return mixed
     */
    public function getSendShopgateShippingConfirmationMail()
    {
        return $this->send_shopgate_shipping_confirmation_mail;
    }

    /**
     * @param $value
     */
    public function setSendShopgateShippingConfirmationMail($value)
    {
        $this->send_shopgate_shipping_confirmation_mail = $value;
    }

    /**
     * @return mixed
     */
    public function getExportNewProductsCategory()
    {
        return $this->export_new_products_category;
    }

    /**
     * @param $value
     */
    public function setExportNewProductsCategory($value)
    {
        $this->export_new_products_category = $value;
    }

    /**
     * @return mixed
     */
    public function getExportNewProductsCategoryId()
    {
        return $this->export_new_products_category_id;
    }

    /**
     * @param $value
     */
    public function setExportNewProductsCategoryId($value)
    {
        $this->export_new_products_category_id = $value;
    }

    /**
     * @return string
     */
    public function getAttributeEanColumn()
    {
        return $this->attribute_ean_column;
    }

    /**
     * @param string $value
     */
    public function setAttributeEanColumn($value)
    {
        $this->attribute_ean_column = $value;
    }

    /**
     * @param string $value
     */
    public function setPaymentNameMapping($value)
    {
        $this->payment_name_mapping = $value;
    }

    /**
     * @return string
     */
    public function getPaymentNameMapping()
    {
        return $this->payment_name_mapping;
    }

    /**
     * Translation of mapping string to
     * array format
     *
     * @return array
     */
    public function getTranslatedPaymentMappings()
    {
        $result = array();
        $map    = explode(';', $this->getPaymentNameMapping());
        foreach ($map as $mapString) {
            $mapList = explode('=', $mapString);

            if (isset($mapList[1])) {
                $result[$mapList[0]] = $mapList[1];
            }
        }

        return $result;
    }

    /**
     * @return int
     */
    public function getFacebookPixelId()
    {
        return $this->facebook_pixel_id;
    }

    /**
     * @param int $facebook_pixel_id
     */
    public function setFacebookPixelId($facebook_pixel_id)
    {
        $this->facebook_pixel_id = $facebook_pixel_id;
    }

    /**
     * @return bool
     */
    public function getFacebookPixelAddPaymentInfoEventActive()
    {
        return $this->facebook_pixel_add_payment_info_event_active;
    }

    /**
     * @param bool $facebook_pixel_add_payment_info_event_active
     */
    public function setFacebookPixelAddPaymentInfoEventActive($facebook_pixel_add_payment_info_event_active)
    {
        $this->facebook_pixel_add_payment_info_event_active = $facebook_pixel_add_payment_info_event_active;
    }

    /**
     * @return bool
     */
    public function getFacebookPixelAddToCartEventActive()
    {
        return $this->facebook_pixel_add_to_cart_event_active;
    }

    /**
     * @param bool $facebook_pixel_add_to_cart_event_active
     */
    public function setFacebookPixelAddToCartEventActive($facebook_pixel_add_to_cart_event_active)
    {
        $this->facebook_pixel_add_to_cart_event_active = $facebook_pixel_add_to_cart_event_active;
    }

    /**
     * @return bool
     */
    public function getFacebookPixelAddToWishlistEventActive()
    {
        return $this->facebook_pixel_add_to_wishlist_event_active;
    }

    /**
     * @param bool $facebook_pixel_add_to_wishlist_event_active
     */
    public function setFacebookPixelAddToWishlistEventActive($facebook_pixel_add_to_wishlist_event_active)
    {
        $this->facebook_pixel_add_to_wishlist_event_active = $facebook_pixel_add_to_wishlist_event_active;
    }

    /**
     * @return bool
     */
    public function getFacebookPixelCompleteRegistrationEventActive()
    {
        return $this->facebook_pixel_complete_registration_event_active;
    }

    /**
     * @param bool $facebook_pixel_complete_registration_event_active
     */
    public function setFacebookPixelCompleteRegistrationEventActive($facebook_pixel_complete_registration_event_active)
    {
        $this->facebook_pixel_complete_registration_event_active = $facebook_pixel_complete_registration_event_active;
    }

    /**
     * @return bool
     */
    public function getFacebookPixelInitiateCheckoutEventActive()
    {
        return $this->facebook_pixel_initiate_checkout_event_active;
    }

    /**
     * @param bool $facebook_pixel_initiate_checkout_event_active
     */
    public function setFacebookPixelInitiateCheckoutEventActive($facebook_pixel_initiate_checkout_event_active)
    {
        $this->facebook_pixel_initiate_checkout_event_active = $facebook_pixel_initiate_checkout_event_active;
    }

    /**
     * @return bool
     */
    public function getFacebookPixelLeadEventActive()
    {
        return $this->facebook_pixel_lead_event_active;
    }

    /**
     * @param bool $facebook_pixel_lead_event_active
     */
    public function setFacebookPixelLeadEventActive($facebook_pixel_lead_event_active)
    {
        $this->facebook_pixel_lead_event_active = $facebook_pixel_lead_event_active;
    }

    /**
     * @return bool
     */
    public function getFacebookPixelPurchaseEventActive()
    {
        return $this->facebook_pixel_purchase_event_active;
    }

    /**
     * @param bool $facebook_pixel_purchase_event_active
     */
    public function setFacebookPixelPurchaseEventActive($facebook_pixel_purchase_event_active)
    {
        $this->facebook_pixel_purchase_event_active = $facebook_pixel_purchase_event_active;
    }

    /**
     * @return bool
     */
    public function getFacebookPixelSearchEventActive()
    {
        return $this->facebook_pixel_search_event_active;
    }

    /**
     * @param bool $facebook_pixel_search_event_active
     */
    public function setFacebookPixelSearchEventActive($facebook_pixel_search_event_active)
    {
        $this->facebook_pixel_search_event_active = $facebook_pixel_search_event_active;
    }

    /**
     * @return bool
     */
    public function getFacebookPixelViewContentEventActive()
    {
        return $this->facebook_pixel_view_content_event_active;
    }

    /**
     * @param bool $facebook_pixel_view_content_event_active
     */
    public function setFacebookPixelViewContentEventActive($facebook_pixel_view_content_event_active)
    {
        $this->facebook_pixel_view_content_event_active = $facebook_pixel_view_content_event_active;
    }
}
