<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
define("SHOPGATE_PLUGIN_VERSION", "2.9.33");

if (!defined('TABLE_ORDERS_SHOPGATE_ORDER')) {
    define('TABLE_ORDERS_SHOPGATE_ORDER', 'orders_shopgate_order');
}

/**
 * XTCommerce 3 Plugin for Shopgate
 */
class ShopgatePluginXtc3 extends ShopgatePlugin
{
    /**
     * Shipping type in case order was placed using plugin shipping
     */
    const SHIPPING_TYPE_API = "PLUGINAPI";

    /**
     * @var ShopgateConfigXtc
     */
    protected $config;

    /**
     * @var int
     */
    private $languageId;

    /**
     * @var int
     */
    private $countryId;

    /**
     * @var int
     */
    private $zoneId;

    /**
     * @var string
     */
    private $currency;

    /** @var string */
    private $currencyId;

    /**
     * @var string
     */
    private $language = "german";

    public function startup()
    {
        $this->requireFiles();

        $initHelper   = new Shopgate_Helpers_PluginInit();
        $this->config = new ShopgateConfigXtc();

        if (!isset($_REQUEST['shop_number'])) {
            $this->config->loadFile();
        } else {
            $this->config->loadByShopNumber($_REQUEST['shop_number']);
        }

        $this->countryId = $initHelper->getCountryByIsoCode2($this->config->getCountry());

        $initHelper->getLanguageByCode($this->config->getLanguage(), $this->languageId, $this->language);

        $initHelper->getCurrencyByCode(
            $this->config->getCurrency(),
            $this->exchangeRate,
            $this->currencyId, $this->currency
        );

        $this->zoneId = $this->config->getTaxZoneId();

        return true;
    }

    /**
     * @inheritdoc
     */
    public function registerCustomer($user, $plainPass, ShopgateCustomer $customer)
    {
        /** @noinspection PhpIncludeInspection */
        require_once(DIR_FS_INC . 'xtc_encrypt_password.inc.php');
        /** @var ShopgateCustomer $customer */
        $customer        = $customer->utf8Decode($this->config->getEncoding());
        $user            = $this->stringFromUtf8($user, $this->config->getEncoding());
        $plainPass       = $this->stringFromUtf8($plainPass, $this->config->getEncoding());
        $userExistResult = xtc_db_query(
            "SELECT count(1) AS exist FROM customers AS c WHERE c.account_type = '0' AND c.customers_email_address = \"{$user}\";"
        );
        $userCount       = xtc_db_fetch_array($userExistResult);
        $userCount       = $userCount['exist'];
        $encPass         = xtc_encrypt_password($plainPass);
        $date            = date("Y-m-d H:i:s");

        if ((int)$userCount >= 1) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::REGISTER_USER_ALREADY_EXISTS, '', true
            );
        }

        if (!defined(TABLE_CUSTOMERS_MEMO)) {
            define(TABLE_CUSTOMERS_MEMO, "customers_memo");
        }

        if (!defined(TABLE_CUSTOMERS_INFO)) {
            define(TABLE_CUSTOMERS_INFO, "customers_info");
        }

        $customerData =
            array(
                'customers_firstname'     => $customer->getFirstName(),
                'customers_lastname'      => $customer->getLastName(),
                'customers_email_address' => $customer->getMail(),
                'customers_telephone'     => $customer->getPhone(),
                'customers_newsletter'    => 0,
                "customers_gender"        => $customer->getGender(),
                'customers_password'      => $encPass,
                'customers_date_added'    => $date,
                'customers_last_modified' => $date,
                'customers_dob'           => $customer->getBirthday(),
                'delete_user'             => 0,
                'customers_status'        => DEFAULT_CUSTOMERS_STATUS_ID,
            );

        /**
         * Import customer pertaining shopgateCustomFields()
         * to Customer table
         *
         * @var ShopgateCustomer $customer
         */
        $orderModel = new Shopgate_Models_Order(new Shopgate_Helpers_Database());
        $list       = array_merge(array($customer), $customer->getAddresses());
        foreach ($list as $object) {
            $customerData = array_merge(
                $customerData,
                $orderModel->addCustomFields($object, TABLE_CUSTOMERS)
            );
        }

        xtc_db_perform(TABLE_CUSTOMERS, $customerData);
        $userId = xtc_db_insert_id();

        $userCidColumnExist = false;
        $qry                = "SHOW COLUMNS FROM `" . TABLE_CUSTOMERS . "`";
        $result             = xtc_db_query($qry);

        while ($row = xtc_db_fetch_array($result)) {
            if ($row['Field'] == 'customers_cid') {
                $userCidColumnExist = true;
                break;
            }
        }

        if ($userCidColumnExist) {
            $query = "UPDATE " . TABLE_CUSTOMERS . " SET customers_cid = {$userId} WHERE customers_id = {$userId};";
            xtc_db_query($query);
        }

        $customersInfo = array(
            'customers_info_id'                         => $userId,
            'customers_info_number_of_logons'           => 0,
            'customers_info_date_account_created'       => $date,
            'customers_info_date_account_last_modified' => $date,
        );
        xtc_db_perform(TABLE_CUSTOMERS_INFO, $customersInfo);

        $memoData = array(
            'customers_id' => $userId, 'memo_date' => $date,
            'memo_title'   => 'Shopgate - Account angelegt',
            'memo_text'    => 'Account wurde von Shopgate angelegt',
        );
        xtc_db_perform(TABLE_CUSTOMERS_MEMO, $memoData);

        $addresses   = array();
        $addressList = $customer->getAddresses();

        foreach ($addressList as $key => $address) {
            foreach ($addressList as $secondKey => $secondAddress) {
                if ($address->equals($secondAddress) && $secondKey < $key) {
                    $key = $secondKey;
                }
            }
            $addresses[$key] = $address;
        }

        $defaultAddr = true;
        foreach ($addresses as $address) {
            $stateCode    = ShopgateXtcMapper::getXtcStateCode($address->getState());
            $zoneQuery    =
                xtc_db_query("SELECT z.zone_id,z.zone_name FROM zones AS z WHERE z.zone_code = '" . $stateCode . "'");
            $zoneResult   = xtc_db_fetch_array($zoneQuery);
            $countryQuery = xtc_db_query(
                "SELECT c.countries_id FROM countries AS c WHERE c.countries_iso_code_2 ='"
                . $address->getCountry() . "'"
            );

            $countryResult = xtc_db_fetch_array($countryQuery);
            $addressData   = array(
                "customers_id"          => $userId,
                "entry_company"         => $address->getCompany(),
                "entry_zone_id"         => $zoneResult['zone_id'],
                "entry_country_id"      => $countryResult['countries_id'],
                "entry_firstname"       => $address->getFirstName(),
                "entry_lastname"        => $address->getLastName(),
                "entry_gender"          => $address->getGender(),
                "entry_street_address"  => $address->getStreet1(),
                "entry_postcode"        => $address->getZipcode(),
                "entry_city"            => $address->getCity(),
                "entry_state"           => $zoneResult['zone_name'],
                "address_date_added"    => "now()",
                "address_last_modified" => "now()",
            );

            xtc_db_perform(TABLE_ADDRESS_BOOK, $addressData);
            if ($defaultAddr) {
                $addressId = xtc_db_insert_id();
                $query     = "UPDATE " . TABLE_CUSTOMERS
                    . " as c SET customers_default_address_id = {$addressId} WHERE c.customers_id={$userId}";
                xtc_db_query($query);
                $defaultAddr = false;
            }
        }

        $this->sendCustomerRegistrationMail(
            $userId,
            $customer->getGender(),
            $customer->getMail(),
            $customer->getFirstName() . ' ' . $customer->getLastName()
        );
    }

    /**
     * send customer registration mail
     * Source was taken from create_account.php xtc file
     * also xtc_php_mailer class was taken from xtc3, changed and used in our plugin
     *
     * @param int    $userId
     * @param string $gender
     * @param string $emailAddress
     * @param string $name
     *
     * @throws ShopgateLibraryException
     */
    private function sendCustomerRegistrationMail($userId, $gender, $emailAddress, $name)
    {
        $_SESSION['language'] = empty($_SESSION['language']) ? $this->language : $_SESSION['language'];
        $xtPrice              = new xtcPrice($this->currency['code'], $userId);
        $smarty               = new Smarty;
        $module_content       =
            array('MAIL_NAME' => $name, 'MAIL_REPLY_ADDRESS' => EMAIL_SUPPORT_REPLY_ADDRESS, 'MAIL_GENDER' => $gender);

        // assign data to smarty
        $smarty->assign('language', $_SESSION['language']);
        $smarty->assign('logo_path', HTTP_SERVER.DIR_WS_CATALOG.'templates/'.CURRENT_TEMPLATE.'/img/');
        $smarty->assign('content', $module_content);
        $smarty->caching = false;

        if (ACTIVATE_GIFT_SYSTEM == 'true') {
            // GV Code Start
            // ICW - CREDIT CLASS CODE BLOCK ADDED  ******************************************************* BEGIN
            if (NEW_SIGNUP_GIFT_VOUCHER_AMOUNT > 0) {
                $couponCode  = create_coupon_code();
                $insertQuery = xtc_db_query(
                    "insert into " . TABLE_COUPONS
                    . " (coupon_code, coupon_type, coupon_amount, date_created) values ('" . $couponCode . "', 'G', '"
                    . NEW_SIGNUP_GIFT_VOUCHER_AMOUNT . "', now())"
                );
                $insertId    = xtc_db_insert_id($insertQuery);
                $insertQuery = xtc_db_query(
                    "insert into " . TABLE_COUPON_EMAIL_TRACK
                    . " (coupon_id, customer_id_sent, sent_firstname, emailed_to, date_sent) values ('" . $insertId
                    . "', '0', 'Admin', '" . $emailAddress . "', now() )"
                );

                $smarty->assign('SEND_GIFT', 'true');
                $smarty->assign('GIFT_AMMOUNT', $xtPrice->xtcFormat(NEW_SIGNUP_GIFT_VOUCHER_AMOUNT, true));
                $smarty->assign('GIFT_CODE', $couponCode);
                $smarty->assign(
                    'GIFT_LINK', xtc_href_link(FILENAME_GV_REDEEM, 'gv_no=' . $couponCode, 'NONSSL', false)
                );

            }
            if (NEW_SIGNUP_DISCOUNT_COUPON != '') {
                $couponCode        = NEW_SIGNUP_DISCOUNT_COUPON;
                $couponQuery       =
                    xtc_db_query("select * from " . TABLE_COUPONS . " where coupon_code = '" . $couponCode . "'");
                $coupon            = xtc_db_fetch_array($couponQuery);
                $couponId          = $coupon['coupon_id'];
                $couponDescQuery = xtc_db_query(
                    "select * from " . TABLE_COUPONS_DESCRIPTION . " where coupon_id = '" . $couponId
                    . "' and language_id = '" . (int)$_SESSION['languages_id'] . "'"
                );
                $couponDesc       = xtc_db_fetch_array($couponDescQuery);
                $insertQuery       = xtc_db_query(
                    "insert into " . TABLE_COUPON_EMAIL_TRACK
                    . " (coupon_id, customer_id_sent, sent_firstname, emailed_to, date_sent) values ('" . $couponId
                    . "', '0', 'Admin', '" . $emailAddress . "', now() )"
                );

                $smarty->assign('SEND_COUPON', 'true');
                $smarty->assign('COUPON_DESC', $couponDesc['coupon_description']);
                $smarty->assign('COUPON_CODE', $coupon['coupon_code']);

            }
            // ICW - CREDIT CLASS CODE BLOCK ADDED  ******************************************************* END
            // GV Code End       // create templates
        }

        $smarty->caching      = 0;
        $html_mail            =
            $smarty->fetch(CURRENT_TEMPLATE . '/mail/' . $_SESSION['language'] . '/create_account_mail.html');
        $smarty->caching      = 0;
        $txt_mail             =
            $smarty->fetch(CURRENT_TEMPLATE . '/mail/' . $_SESSION['language'] . '/create_account_mail.txt');

        if (!shopgate_php_mail(
            EMAIL_SUPPORT_ADDRESS,
            EMAIL_SUPPORT_NAME,
            $emailAddress,
            $name,
            EMAIL_SUPPORT_FORWARDING_STRING,
            EMAIL_SUPPORT_REPLY_ADDRESS,
            EMAIL_SUPPORT_REPLY_ADDRESS_NAME,
            '',
            '',
            EMAIL_SUPPORT_SUBJECT,
            $html_mail,
            $txt_mail
        )
        ) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_EMAIL_SEND_ERROR,
                "Error while sending customer registration mail"
            );
        }
    }

    public function createPluginInfo()
    {
        $projectVersion = '-';

        if (defined('PROJECT_VERSION')) {
            $projectVersion = PROJECT_VERSION;
        }

        return array('xtCommerce 3 Version' => $projectVersion);
    }

    protected function createCategoriesCsv()
    {
        $this->log("Start export categories tree...", ShopgateLogger::LOGTYPE_DEBUG);

        if ($this->config->getReverseCategoriesSortOrder()) {
            $maxOrder = 0;
        } else {
            $qry      =
                "SELECT MAX( sort_order ) sort_order FROM " . TABLE_CATEGORIES;
            $result   = xtc_db_query($qry);
            $maxOrder = xtc_db_fetch_array($result);
            $maxOrder = $maxOrder["sort_order"] + 1;
        }

        $this->buildCategoriesTree(0, $maxOrder, 'csv', $this->exportLimit, $this->exportOffset);

        if ($this->config->getExportNewProductsCategory()) {
            $row                    = $this->buildDefaultCategoryRow();
            $row['parent_id']       = '';
            $row['category_number'] = $this->config->getExportNewProductsCategoryId();
            $row['category_name']   = 'Neue Produkte';
            $row["is_active"]       = 1;
            $row['url_deeplink']    = xtc_href_link('products_new.php');

            $this->addCategoryRow($row);
        }

    }

    protected function createMediaCsv()
    {
        /*
            $this->log("Start export media files to items ...", ShopgateLogger::LOGTYPE_DEBUG);
            $row    = $this->buildDefaultMediaRow();

            $qry     =     'select p.products_id,p.products_video from products AS p WHERE p.products_video != "" GROUP BY p.products_id ' . (($this->splittedExport) ? " LIMIT {$this->exportOffset}, {$this->exportLimit}" : "");
            $this->log("execute SQL media files ...", ShopgateLogger::LOGTYPE_DEBUG);
            $products = xtc_db_query($qry);

            if(!empty($products)) {
                $item=array();
                $folderAndSuffix = "vorschau";
                while($product = xtc_db_fetch_array($products)) {
                    $dummyRow = $this->buildDefaultMediaRow();
                    $dummyRow['item_number'] = $product['products_id'];
                    $dummyRow['type']               = 'document';
                    $this->log("getting media information ...", ShopgateLogger::LOGTYPE_DEBUG);
                    $file = DIR_FS_CATALOG . "{$folderAndSuffix}/$";

                    if(!empty($content)){
                        parse_str($content, $videoInfos);

                        $formats = explode('/',$videoInfos['fmt_list']);
                        $width = 0;
                        $height = 0;

                        if(!empty($formats))
                        {
                            foreach($formats AS $format)
                            {
                                if(strpos($format, "x") !== false)
                                {
                                    $formatValues = explode('x',$format);
                                    if(count($formatValues) > 1)
                                    {
                                        $width  = $formatValues[0];
                                        $height = $formatValues[1];
                                        break;
                                    }
                                }
                            }
                        }

                        $dummyRow['parameter'] = $this->jsonEncode(
                            array(
                                'name'  => $videoInfos['title'],
                                'thumb' => $videoInfos['iurl'],
                                'width' => $width,
                                'height'=> $height,
                                'format'=> 'Flash',
                                'length'=> $videoInfos['length_seconds'],
                                'url'   => 'http://www.youtube.com/watch?v=' .  $videoInfos['video_id'],
                            )
                        );
                    }
                    $this->addMediaRow($dummyRow);
                }
            }
            */
    }

    protected function createItems($limit = null, $offset = null, array $uids = array())
    {
        $this->log("Start export of items ...", ShopgateLogger::LOGTYPE_DEBUG);

        $customerModel = new Shopgate_Models_Customer($this->config, $this->languageId);
        $productsResult = $this->buildItems($limit, $offset, $uids);

        $xtPricesByCustomerGroups = array();
        foreach ($customerModel->getCustomerGroups() as $customerGroup) {
            $xtPricesByCustomerGroups[$customerGroup['id']] = new xtcPrice($this->currency['code'], $customerGroup['id']);
            $xtPricesByCustomerGroups[$customerGroup['id']]->cStatus['customers_status_show_price_tax'] = 1;
        }

        while ($item = xtc_db_fetch_array($productsResult)) {
            /** @var Shopgate_Models_Item_Xml $productExportModel */
            $itemExportModel = new Shopgate_Models_Item_Xml($this->config, $this->languageId);
            $itemExportModel->setPriceHelper($this->getHelper(ShopgateObject::HELPER_PRICING));
            $itemExportModel->setXtPricesByCustomerGroups($xtPricesByCustomerGroups);
            $itemExportModel->setShopCurrency($this->currency);
            $itemExportModel->setItem($item);
            $this->addItemModel($itemExportModel->generateData());
        }
    }

    protected function createItemsCsv()
    {
        $this->log("Start export of items ...", ShopgateLogger::LOGTYPE_DEBUG);

        $uids = is_array($_REQUEST['item_numbers'])
            ? $_REQUEST['item_numbers']
            : array();

        $productsResult = $this->buildItems($this->exportLimit, $this->exportOffset, $uids);

        /** @var Shopgate_Models_Item_Csv $productExportModel */
        $itemExportModel = new Shopgate_Models_Item_Csv($this->config, $this->languageId);
        $itemExportModel->setDefaultRow($this->buildDefaultItemRow());
        $itemExportModel->setPriceHelper($this->getHelper(ShopgateObject::HELPER_PRICING));
        $itemExportModel->setShopCurrency($this->currency);

        while ($item = xtc_db_fetch_array($productsResult)) {
            $itemExportModel->setItem($item);
            $generatedData = $itemExportModel->generateData();
            $variations    = $itemExportModel->generateVariations();

            if (!empty($variations)) {
                if ($variations['has_options']) {
                    // fix for products with more than 10 options:
                    if (isset($variations['option_11'])) {
                        continue;
                    }
                    $this->addItemRow($variations);
                } else {
                    $generatedData['has_children'] = 1;

                    foreach ($variations as $variation) {
                        $this->addItemRow($variation);
                    }
                }
            } else {
                $this->addItemRow($generatedData);
            }
        }
    }

    /**
     * Prepares item model for export.
     *
     * @param int|null $limit
     * @param int|null $offset
     * @param array    $uids
     *
     * @return array
     */
    protected function buildItems($limit = null, $offset = null, array $uids = array())
    {
        $categoryModel     = new Shopgate_Models_Catalog_Category($this->config);
        $categoryBlacklist = $categoryModel->getBlacklistedCategoryIds();
        $this->log(
            "generate SQL get products ...", ShopgateLogger::LOGTYPE_DEBUG
        );

        $itemModel = new Shopgate_Models_Item($this->config, $this->languageId, $this->exchangeRate);

        $productsResult = $itemModel->getProductsFromDb($categoryBlacklist, $offset, $limit, $uids);
        $this->log("execute SQL get products ...", ShopgateLogger::LOGTYPE_DEBUG);

        return $productsResult;
    }

    public function getCustomer($user, $pass)
    {
        // save the UTF-8 version for logging etc.
        $userUtf8 = $user;

        // decode the parameters if necessary to make them work with xtc_* functions
        $user = $this->stringFromUtf8($user, $this->config->getEncoding());
        $pass = $this->stringFromUtf8($pass, $this->config->getEncoding());

        // find customer
        $qry = "SELECT"
            // basic user information
            . " customer.customers_id," . " customer.customers_cid,"
            . " status.customers_status_name," . " status.customers_status_id,"
            . " customer.customers_gender," . " customer.customers_firstname,"
            . " customer.customers_lastname,"
            . " date_format(customer.customers_dob,'%Y-%m-%d') as customers_birthday,"
            . " customer.customers_telephone,"
            . " customer.customers_email_address,"

            // additional information for password verification, default address etc.
            . " customer.customers_password,"
            . " customer.customers_default_address_id"

            . " FROM " . TABLE_CUSTOMERS . " AS customer"

            . " INNER JOIN " . TABLE_CUSTOMERS_STATUS . " AS status"
            . " ON customer.customers_status = status.customers_status_id"
            . " AND status.language_id = " . $this->languageId

            . " WHERE customer.account_type = '0'"
            . " AND customers_email_address = '" . xtc_db_input($user) . "';";

        // user exists?
        $customerResult = xtc_db_query($qry);
        if (empty($customerResult)) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_WRONG_USERNAME_OR_PASSWORD,
                'User: ' . $userUtf8
            );
        }

        // password's correct?
        $customerData = xtc_db_fetch_array($customerResult);
        if (!xtc_validate_password($pass, $customerData['customers_password'])) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_WRONG_USERNAME_OR_PASSWORD,
                'User: ' . $userUtf8
            );
        }

        // fetch customers' addresses
        $qry = "SELECT"

            . " address.address_book_id," . " address.entry_gender,"
            . " address.entry_firstname," . " address.entry_lastname,"
            . " address.entry_company," . " address.entry_street_address,"
            . " address.entry_postcode," . " address.entry_city,"
            . " country.countries_iso_code_2," . " zone.zone_code"


            . " FROM " . TABLE_ADDRESS_BOOK . " AS address"

            . " LEFT JOIN " . TABLE_COUNTRIES . " AS country"
            . " ON country.countries_id = address.entry_country_id"

            . " LEFT JOIN " . TABLE_ZONES . " AS zone"
            . " ON address.entry_zone_id = zone.zone_id"
            . " AND country.countries_id = zone.zone_country_id"

            . " WHERE address.customers_id = " . xtc_db_input(
                $customerData['customers_id']
            ) . ";";

        $addressResult = xtc_db_query($qry);
        if (empty($addressResult)) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_NO_ADDRESSES_FOUND,
                'User: ' . $userUtf8
            );
        }

        $addresses = array();
        while ($addressData = xtc_db_fetch_array($addressResult)) {
            try {
                $stateCode = ShopgateXtcMapper::getShopgateStateCode(
                    $addressData["countries_iso_code_2"],
                    $addressData["zone_code"]
                );
            } catch (ShopgateLibraryException $e) {
                // if state code can't be mapped to ISO use xtc3 state code
                $stateCode = $addressData['zone_code'];
            }

            try {
                $address = new ShopgateAddress();
                $address->setId($addressData['address_book_id']);
                // xtc3 doesn't make a difference
                $address->setAddressType(ShopgateAddress::BOTH);
                $address->setGender($addressData["entry_gender"]);
                $address->setFirstName($addressData["entry_firstname"]);
                $address->setLastName($addressData["entry_lastname"]);
                $address->setCompany($addressData["entry_company"]);
                $address->setStreet1($addressData["entry_street_address"]);
                $address->setZipcode($addressData["entry_postcode"]);
                $address->setCity($addressData["entry_city"]);
                $address->setCountry($addressData["countries_iso_code_2"]);
                $address->setState($stateCode);
            } catch (ShopgateLibraryException $e) {
                // don't abort here
            }

            // put default address in front, append the others
            if ($address->getId() == $customerData['customers_default_address_id']) {
                array_unshift($addresses, $address);
            } else {
                $addresses[] = $address;
            }
        }

        //xtc3 only allows exactly one customer group
        $customerGroup = new ShopgateCustomerGroup();
        $customerGroup->setId($customerData['customers_status_id']);
        $customerGroup->setName($customerData['customers_status_name']);

        try {
            $customer = new ShopgateCustomer();
            $customer->setCustomerId($customerData["customers_id"]);
            $customer->setCustomerNumber($customerData["customers_cid"]);
            $customer->setCustomerGroups(array($customerGroup));
            $customer->setGender($customerData["customers_gender"]);
            $customer->setFirstName($customerData["customers_firstname"]);
            $customer->setLastName($customerData["customers_lastname"]);
            $customer->setBirthday($customerData["customers_birthday"]);
            $customer->setPhone($customerData["customers_telephone"]);
            $customer->setMail($customerData["customers_email_address"]);
            $customer->setAddresses($addresses);
            $customer->setCustomerToken($this->getCustomerToken($customerData));

            // utf-8 encode the values recursively
            $customer = $customer->utf8Encode($this->config->getEncoding());
        } catch (ShopgateLibraryException $e) {
            // don't abort here
        }

        return $customer;
    }

    /**
     * @param array $customerData
     *
     * @return bool|string
     */
    private function getCustomerToken($customerData)
    {
        $customerModel = new Shopgate_Models_Customer($this->config, $this->languageId);

        if (!$customerModel->hasCustomerToken($customerData["customers_id"])) {
            return $customerModel->insertToken(
                $customerData['customers_id'],
                $customerData['customers_email_address']
            );
        }

        return $customerModel->getCustomerToken($customerData["customers_id"]);
    }

    public function addOrder(ShopgateOrder $order)
    {
        $locationModel = new Shopgate_Models_Location();
        $customerModel = new Shopgate_Models_Customer($this->config, $this->languageId);
        $orderModel    = new Shopgate_Models_Order(new Shopgate_Helpers_Database());
        $couponModel   = new Shopgate_Models_Coupon(
            $this->currency['code'],
            $this->config,
            $this->languageId,
            $this->language,
            $this->countryId
        );

        // save UTF-8 payment infos (to build proper json)
        $paymentInfosUtf8 = $order->getPaymentInfos();

        $this->log('start add_order()', ShopgateLogger::LOGTYPE_DEBUG);

        /* @var ShopgateOrder $order */// data needs to be utf-8 decoded for äöüß and the like to be saved correctly
        $order = $order->utf8Decode($this->config->getEncoding());

        $this->log('db: duplicate_order', ShopgateLogger::LOGTYPE_DEBUG);
        $orderModel->orderExists($order->getOrderNumber());

        // retrieve address information
        $delivery   = $order->getDeliveryAddress();
        $invoice    = $order->getInvoiceAddress();
        $customerId = $order->getExternalCustomerId();// find customer

        $shopCustomer = array();
        if (!empty($customerId)) {
            $this->log('db: customer', ShopgateLogger::LOGTYPE_DEBUG);
            $shopCustomer = $customerModel->getCustomerById($customerId);
        }

        if (empty($shopCustomer)) {
            $this->log('create Guest User', ShopgateLogger::LOGTYPE_DEBUG);
            $shopCustomer = $this->createGuestUser($order);
        }

        $customersAddress = $customerModel->getAddressByCustomersId(
            $shopCustomer['customers_id'],
            $shopCustomer['customers_default_address_id']
        );

        // get address format
        if (!empty($customersAddress)) {
            $addressFormatCustomer = $this->getAddressFormatId(
                null,
                $customersAddress['entry_country_id']
            );
        } else {
            $customersAddress =
                array(
                    'entry_gender'         => $shopCustomer['customers_gender'],
                    'entry_company'        => '',
                    'entry_firstname'      => $shopCustomer['customers_firstname'],
                    'entry_lastname'       => $shopCustomer['customers_lastname'],
                    'entry_street_address' => '', 'entry_suburb' => '',
                    'entry_postcode'       => '', 'entry_city' => '',
                    'entry_state'          => '', 'entry_country_id' => '',
                    'entry_zone_id'        => '',
                );
        }

        $addressFormatDelivery = $this->getAddressFormatId($delivery->getCountry());
        $addressFormatInvoice  = $this->getAddressFormatId($invoice->getCountry());

        if (empty($addressFormatCustomer)) {
            $addressFormatCustomer = $addressFormatInvoice;
        }

        if (empty($addressFormatCustomer)) {
            $addressFormatCustomer = $addressFormatDelivery;
        }

        $this->log('db: customer_status', ShopgateLogger::LOGTYPE_DEBUG);

        $customersStatus = $customerModel->customerStatusExist($shopCustomer);

        $this->log('before ShopgateMapper', ShopgateLogger::LOGTYPE_DEBUG);

        // map state codes (called "zone id" in shopsystem)
        $customersStateCode = $customersAddress['entry_state'];
        $invoiceStateCode   = $invoice->getState();
        $deliveryStateCode  = $delivery->getState();

        $this->log('db: countries', ShopgateLogger::LOGTYPE_DEBUG);
        $customersCountry = $locationModel->getCountryById($customersAddress['entry_country_id']);
        $deliveryCountry  = $locationModel->getCountryByIso2Name($delivery->getCountry());
        $invoiceCountry   = $locationModel->getCountryByIso2Name($invoice->getCountry());

        if (empty($customersCountry)) {
            $customersCountry = $invoiceCountry;
        }
        if (empty($customersCountry)) {
            $customersCountry = $deliveryCountry;
        }

        $orderData                              = array();
        $orderData["customers_id"]              = $shopCustomer["customers_id"];
        $orderData["customers_cid"]             = $shopCustomer["customers_cid"];
        $orderData["customers_vat_id"]          = $shopCustomer["customers_vat_id"];
        $orderData["customers_status"]          = $customersStatus["customers_status_id"];
        $orderData["customers_status_name"]     = $customersStatus["customers_status_name"];
        $orderData["customers_status_image"]    = $customersStatus["customers_status_image"];
        $orderData["customers_status_discount"] = 0;

        $orderData["customers_name"]              = $customersAddress['entry_firstname'] .
            " " . $customersAddress['entry_lastname'];
        $orderData["customers_firstname"]         = $customersAddress['entry_firstname'];
        $orderData["customers_lastname"]          = $customersAddress['entry_lastname'];
        $orderData["customers_company"]           = $customersAddress['entry_company'];
        $orderData["customers_street_address"]    = $customersAddress['entry_street_address'];
        $orderData["customers_suburb"]            = $customersAddress['entry_suburb'];
        $orderData["customers_city"]              = $customersAddress['entry_city'];
        $orderData["customers_postcode"]          = $customersAddress['entry_postcode'];
        $orderData["customers_state"]             = $customersStateCode;
        $orderData["customers_country"]           = $customersCountry['countries_name'];
        $orderData["customers_telephone"]         = $shopCustomer['customers_telephone'];
        $orderData["customers_email_address"]     = $shopCustomer['customers_email_address'];
        $orderData["customers_address_format_id"] = $addressFormatCustomer;

        $orderData["delivery_name"]               = $delivery->getFirstName() . " " . $delivery->getLastName();
        $orderData["delivery_firstname"]          = $delivery->getFirstName();
        $orderData["delivery_lastname"]           = $delivery->getLastName();
        $orderData["delivery_company"]            = $delivery->getCompany();
        $orderData["delivery_street_address"]     = $delivery->getStreet1() . (strlen($delivery->getStreet2()) > 0
                ? (' ' . $delivery->getStreet2())
                : '');
        $orderData["delivery_suburb"]             = "";
        $orderData["delivery_city"]               = $delivery->getCity();
        $orderData["delivery_postcode"]           = $delivery->getZipcode();
        $orderData["delivery_state"]              = $deliveryStateCode;
        $orderData["delivery_country"]            = $deliveryCountry["countries_name"];
        $orderData["delivery_country_iso_code_2"] = $delivery->getCountry();
        $orderData["delivery_address_format_id"]  = $addressFormatDelivery;

        $orderData["billing_name"]               = $invoice->getFirstName() . " " . $invoice->getLastName();
        $orderData["billing_firstname"]          = $invoice->getFirstName();
        $orderData["billing_lastname"]           = $invoice->getLastName();
        $orderData["billing_company"]            = $invoice->getCompany();
        $orderData["billing_street_address"]     =
            $invoice->getStreet1() . (strlen($invoice->getStreet2()) > 0 ? (' '
                . $invoice->getStreet2()) : '');
        $orderData["billing_suburb"]             = "";
        $orderData["billing_city"]               = $invoice->getCity();
        $orderData["billing_postcode"]           = $invoice->getZipcode();
        $orderData["billing_state"]              = $invoiceStateCode;
        $orderData["billing_country"]            = $invoiceCountry["countries_name"];
        $orderData["billing_country_iso_code_2"] = $invoice->getCountry();
        $orderData["billing_address_format_id"]  = $addressFormatInvoice;

        // load all languages
        $qry = xtc_db_query("SELECT directory as dir FROM languages as l WHERE l.languages_id = {$this->languageId};");

        $languageDir = null;
        while ($row = xtc_db_fetch_array($qry)) {
            $languageDirectory = $row['dir'];
        }

        $shippingInfos      = $order->getShippingInfos();
        $shippingName       = strtolower($shippingInfos->getName());
        $shippingMethodFile = DIR_WS_INCLUDES . '/modules/shipping/' . $shippingName . '.php';

        if ($order->getShippingType() == self::SHIPPING_TYPE_API
            && file_exists($shippingMethodFile)
        ) {
            $shippingModuleName = $shippingName;
        } else {
            $shippingModuleName = $this->config->getShipping();
        }

        if (!$shippingModuleName) {
            $shippingModuleName = 'flat';
        }

        require_once(DIR_FS_LANGUAGES . $languageDirectory . '/modules/shipping/' . $shippingModuleName . '.php');
        $shippingMethod = constant(MODULE_SHIPPING_ . strtoupper($shippingModuleName) . _TEXT_TITLE);
        $shippingClass  = $shippingModuleName . '_' . $shippingModuleName;

        $orderData["shipping_method"] = $shippingMethod;
        $orderData["shipping_class"]  = $shippingClass;

        $orderData["cc_type"]    = "";
        $orderData["cc_owner"]   = "";
        $orderData["cc_number"]  = "";
        $orderData["cc_expires"] = "";
        $orderData["cc_start"]   = "";
        $orderData["cc_issue"]   = "";
        $orderData["cc_cvv"]     = "";
        $orderData["comments"]   = "";

        $orderData["last_modified"]  = date('Y-m-d H:i:s');
        $orderData["date_purchased"] = $order->getCreatedTime('Y-m-d H:i:s');

        $orderData["currency"]       = $order->getCurrency();
        $orderData["currency_value"] = $this->exchangeRate;

        $orderData["account_type"] = "";

        $orderData["payment_method"] = $order->getPaymentMethod();//"shopgate";
        $orderData["payment_class"]  = "shopgate";

        $orderData["customers_ip"] = "";
        $orderData["language"]     = $this->language;

        $orderData["afterbuy_success"] = 0;
        $orderData["afterbuy_id"]      = 0;

        $orderData["refferers_id"]    = 0;
        $orderData["conversion_type"] = "2";

        $orderData["orders_status"] = $this->config->getOrderStatusOpen();

        $orderData["orders_date_finished"] = null;

        /**
         * Add custom variables to order database if columns exist
         */
        $orderModel = new Shopgate_Models_Order(new Shopgate_Helpers_Database());
        foreach (array($delivery, $order, $invoice) as $object) {
            $orderData = array_merge(
                $orderData,
                $orderModel->addCustomFields($object)
            );
        }

        $this->log('db: save order', ShopgateLogger::LOGTYPE_DEBUG);

        // Speichere die Bestellung
        xtc_db_perform(TABLE_ORDERS, $orderData);
        $dbOrderId = xtc_db_insert_id();

        $this->log('db: save', ShopgateLogger::LOGTYPE_DEBUG);

        $ordersShopgateOrder = array(
            "orders_id"                        => $dbOrderId,
            "shopgate_order_number"            => $order->getOrderNumber(),
            "is_paid"                          => $order->getIsPaid(),
            "is_shipping_blocked"              => $order->getIsShippingBlocked(),
            "payment_infos"                    => $this->jsonEncode($paymentInfosUtf8),
            "is_sent_to_shopgate"              => 0,
            'is_cancellation_sent_to_shopgate' => 0,
            'order_data'                       => serialize($order),
            "modified"                         => "now()",
            "created"                          => "now()",
        );

        xtc_db_perform(TABLE_SHOPGATE_ORDERS, $ordersShopgateOrder);

        $this->log('method: _insertStatusHistory() ', ShopgateLogger::LOGTYPE_DEBUG);
        $this->_insertStatusHistory($order, $dbOrderId, $orderData['orders_status']);

        $this->log('method: setOrderPayment() ', ShopgateLogger::LOGTYPE_DEBUG);
        $this->setOrderPayment($order, $dbOrderId, $orderData['orders_status']);

        $this->log('method: insertOrderItems() ', ShopgateLogger::LOGTYPE_DEBUG);
        $this->insertOrderItems($order, $dbOrderId, $orderData['orders_status'], $couponModel);

        $this->log('method: insertOrderTotal() ', ShopgateLogger::LOGTYPE_DEBUG);
        $this->insertOrderTotal($order, $dbOrderId);

        $this->log('db: update order ', ShopgateLogger::LOGTYPE_DEBUG);

        // Save status in order
        $orderUpdateData                  = array();
        $orderUpdateData["orders_status"] = $orderData["orders_status"];
        $orderUpdateData["last_modified"] = date('Y-m-d H:i:s');
        xtc_db_perform(TABLE_ORDERS, $orderUpdateData, "update", "orders_id = {$dbOrderId}");

        $this->log('method: pushOrderToAfterbuy', ShopgateLogger::LOGTYPE_DEBUG);
        $this->pushOrderToAfterbuy($dbOrderId, $order);
        $this->log('method: pushOrderToDreamRobot', ShopgateLogger::LOGTYPE_DEBUG);
        $this->pushOrderToDreamRobot($dbOrderId, $order);

        if ($this->config->getSendOrderConfirmationMail()) {
            $this->sendOrderEmail($dbOrderId, $shopCustomer['customers_id']);
        }

        $this->log('return: end addOrder()', ShopgateLogger::LOGTYPE_DEBUG);

        return array(
            'external_order_id'     => $dbOrderId,
            'external_order_number' => $dbOrderId
        );
    }

    public function updateOrder(ShopgateOrder $order)
    {
        // save UTF-8 payment infos (to build proper json)
        $paymentInfosUtf8 = $order->getPaymentInfos();

        // data needs to be utf-8 decoded for äöüß and the like to be saved correctly
        $order = $order->utf8Decode($this->config->getEncoding());

        $qry     = "
                    SELECT
                        o.*,
                        so.shopgate_order_id,
                        so.shopgate_order_number,
                        so.is_paid,
                        so.is_shipping_blocked,
                        so.payment_infos
                    FROM " . TABLE_ORDERS . " o
                    INNER JOIN " . TABLE_SHOPGATE_ORDERS . " so ON (so.orders_id = o.orders_id)
                    WHERE so.shopgate_order_number = '{$order->getOrderNumber()}'
                    ";
        $result  = xtc_db_query($qry);
        $dbOrder = xtc_db_fetch_array($result);

        if ($dbOrder == false) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_ORDER_NOT_FOUND,
                "Shopgate order number: '{$order->getOrderNumber()}'."
            );
        }

        $errorOrderStatusIsSent                     = false;
        $errorOrderStatusAlreadySet                 = array();
        $statusShoppingsystemOrderIsPaid            = $dbOrder['is_paid'];
        $statusShoppingsystemOrderIsShippingBlocked = $dbOrder['is_shipping_blocked'];
        $status                                     = $dbOrder["orders_status"];

        // check if shipping is already done, then throw at end of method a OrderStatusIsSent - Exception
        if ($status == $this->config->getOrderStatusShipped()
            && ($statusShoppingsystemOrderIsShippingBlocked
                || $order->getIsShippingBlocked())
        ) {
            $errorOrderStatusIsSent = true;
        }

        if ($order->getUpdatePayment() == 1) {

            if (!is_null($statusShoppingsystemOrderIsPaid)
                && $order->getIsPaid() == $statusShoppingsystemOrderIsPaid
                && !is_null($dbOrder['payment_infos'])
                && $dbOrder['payment_infos'] == $this->jsonEncode(
                    $paymentInfosUtf8
                )
            ) {
                $errorOrderStatusAlreadySet[] = 'payment';
            }

            if (!is_null($statusShoppingsystemOrderIsPaid)
                && $order->getIsPaid() == $statusShoppingsystemOrderIsPaid
            ) {
                // do not update is_paid
            } else {

                // Save order status
                $orderStatus                      = array();
                $orderStatus["orders_id"]         = $dbOrder["orders_id"];
                $orderStatus["orders_status_id"]  = $status;
                $orderStatus["date_added"]        = date('Y-m-d H:i:s');
                $orderStatus["customer_notified"] = false;
                if ($order->getIsPaid()) {
                    $orderStatus['comments'] =
                        'Bestellstatus von Shopgate geändert: Zahlung erhalten';
                } else {
                    $orderStatus['comments'] =
                        'Bestellstatus von Shopgate geändert: Zahlung noch nicht erhalten';
                }

                $orderStatus['comments'] = $this->stringFromUtf8(
                    $orderStatus['comments'], $this->config->getEncoding()
                );

                xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $orderStatus);

                // update the shopgate order status information
                $ordersShopgateOrder =
                    array(
                        "is_paid"  => (int)$order->getIsPaid(),
                        "modified" => "now()",
                    );
                xtc_db_perform(
                    TABLE_SHOPGATE_ORDERS, $ordersShopgateOrder, "update",
                    "shopgate_order_id = {$dbOrder['shopgate_order_id']}"
                );

                // update var
                $statusShoppingsystemOrderIsPaid = $order->getIsPaid();

                // Save status in order
                $orderData                  = array();
                $orderData["orders_status"] = $status;
                $orderData["last_modified"] = date('Y-m-d H:i:s');
                xtc_db_perform(
                    TABLE_ORDERS, $orderData, "update",
                    "orders_id = {$dbOrder['orders_id']}"
                );

            }

            // update paymentinfos
            if (!is_null($dbOrder['payment_infos'])
                && $dbOrder['payment_infos'] != $this->jsonEncode(
                    $paymentInfosUtf8
                )
            ) {

                $dbPaymentInfos = $this->jsonDecode($dbOrder['payment_infos'], true);
                $paymentInfos   = $order->getPaymentInfos();
                $histories      = array();

                switch ($order->getPaymentMethod()) {
                    case ShopgateOrder::SHOPGATE:
                    case ShopgateOrder::INVOICE:
                    case ShopgateOrder::COD:
                        break;
                    case ShopgateOrder::PREPAY:

                        if (isset($dbPaymentInfos['purpose'])
                            && $paymentInfos['purpose']
                            != $dbPaymentInfos['purpose']
                        ) {
                            $comments = $this->stringFromUtf8(
                                "Shopgate: Zahlungsinformationen wurden aktualisiert: \n\nDer Kunde wurde angewiesen Ihnen das Geld mit dem Verwendungszweck \"",
                                $this->config->getEncoding()
                            );
                            $comments .= $paymentInfos["purpose"];
                            $comments .= $this->stringFromUtf8(
                                "\" auf Ihr Bankkonto zu überweisen",
                                $this->config->getEncoding()
                            );

                            // Order is not paid yet
                            $histories[] =
                                array(
                                    "orders_id"            => $dbOrder["orders_id"],
                                    "orders_status_id"     => $status,
                                    "date_added"           => date(
                                        'Y-m-d H:i:s'
                                    ), "customer_notified" => false,
                                    "comments"             => xtc_db_prepare_input(
                                        $comments
                                    )
                                );
                        }

                        break;
                    case ShopgateOrder::DEBIT:
                        $qry            = "SELECT * FROM banktransfer b WHERE b.orders_id = '{$dbOrder['orders_id']}'";
                        $result         = xtc_db_query($qry);
                        $dbBanktransfer = xtc_db_fetch_array($result);

                        if (!empty($dbBanktransfer)) {
                            $banktransferData                          = array();
                            $banktransferData["banktransfer_owner"]    = $paymentInfos["bank_account_holder"];
                            $banktransferData["banktransfer_number"]   = $paymentInfos["bank_account_number"];
                            $banktransferData["banktransfer_bankname"] = $paymentInfos["bank_name"];
                            $banktransferData["banktransfer_blz"]      = $paymentInfos["bank_code"];
                            xtc_db_perform(
                                "banktransfer", $banktransferData, "update", "orders_id = {$dbOrder['orders_id']}"
                            );

                            $comments = $this->stringFromUtf8(
                                "Shopgate: Zahlungsinformationen wurden aktualisiert: \n\n",
                                $this->config->getEncoding()
                            );
                            $comments .= $this->createPaymentInfos(
                                $paymentInfos, $dbOrder['orders_id'], $status,
                                false
                            );

                            $histories[] =
                                array(
                                    "orders_id"         => $dbOrder["orders_id"],
                                    "orders_status_id"  => $status,
                                    "date_added"        => date('Y-m-d H:i:s'),
                                    "customer_notified" => false,
                                    "comments"          => xtc_db_prepare_input($comments)
                                );
                        }

                        break;
                    case ShopgateOrder::PAYPAL:

                        // Save paymentinfos in history
                        $history             = $this->createPaymentInfos(
                            $paymentInfos, $dbOrder["orders_id"], $status
                        );
                        $history['comments'] = $this->stringFromUtf8(
                                "Shopgate: Zahlungsinformationen wurden aktualisiert: \n\n",
                                $this->config->getEncoding()
                            ) . $history['comments'];
                        $histories[]         = $history;

                        break;
                    default:
                        // mobile_payment

                        // Save paymentinfos in history
                        $history             = $this->createPaymentInfos(
                            $paymentInfos, $dbOrder["orders_id"], $status
                        );
                        $history['comments'] = $this->stringFromUtf8(
                                "Shopgate: Zahlungsinformationen wurden aktualisiert: \n\n",
                                $this->config->getEncoding()
                            ) . $history['comments'];
                        $histories[]         = $history;

                        break;
                }

                foreach ($histories as $history) {
                    xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $history);
                }
            }

            $ordersShopgateOrder =
                array(
                    "payment_infos" => $this->jsonEncode($paymentInfosUtf8),
                    "modified"      => "now()",
                );
            xtc_db_perform(
                TABLE_SHOPGATE_ORDERS, $ordersShopgateOrder, "update",
                "shopgate_order_id = {$dbOrder['shopgate_order_id']}"
            );

        }

        if ($order->getUpdateShipping() == 1) {

            if (!is_null($statusShoppingsystemOrderIsShippingBlocked)
                && $order->getIsShippingBlocked()
                == $statusShoppingsystemOrderIsShippingBlocked
            ) {
                $errorOrderStatusAlreadySet[] = 'shipping';
            } else {
                if ($status != $this->config->getOrderStatusShipped()) {
                    if ($order->getIsShippingBlocked() == 1) {
                        $status =
                            $this->config->getOrderStatusShippingBlocked();
                    } else {
                        $status = $this->config->getOrderStatusOpen();
                    }
                }

                $orderStatus                      = array();
                $orderStatus["orders_id"]         = $dbOrder["orders_id"];
                $orderStatus["date_added"]        = date('Y-m-d H:i:s');
                $orderStatus["customer_notified"] = false;
                $orderStatus['orders_status_id']  = $status;
                if ($order->getIsShippingBlocked() == 0) {
                    $orderStatus["comments"] =
                        "Bestellstatus von Shopgate geändert: Versand ist nicht mehr blockiert!";
                } else {
                    $orderStatus['comments'] =
                        'Bestellstatus von Shopgate geändert: Versand ist blockiert!';
                }

                $orderStatus['comments'] = $this->stringFromUtf8(
                    $orderStatus['comments'], $this->config->getEncoding()
                );

                xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $orderStatus);

                $ordersShopgateOrder = array(
                    "is_shipping_blocked" => (int)$order->getIsShippingBlocked(), "modified" => "now()",
                );
                xtc_db_perform(
                    TABLE_SHOPGATE_ORDERS, $ordersShopgateOrder, "update",
                    "shopgate_order_id = {$dbOrder['shopgate_order_id']}"
                );

                $statusShoppingsystemOrderIsShippingBlocked = $order->getIsShippingBlocked();

                // Save status in order
                $orderData                  = array();
                $orderData["orders_status"] = $status;
                $orderData["last_modified"] = date('Y-m-d H:i:s');
                xtc_db_perform(TABLE_ORDERS, $orderData, "update", "orders_id = {$dbOrder['orders_id']}");

                $this->pushOrderToAfterbuy($dbOrder["orders_id"], $order);
                $this->pushOrderToDreamRobot($dbOrder["orders_id"], $order);
            }
        }

        if ($errorOrderStatusIsSent) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_ORDER_STATUS_IS_SENT
            );
        }

        if (!empty($errorOrderStatusAlreadySet)) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_ORDER_ALREADY_UP_TO_DATE,
                implode(',', $errorOrderStatusAlreadySet), true
            );
        }

        return array(
            'external_order_id'     => $dbOrder["orders_id"],
            'external_order_number' => $dbOrder["orders_id"]
        );
    }


    protected function createReviewsCsv()
    {
        $this->buildReviews('csv', $this->exportLimit, $this->exportOffset);
    }

    public function getSettings()
    {
        $customerModel  = new Shopgate_Models_Customer($this->config, $this->languageId);
        $customerGroups = $customerModel->getCustomerGroups();

        // Tax rates are pretty much a combination of tax rules and tax rates in osCommerce. So we're using them to generate both:
        $oscTaxRates   = $this->getTaxRates();
        $customerTaxClass = array(
            'id'         => "1",
            'key'        => 'default',
            'is_default' => "1");
        $taxRates      = array();
        $taxRules      = array();
        foreach ($oscTaxRates as $oscTaxRate) {
            // build and append tax rate
            $taxRates[] = array(
                'id'            => $oscTaxRate['countries_iso_code_2'] . '-' . $oscTaxRate['tax_rates_id'],
                'key'           => $oscTaxRate['countries_iso_code_2'] . '-' . $oscTaxRate['tax_rates_id'],
                'display_name'  => $oscTaxRate['tax_description'],
                'tax_percent'   => $oscTaxRate['tax_rate'],
                'country'       => $oscTaxRate['countries_iso_code_2'],
                'state'         => (!empty($oscTaxRate['countries_iso_code_2']) && !empty($oscTaxRate['zone_code']))
                    ? ShopgateXtcMapper::getShopgateStateCode(
                        $oscTaxRate['countries_iso_code_2'], $oscTaxRate['zone_code']
                    )
                    : '',
                'zip_code_type' => 'all',
            );

            // build and append tax rule
            if (!empty($taxRules[$oscTaxRate['tax_rates_id']])) {
                $taxRules[$oscTaxRate['tax_rates_id']]['tax_rates'][] = array(
                    // one rate per rule (since rates are in fact also rules) in xtModified
                    'id'  => $oscTaxRate['countries_iso_code_2'] . '-' . $oscTaxRate['tax_rates_id'],
                    'key' => $oscTaxRate['countries_iso_code_2'] . '-' . $oscTaxRate['tax_rates_id'],
                );
            } else {
                $ruleName = !empty($oscTaxRate['tax_description'])
                    ? $oscTaxRate['tax_description']
                    : sprintf('%f%%', $oscTaxRate['tax_percent']);

                $taxRules[$oscTaxRate['tax_rates_id']] = array(
                    'id'                   => $oscTaxRate['tax_rates_id'],
                    'name'                 => $ruleName,
                    'priority'             => $oscTaxRate['tax_priority'],
                    'product_tax_classes'  => array(
                        array(
                            'id'  => $oscTaxRate['tax_class_id'],
                            'key' => $oscTaxRate['tax_class_title'],
                        )
                    ),
                    'customer_tax_classes' => array(
                        array(
                            'id' => 1,
                            'key' => 'default'
                        )
                    ),
                    'tax_rates'            => array(
                        array(
                            'id'  => $oscTaxRate['countries_iso_code_2'] . '-' . $oscTaxRate['tax_rates_id'],
                            'key' => $oscTaxRate['countries_iso_code_2'] . '-' . $oscTaxRate['tax_rates_id'],
                        )
                    ),
                );
            }
        }

        return array(
            'customer_groups' => $customerGroups,
            'tax'             => array(
                'product_tax_classes'  => $this->getTaxClasses(),
                'customer_tax_classes' => array($customerTaxClass),
                'tax_rates'            => $taxRates,
                'tax_rules'            => $taxRules,
            ),
        );
    }

    public function createShopInfo()
    {
        $shopInfo = array();

        $productCountQuery      = "SELECT count(*) cnt FROM " . TABLE_PRODUCTS
            . " as p WHERE p.products_status = 1";
        $result                 = xtc_db_query($productCountQuery);
        $row                    = xtc_db_fetch_array($result);
        $shopInfo['item_count'] = $row['cnt'];

        $catQry                     =
            "SELECT count(*) cnt FROM " . TABLE_CATEGORIES;
        $result                     = xtc_db_query($catQry);
        $row                        = xtc_db_fetch_array($result);
        $shopInfo['category_count'] = $row['cnt'];

        $revQry                   =
            "SELECT COUNT(*) AS cnt FROM " . TABLE_REVIEWS;
        $result                   = xtc_db_query($revQry);
        $row                      = xtc_db_fetch_array($result);
        $shopInfo['review_count'] = $row['cnt'];

        // Not provided by Osc
        $shopInfo['plugins_installed '] = array();

        return $shopInfo;
    }

    public function checkCart(ShopgateCart $cart)
    {
        $customerModel = new Shopgate_Models_Customer($this->config, $this->languageId);

        $result['shipping_methods'] = $this->getShipping($cart);
        $result['currency']         = $this->config->getCurrency();
        $result['external_coupons'] = $this->checkCoupons($cart);
        $result['items']            = $this->checkCartItems($cart);
        $result['customer']         = $customerModel->getCartCustomer($cart->getExternalCustomerId());

        return $result;
    }

    /**
     * Check the validity of coupons
     *
     * @param ShopgateCart $cart
     *
     * @return array
     */
    public function checkCoupons(ShopgateCart $cart)
    {
        $result = array();
        if (!defined('MODULE_ORDER_TOTAL_COUPON_STATUS') || MODULE_ORDER_TOTAL_COUPON_STATUS !== 'true') {
            foreach ($cart->getExternalCoupons() as $coupon) {
                $coupon->setIsValid(false);
                $result[] = $coupon;
            }

            return $result;
        }

        $couponModel   = new Shopgate_Models_Coupon(
            $this->currency['code'],
            $this->config,
            $this->languageId,
            $this->language,
            $this->countryId
        );
        $cartItemModel = new Shopgate_Models_Item_CartItem($this->languageId);
        $orderAmount   = $cartItemModel->getCompleteAmount($cart);

        foreach ($cart->getExternalCoupons() as $sgCoupon) {
            $coupon = $couponModel->getCouponByCode($sgCoupon->getCode());

            if (empty($coupon)) {
                $sgCoupon->setNotValidMessage(ShopgateLibraryException::COUPON_CODE_NOT_VALID);
                $sgCoupon->setIsValid(false);
                $result[] = $sgCoupon;
                continue;
            }

            $validationResult = $couponModel->validateCoupon($cart, $cartItemModel, $coupon, $orderAmount);
            if (!empty($validationResult)) {
                $sgCoupon->setNotValidMessage($validationResult);
                $sgCoupon->setIsValid(false);
            } else {
                $sgCoupon->setIsValid(true);
                $couponModel->setCouponData($coupon, $sgCoupon, $cart, $cartItemModel, $orderAmount);
            }
            $result[] = $sgCoupon;
        }

        $reverse          = array_reverse($result);
        $validCouponFound = false;
        /** @var ShopgateExternalCoupon $cp */
        foreach ($reverse as $cp) {

            if ($cp->getIsValid()) {
                if (!$validCouponFound) {
                    $validCouponFound = true;
                } else {
                    $cp->setIsValid(false);
                }
            }
        }

        return $result;
    }

    public function redeemCoupons(ShopgateCart $cart)
    {
    }

    public function cron($jobname, $params, &$message, &$errorcount)
    {
        switch ($jobname) {
            case 'set_shipping_completed':
                $this->cronSetOrdersShippingCompleted($message, $errorcount);
                break;
            case 'cancel_orders':
                $this->cronCancelOrder($jobname, $message, $errorcount);
                break;
            default:
                throw new ShopgateLibraryException(
                    ShopgateLibraryException::PLUGIN_CRON_UNSUPPORTED_JOB,
                    'Job name: "' . $jobname . '"', true
                );
        }
    }

    protected function createCategories($limit = null, $offset = null, array $uids = array())
    {
        $this->log("Start export categories tree...", ShopgateLogger::LOGTYPE_DEBUG);

        if ($this->config->getReverseCategoriesSortOrder()) {
            $maxOrder = 0;
        } else {
            $qry      = "SELECT MAX( sort_order ) sort_order FROM " . TABLE_CATEGORIES;
            $result   = xtc_db_query($qry);
            $maxOrder = xtc_db_fetch_array($result);
            $maxOrder = $maxOrder["sort_order"] + 1;
        }

        $this->buildCategoriesTree(0, $maxOrder, 'xml', $limit, $offset, $uids);

        if ($this->config->getExportNewProductsCategory()) {
            $row                    = $this->buildDefaultCategoryRow();
            $row['parent_id']       = '';
            $row['category_number'] = $this->config->getExportNewProductsCategoryId();
            $row['category_name']   = 'Neue Produkte';
            $row["is_active"]       = 1;
            $row['url_deeplink']    = xtc_href_link('products_new.php');

            $this->addCategoryRow($row);
        }

    }

    public function getOrders(
        $customerToken, $customerLanguage, $limit = 10, $offset = 0, $orderDateFrom = '', $sortOrder = 'created_desc'
    ) {
        $customerOrderModel = new Shopgate_Models_Customer_Order($customerToken, $this->config, $this->languageId);

        return $customerOrderModel->getOrders($customerLanguage, $limit, $offset, $orderDateFrom, $sortOrder);
    }

    public function syncFavouriteList($customerToken, $items)
    {
        // TODO: Implement syncFavouriteList() method.
    }

    public function checkStock(ShopgateCart $cart)
    {
        $includeXtcFiles = array(
            "/inc/xtc_get_tax_class_id.inc.php",
            "/inc/xtc_get_products_stock.inc.php",
        );

        foreach ($includeXtcFiles as $xtcFile) {
            $file = rtrim(DIR_FS_CATALOG, "/") . $xtcFile;
            if (file_exists($file)) {
                include_once $file;
            }
        }

        $returnProducts = array();
        $itemModel      = new Shopgate_Models_Item($this->config, $this->languageId, $this->exchangeRate);
        $cartItemModel  = new Shopgate_Models_Item_CartItem($this->languageId);

        foreach ($cart->getItems() AS $product) {

            $sgProduct    = new ShopgateCartItem();
            $sgOrderInfos = $this->jsonDecode($product->getInternalOrderInfo(), true);
            $productId    = $cartItemModel->getProductIdFromCartItem($product);
            $dbproduct    = $itemModel->getProductById($productId);

            if (empty($dbproduct)) {
                $sgProduct->setIsBuyable(false);
                $sgProduct->setError(ShopgateLibraryException::CART_ITEM_PRODUCT_NOT_FOUND);
                $returnProducts[] = $sgProduct;
                continue;
            }

            $sgProduct->setItemNumber($productId);
            $sgProduct->setIsBuyable(true);
            $quantitiy = xtc_get_products_stock($productId);

            if (!empty($dbproduct["specials_new_products_price"])
                && (STOCK_CHECK == 'true' && STOCK_ALLOW_CHECKOUT == 'false')
                && ($dbproduct["specials_quantity"] > 0)
            ) {
                // Nur wenn die quantity > 0 ist dann specialprice setzen,
                // ansonsten normalen Preis mit normalem Stock
                $quantitiy =
                    $dbproduct["specials_quantity"] > $quantitiy ? $quantitiy : $dbproduct["specials_quantity"];
            }

            $sgProduct->setStockQuantity($quantitiy);
            $sgProduct->setQtyBuyable($quantitiy);

            if (($product->getQuantity() > $quantitiy || $quantitiy == 0)
                && (STOCK_CHECK == 'true' && STOCK_ALLOW_CHECKOUT == 'false')
            ) {
                $sgProduct->setIsBuyable(false);
                $sgProduct->setError(
                    ShopgateLibraryException::CART_ITEM_OUT_OF_STOCK
                );
            }

            $attributeIds = array();
            foreach ($sgOrderInfos as $infoName => $infoValue) {
                if (strpos($infoName, "attribute_") === 0) {
                    if (is_array($infoValue)) {
                        foreach ($infoValue as $attributeKey => $attributeArray) {
                            $attributeIds[] = array_merge(
                                array("products_attributes_id" => $attributeKey),
                                $attributeArray
                            );
                        }
                    }
                }
            }

            //tax
            $productTaxClassId = xtc_get_tax_class_id($productId);
            $productTaxRate    = xtc_get_tax_rate($productTaxClassId, $this->countryId, $this->zoneId);

            $sgProduct->setOptions($cartItemModel->getOptionsToProduct($productId, $attributeIds, $productTaxRate));
            $sgProduct->setAttributes($cartItemModel->getAttributesToProduct($product, $sgProduct));
            //not supported by xtcommerce3
            $sgProduct->setInputs(array());
            $returnProducts[] = $sgProduct;
        }

        return $returnProducts;
    }

    protected function createReviews($limit = null, $offset = null, array $uids = array())
    {
        $this->buildReviews('xml', $limit, $offset, $uids);
    }

    /**
     * read all category data from the database an prepare data for export
     * - the $type variable defines the type of the export format
     *    - xml
     *    - csv
     *
     * @param int    $parentId
     * @param int    $maxOrder
     * @param string $type
     * @param null   $limit
     * @param null   $offset
     * @param array  $uids
     */
    private function buildCategoriesTree(
        $parentId = 0, $maxOrder = 0, $type = "csv", $limit = null, $offset = null, array $uids = array()
    ) {

        $this->log(
            "Start buldiding Categories tree: parent_id = " . $parentId . "...",
            ShopgateLogger::LOGTYPE_DEBUG
        );

        $qry = "SELECT DISTINCT
                    c.categories_id,
                    c.parent_id,
                    c.categories_image,
                    c.categories_status,
                    c.sort_order,
                    cd.categories_name
                FROM " . TABLE_CATEGORIES . " c
                LEFT JOIN " . TABLE_CATEGORIES_DESCRIPTION . " cd ON (c.categories_id = cd.categories_id
                AND cd.language_id = $this->languageId)";

        if (!($uids == array())) {
            $qry .= " WHERE c.categories_id IN (" . implode(',', $uids) . ")";
        }

        $qry .= " ORDER BY c.categories_id ASC";

        if (is_numeric($limit) && is_numeric($offset)) {
            $qry .= " LIMIT " . $offset . "," . $limit . "";
        }

        $qry = xtc_db_query($qry);

        $categoryModel = new Shopgate_Models_Catalog_Category($this->config);

        while ($item = xtc_db_fetch_array($qry)) {

            $this->log("cheking if category is blacklisted ...", ShopgateLogger::LOGTYPE_DEBUG);

            if ($categoryModel->isCategoryBlacklisted($item["categories_id"])) {
                continue;
            }

            $row = $this->buildDefaultCategoryRow();

            $row["category_number"] = $item["categories_id"];
            $row["parent_id"]       = (empty($item["parent_id"]) || ($item['parent_id'] == $item['categories_id']))
                ? ""
                : $item["parent_id"];
            $row["category_name"]   =
                htmlentities($item["categories_name"], ENT_NOQUOTES, $this->config->getEncoding());

            if (!empty($item["categories_image"])) {
                $row["url_image"] =
                    HTTP_SERVER . DIR_WS_CATALOG . DIR_WS_IMAGES . "categories/" . $item["categories_image"];
            }

            if (!empty($item["sort_order"]) || ((string)$item['sort_order'] === '0')) {
                if ($this->config->getReverseCategoriesSortOrder()) {
                    // reversed means the contrary to ordering system in shopgate - order_index is a priority system - high number = top position
                    // so just taking over the values means reversing the order
                    $row["order_index"] = $item["sort_order"];
                } else {
                    $row["order_index"] = $maxOrder - $item["sort_order"];
                }
            }

            $row["is_active"]    = $item["categories_status"];
            $row['url_deeplink'] = xtc_href_link(
                FILENAME_DEFAULT,
                xtc_category_link($item['categories_id'], $item['categories_name']),
                'NONSSL',
                false
            );

            if ($type == 'csv') {
                $this->addCategoryRow($row);
            } else {
                $node = new Shopgate_Models_Catalog_CategoryXml($this->config);
                $node->setItem($row);
                $this->addCategoryModel($node->generateData());
            }
        }
    }


    /**
     * read all review data from the database an prepare data for export
     * - the $type variable defines the type of the export format
     *    - xml
     *    - csv
     *
     * @param string $type
     * @param null   $limit
     * @param null   $offset
     * @param array  $uids
     */
    private function buildReviews($type = "csv", $limit = null, $offset = null, array $uids = array())
    {
        $qry = "
        SELECT
            r.reviews_id,
            r.products_id,
            r.customers_name,
            r.reviews_rating,
            r.date_added,
            rd.reviews_text
        FROM
        " . TABLE_REVIEWS . " as r
        INNER JOIN
        " . TABLE_REVIEWS_DESCRIPTION . " as rd ON r.reviews_id = rd.reviews_id
        WHERE rd.languages_id = '" . $this->languageId . "'";

        if (!($uids == array())) {
            $qry .= " AND r.reviews_id IN (" . implode(',', $uids) . ")";
        }

        $qry .= " ORDER BY r.reviews_id ASC";

        if (is_numeric($limit) && is_numeric($offset)) {
            $qry .= " LIMIT " . $offset . "," . $limit . "";
        }

        $qry = xtc_db_query($qry);
        while ($entry = xtc_db_fetch_array($qry)) {
            $review = $this->buildDefaultReviewRow();

            if ($type == 'csv') {
                $review['item_number']      = $entry['products_id'];
                $review['update_review_id'] = $entry['reviews_id'];
                $review['score']            = $entry['reviews_rating'] * 2;
                $review['name']             = $entry['customers_name'];
                $review['date']             = $entry['date_added'];
                $review['title']            = '';
                $review['text']             = $entry['reviews_text'];
                $this->addReviewRow($review);
            } else {
                $node = new Shopgate_Models_Review_Xml($this->config);
                $node->setItem($entry);
                $this->addReviewModel($node->generateData());
            }
        }
    }

    /**
     * creates a guest customer in the shop system
     *
     * @param ShopgateOrder $order
     *
     * @return array
     */
    private function createGuestUser(ShopgateOrder $order)
    {
        //        $order = new ShopgateOrder();
        $address = $order->getInvoiceAddress();

        $customerStatus = $this->config->getCustomersStatusId();
        if ($customerStatus === -1) {
            $customerStatus = DEFAULT_CUSTOMERS_STATUS_ID;
        }

        $customer                                 = array();
        $customer["customers_vat_id_status"]      = 0;
        $customer["customers_status"]             = $customerStatus;
        $customer["customers_gender"]             = $address->getGender();
        $customer["customers_firstname"]          = $address->getFirstName();
        $customer["customers_lastname"]           = $address->getLastName();
        $customer["customers_email_address"]      = $order->getMail();
        $customer["customers_default_address_id"] = "";
        $customer["customers_telephone"]          = $order->getPhone();
        $customer["customers_fax"]                = "";
        $customer["customers_newsletter"]         = 0;
        $customer["customers_newsletter_mode"]    = 0;
        $customer["member_flag"]                  = 0;
        $customer["delete_user"]                  = 1;
        $customer["account_type"]                 = 1;
        $customer["refferers_id"]                 = 0;
        $customer["customers_date_added"]         = date('Y-m-d H:i:s');
        $customer["customers_last_modified"]      = date('Y-m-d H:i:s');

        xtc_db_perform(TABLE_CUSTOMERS, $customer);
        $customerId = xtc_db_insert_id();

        $qry     = "SELECT countries_id FROM " . TABLE_COUNTRIES
            . " WHERE UPPER(countries_iso_code_2) = UPPER('"
            . $address->getCountry() . "')";
        $qry     = xtc_db_query($qry);
        $country = xtc_db_fetch_array($qry);

        if (empty($country)) {
            $country = array('countries_id' => 81,);
        }

        $qry  = "SELECT zone_id, zone_name FROM " . TABLE_ZONES
            . " WHERE zone_country_id = {$country['countries_id']} AND zone_code = '"
            . ShopgateXtcMapper::getXtcStateCode($address->getState()) . "'";
        $qry  = xtc_db_query($qry);
        $zone = xtc_db_fetch_array($qry);

        if (empty($zone)) {
            $zone = array('zone_id' => null, 'zone_name' => $address->getState());
        }

        $_address = array(
            "customers_id"          => $customerId,
            "entry_gender"          => $address->getGender(),
            "entry_company"         => $address->getCompany(),
            "entry_firstname"       => $address->getFirstName(),
            "entry_lastname"        => $address->getLastName(),
            "entry_street_address"  => $address->getStreet1()
                . (strlen($address->getStreet2()) > 0 ? (' '
                    . $address->getStreet2()) : ''),
            "entry_suburb"          => "",
            "entry_postcode"        => $address->getZipcode(),
            "entry_city"            => $address->getCity(),
            "entry_state"           => $zone['zone_name'],
            "entry_country_id"      => $country["countries_id"],
            "entry_zone_id"         => $zone['zone_id'],
            "address_date_added"    => date('Y-m-d H:i:s'),
            "address_last_modified" => date('Y-m-d H:i:s'),
        );
        xtc_db_perform(TABLE_ADDRESS_BOOK, $_address);
        $addressId = xtc_db_insert_id();

        $customer = array("customers_default_address_id" => $addressId);
        xtc_db_perform(TABLE_CUSTOMERS, $customer, "update", "customers_id = $customerId");

        $_info = array(
            "customers_info_id"                         => $customerId,
            "customers_info_date_of_last_logon"         => date('Y-m-d H:i:s'),
            "customers_info_number_of_logons"           => '1',
            "customers_info_date_account_created"       => date('Y-m-d H:i:s'),
            "customers_info_date_account_last_modified" => date('Y-m-d H:i:s'),
            "global_product_notifications"              => 0
        );
        xtc_db_perform(TABLE_CUSTOMERS_INFO, $_info);

        $customerMemo                 = array();
        $customerMemo["customers_id"] = $customerId;
        $customerMemo["memo_date"]    = date('Y-m-d');
        $customerMemo["memo_title"]   = "Shopgate - Account angelegt";
        $customerMemo["memo_text"]    = "Account wurde von Shopgate angelegt";
        $customerMemo["poster_id"]    = null;

        xtc_db_perform("customers_memo", $customerMemo);

        $result   = xtc_db_query("SELECT * FROM " . TABLE_CUSTOMERS . " WHERE customers_id = " . $customerId);
        $customer = xtc_db_fetch_array($result);

        return $customer;
    }

    /**
     * read address format id from the database by the iso-2 code
     *
     * @param string $isoCode2
     * @param null   $countryId
     *
     * @return mixed
     */
    private function getAddressFormatId($isoCode2 = 'DE', $countryId = null)
    {
        $isoCode2 = strtoupper($isoCode2);
        if (!empty($countryId)) {
            $qry = "
                SELECT c.address_format_id
                FROM " . TABLE_COUNTRIES . " c
                WHERE c.countries_id = '$countryId'
            ";
        } else {
            $qry = "
                SELECT c.address_format_id
                FROM " . TABLE_COUNTRIES . " c
                WHERE UPPER(c.countries_iso_code_2) = '$isoCode2'
            ";
        }

        $result = xtc_db_query($qry);
        $item   = xtc_db_fetch_array($result);

        return $item["address_format_id"];
    }

    /**
     * Save information (comments) to an order.
     * Custom fields order, Custom fields addresses, test order, status shipping blocked, status invoice blocked
     *
     * @param ShopgateOrder $order
     * @param               $dbOrderId
     * @param               $currentOrderStatus
     */
    private function _insertStatusHistory(ShopgateOrder $order, $dbOrderId, &$currentOrderStatus)
    {
        $comment = "";
        $comment .= ($order->getIsTest()) ? SHOPGATE_ORDER_TEST_ORDER . "\n"
            : "";
        $comment .= SHOPGATE_ORDER_ADDED_COMMENT . "\n" . SHOPGATE_ORDER_ORDER_NUMBER . $order->getOrderNumber() . "\n";

        $paymentTransactionNumber = $order->getPaymentTransactionNumber();

        $comment .= (!empty($paymentTransactionNumber))
            ? SHOPGATE_ORDER_PAYMENT_TRANSACTION_NUMBER . " " . $paymentTransactionNumber . "\n"
            : "";

        $comment .= ($order->getIsShippingBlocked() == 0)
            ? SHOPGATE_ORDER_SHIPPING_NOT_BLOCKED . "\n"
            : SHOPGATE_ORDER_SHIPPING_BLOCKED . "\n";

        $comment .= ($order->getIsCustomerInvoiceBlocked())
            ? SHOPGATE_ORDER_CUSTOMER_INVOICE_BLOCKED . "\n"
            : "";

        $comment .= (count($order->getCustomFields()) > 0)
            ? $this->generateCustomFieldsComment(
                $order, SHOPGATE_ORDER_CUSTOM_FIELD
            ) : "";

        $invoiceAddress = $order->getInvoiceAddress();
        $comment .= (!empty($invoiceAddress))
            ? $this->generateCustomFieldsComment($invoiceAddress, SHOPGATE_ORDER_INVOICE_CUSTOM_FIELD)
            : "";

        $deliveryAddress = $order->getDeliveryAddress();
        $comment .= (!empty($deliveryAddress))
            ? $this->generateCustomFieldsComment($deliveryAddress, SHOPGATE_ORDER_DELIVERY_CUSTOM_FIELD)
            : "";

        $comment = $this->stringFromUtf8($comment, $this->config->getEncoding());

        $histories = array(
            array(
                "orders_id"         => $dbOrderId,
                "orders_status_id"  => ($order->getIsShippingBlocked() == 0)
                    ? $currentOrderStatus
                    : $this->config->getOrderStatusShippingBlocked(),
                "date_added"        => date('Y-m-d H:i:s'),
                "customer_notified" => false,
                "comments"          => xtc_db_prepare_input($comment),
            )
        );

        foreach ($histories as $history) {
            xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $history);
        }
    }

    /**
     * generates a comment from custom fields
     *
     * @param ShopgateOrder|ShopgateAddress $object
     * @param string                        $title
     *
     * @return string
     */
    public function generateCustomFieldsComment($object, $title = "")
    {
        if (empty($object)) {
            return "";
        }

        $comment      = "";
        $customFields = $object->getCustomFields();
        $isFirstItem  = true;
        foreach ($customFields as $customField) {
            $comment .=
                ((!empty($title) && $isFirstItem)
                    ? ($title . "\n")
                    : "") .
                $customField->getLabel() . ": " . $customField->getValue() . "\n";

            $isFirstItem = false;
        }

        return $comment;
    }


    /**
     * Store the payment information to an order in the database
     *
     * @param ShopgateOrder $order
     * @param               $dbOrderId
     * @param               $currentOrderStatus
     */
    private function setOrderPayment(ShopgateOrder $order, $dbOrderId, &$currentOrderStatus)
    {
        $factory      = new Shopgate_Helpers_Factory();
        $paymentClass = $factory->getPayment($order);
        $payment      = $order->getPaymentMethod();
        $histories    = array();

        $paymentName      = '';
        $paymentWasMapped = false;
        $paymentMapping   = $this->config->getTranslatedPaymentMappings();

        $this->log('methods to map: ' . print_r($paymentMapping, 1), ShopgateLogger::LOGTYPE_DEBUG);
        $this->log('current payment: ' . $payment, ShopgateLogger::LOGTYPE_DEBUG);
        if (isset($paymentMapping[$payment])) {
            $comments         = $this->stringFromUtf8(
                "Zahlungsweise '" . $payment . "' durch '" . $paymentMapping[$payment] . "' ersetzt",
                $this->config->getEncoding()
            );
            $histories[]      = array(
                'orders_id'         => $dbOrderId,
                'orders_status_id'  => $currentOrderStatus,
                'date_added'        => date('Y-m-d H:i:s'),
                'customer_notified' => false,
                'comments'          => xtc_db_prepare_input($comments)
            );
            $paymentName      = $paymentMapping[$payment];
            $paymentWasMapped = true;
        }

        $moduleName                  = $paymentClass->getModuleName();
        $orderData['payment_method'] = ($paymentWasMapped) ? $paymentName : $moduleName;
        $orderData['payment_class']  = $moduleName === 'mobile_payment' ? 'shopgate' : $moduleName;
        $comments                    = $paymentClass->getHistoryComments();
        $paymentClass->saveAdditionalInfo($dbOrderId, array());

        if ($comments) {
            $histories[] = array(
                'orders_id'         => $dbOrderId,
                'orders_status_id'  => $currentOrderStatus,
                'date_added'        => date('Y-m-d H:i:s'),
                'customer_notified' => false,
                'comments'          => xtc_db_prepare_input(
                    $this->stringFromUtf8($comments, $this->config->getEncoding())
                )
            );
        }

        foreach ($histories as $history) {
            xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $history);
        }

        xtc_db_perform(TABLE_ORDERS, $orderData, "update", "orders_id = {$dbOrderId}");
    }

    /**
     * Parse the paymentInfo - array and get as output a array or a string
     *
     * @param Array   $paymentInfos
     * @param Integer $dbOrderId
     * @param Integer $currentOrderStatus
     *
     * @return mixed History-Array or String
     */
    private function createPaymentInfos($paymentInfos, $dbOrderId, $currentOrderStatus, $asArray = true) {
        $paymentInformation = '';
        foreach ($paymentInfos as $key => $value) {
            $paymentInformation .= $key . ': ' . $value . "\n";
        }

        if ($asArray) {
            return array(
                "orders_id"         => $dbOrderId,
                "orders_status_id"  => $currentOrderStatus,
                "date_added"        => date('Y-m-d H:i:s'),
                "customer_notified" => false,
                "comments"          => xtc_db_prepare_input(
                    $paymentInformation
                )
            );
        } else {
            return $paymentInformation;
        }
    }

    /**
     * insert all products to an Shopgate order into the database
     *
     * @param ShopgateOrder          $order
     * @param int                    $dbOrderId
     * @param int                    $currentOrderStatus
     * @param Shopgate_Models_Coupon $couponModel
     */
    private function insertOrderItems(ShopgateOrder $order, $dbOrderId, &$currentOrderStatus, Shopgate_Models_Coupon $couponModel)
    {
        $saveShippingStatus = false;
        $qry                =
            "SHOW COLUMNS FROM `" . TABLE_ORDERS_PRODUCTS . "`";
        $result             = xtc_db_query($qry);
        while ($row = xtc_db_fetch_array($result)) {
            if ($row['Field'] == 'products_shipping_time') {
                $saveShippingStatus = true;
                break;
            }
        }
        // Alle Versandstatus abrufen
        $shippingStatus = array();
        if ($saveShippingStatus) {
            $qry    = "SELECT * FROM `" . TABLE_SHIPPING_STATUS
                . "` where `language_id` = '" . $this->languageId . "'";
            $result = xtc_db_query($qry);
            while ($row = xtc_db_fetch_array($result)) {
                $shippingStatus[$row['shipping_status_id']] = $row;
            }
        }

        $errors = '';
        foreach ($order->getItems() as $orderItem) {

            $order_infos = $orderItem->getInternalOrderInfo();
            $order_infos = $this->jsonDecode($order_infos, true);

            $item_number = $orderItem->getItemNumber();
            if (isset($order_infos["base_item_number"])) {
                $item_number = $order_infos["base_item_number"];
            }

            $this->log('db: get product ', ShopgateLogger::LOGTYPE_DEBUG);

            $qry = xtc_db_query(
                "SELECT * FROM " . TABLE_PRODUCTS . " WHERE"
                . " products_id = '" . $item_number . "'" . " LIMIT 1"
            );

            $dbProduct = xtc_db_fetch_array($qry);
            if (empty($dbProduct) && ($item_number == 'COUPON' || $item_number == 'PAYMENT_FEE')) {

                $this->log('product is COUPON or PAYMENTFEE', ShopgateLogger::LOGTYPE_DEBUG);

                // workaround for shopgate coupons
                $dbProduct                   = array();
                $dbProduct['products_id']    = 0;
                $dbProduct['products_model'] = $item_number;
            } else {
                if (empty($dbProduct)) {
                    $this->log('no product found', ShopgateLogger::LOGTYPE_DEBUG);

                    $this->log(
                        ShopgateLibraryException::buildLogMessageFor(
                            ShopgateLibraryException::PLUGIN_ORDER_ITEM_NOT_FOUND,
                            'Shopgate-Order-Number: ' . $order->getOrderNumber()
                            . ', DB-Order-Id: ' . $dbOrderId
                            . '; item (item_number(id/parentId): ['
                            . $orderItem->getItemNumber() . "/"
                            . $orderItem->getParentItemNumber() . "]"
                            . '). The item will be skipped.'
                        )
                    );
                    $errors .= "\nItem (item_number: " . $item_number
                        . ") can not be found in your shoppingsystem. Please contact Shopgate. "
                        . "The item will be skipped.";

                    $dbProduct['products_id']    = 0;
                    $dbProduct['products_model'] = $item_number;
                }
            }

            $this->log('db: orders_products', ShopgateLogger::LOGTYPE_DEBUG);

            $productData = array(
                "orders_id"         => $dbOrderId,
                "products_model"    => $dbProduct["products_model"],
                "products_id"       => $item_number,
                "products_name"     => xtc_db_prepare_input($orderItem->getName()),
                "products_price"    => $orderItem->getUnitAmountWithTax(), "products_discount_made" => 0,
                "final_price"       => $orderItem->getQuantity() * ($orderItem->getUnitAmountWithTax()),
                "products_tax"      => $orderItem->getTaxPercent(),
                "products_quantity" => $orderItem->getQuantity(), "allow_tax" => 1,
            );

            if ($saveShippingStatus) {
                if (!empty($shippingStatus[$dbProduct['products_shippingtime']]['shipping_status_name'])) {
                    $productData['products_shipping_time'] =
                        $shippingStatus[$dbProduct['products_shippingtime']]['shipping_status_name'];
                }
            }

            $qry             = xtc_db_perform(TABLE_ORDERS_PRODUCTS, $productData);
            $productsOrderId = xtc_db_insert_id();
            $options         = $orderItem->getOptions();

            if (!empty($options)) {

                $this->log('process options', ShopgateLogger::LOGTYPE_DEBUG);

                foreach ($options as $option) {
                    $attribute_model  = $option->getValueNumber();
                    $attribute_number = $option->getOptionNumber();

                    $this->log('db: get attributes', ShopgateLogger::LOGTYPE_DEBUG);

                    $qry = "
                        SELECT
                            po.products_options_name,
                            pov.products_options_values_name,
                            pa.options_values_price,
                            pa.price_prefix
                        FROM " . TABLE_PRODUCTS_ATTRIBUTES . " pa
                        INNER JOIN " . TABLE_PRODUCTS_OPTIONS
                        . " po ON pa.options_id = po.products_options_id AND po.language_id = $this->languageId
                        INNER JOIN "
                        . TABLE_PRODUCTS_OPTIONS_VALUES_TO_PRODUCTS_OPTIONS . " povtpo ON povtpo.products_options_id = po.products_options_id
                        INNER JOIN " . TABLE_PRODUCTS_OPTIONS_VALUES
                        . " pov ON (povtpo.products_options_values_id = pov.products_options_values_id AND pa.options_values_id = pov.products_options_values_id AND pov.language_id = $this->languageId)
                        WHERE pa.products_id = '" . $dbProduct["products_id"]
                        . "'
                        " . (!empty($attribute_number)
                            ? "AND pa.options_id = '{$attribute_number}'
                        " : "")
                        . "AND pa.options_values_id = '{$attribute_model}'
                        LIMIT 1
                    ";

                    $qry         = xtc_db_query($qry);
                    $dbattribute = xtc_db_fetch_array($qry);

                    if (empty($dbattribute)) {
                        continue;
                    }

                    $this->log('db: save order product attributes', ShopgateLogger::LOGTYPE_DEBUG);

                    $productAttributeData = array(
                        "orders_id"               => $dbOrderId,
                        "orders_products_id"      => $productsOrderId,
                        "products_options"        => $dbattribute['products_options_name'],
                        "products_options_values" => $dbattribute["products_options_values_name"],
                        "options_values_price"    => $dbattribute["options_values_price"],
                        "price_prefix"            => $dbattribute["price_prefix"],
                    );

                    xtc_db_perform(TABLE_ORDERS_PRODUCTS_ATTRIBUTES, $productAttributeData);
                }
            } else {

                $this->log('attributes?', ShopgateLogger::LOGTYPE_DEBUG);

                for ($i = 1; $i <= 10; $i++) {
                    if (!isset($order_infos["attribute_$i"])) {
                        break;
                    }
                    $tmpAttr = $order_infos["attribute_$i"];
                    // Code for support of the old internal_order_info structure
                    if (!is_array($tmpAttr)) {
                        $attribute_model = $tmpAttr;
                    } else {
                        // Den ersten und einzigen key nutzen (zur Sicherheit auf den start des Arrays setzen)
                        reset($tmpAttr);
                        $attribute_number = $tmpAttr[key($tmpAttr)]['options_id'];
                        $attribute_model  = $tmpAttr[key($tmpAttr)]['options_values_id'];
                    }

                    $this->log(
                        'db: get attribute', ShopgateLogger::LOGTYPE_DEBUG
                    );

                    // Hole das Attribut aus der Datenbank
                    $qry = "
                        SELECT
                            po.products_options_name,
                            pov.products_options_values_name,
                            pa.options_values_price,
                            pa.price_prefix
                        FROM " . TABLE_PRODUCTS_ATTRIBUTES . " pa
                        INNER JOIN " . TABLE_PRODUCTS_OPTIONS
                        . " po ON pa.options_id = po.products_options_id AND po.language_id = $this->languageId
                        INNER JOIN "
                        . TABLE_PRODUCTS_OPTIONS_VALUES_TO_PRODUCTS_OPTIONS . " povtpo ON povtpo.products_options_id = po.products_options_id
                        INNER JOIN " . TABLE_PRODUCTS_OPTIONS_VALUES
                        . " pov ON (povtpo.products_options_values_id = pov.products_options_values_id AND pa.options_values_id = pov.products_options_values_id AND pov.language_id = $this->languageId)
                        WHERE pa.products_id = '" . $dbProduct["products_id"]
                        . "'
                        "
                        . // Still support the old internal_order_info structure
                        (!empty($attribute_id)
                            ? "AND pa.products_attributes_id = '"
                            . $attribute_model . "'
                        "
                            : "AND pa.options_id = '{$attribute_number}'
                        AND pa.options_values_id = '{$attribute_model}'
                        ") . "LIMIT 1
                        ";

                    $qry         = xtc_db_query($qry);
                    $dbattribute = xtc_db_fetch_array($qry);

                    if (empty($dbattribute)) {
                        continue;
                    }

                    $this->log('db: save order product attributes', ShopgateLogger::LOGTYPE_DEBUG);

                    $productAttributeData = array(
                        "orders_id"               => $dbOrderId,
                        "orders_products_id"      => $productsOrderId,
                        "products_options"        => $dbattribute["products_options_name"],
                        "products_options_values" => $dbattribute["products_options_values_name"],
                        "options_values_price"    => $dbattribute["options_values_price"],
                        "price_prefix"            => $dbattribute["price_prefix"],
                    );
                    xtc_db_perform(TABLE_ORDERS_PRODUCTS_ATTRIBUTES, $productAttributeData);
                }
            }

            $inputfields = $orderItem->getInputs();

            foreach ($inputfields as $inputfield) {
                $productAttributeData = array(
                    "orders_id"               => $dbOrderId,
                    "orders_products_id"      => $productsOrderId,
                    "products_options"        => $inputfield->getLabel(),
                    "products_options_values" => $inputfield->getUserInput(),
                    "options_values_price"    => ($inputfield->getAdditionalAmountWithTax() < 0)
                        ? ($inputfield->getAdditionalAmountWithTax() * -1)
                        : $inputfield->getAdditionalAmountWithTax(),
                    "price_prefix"            => ($inputfield->getAdditionalAmountWithTax() < 0) ? "-" : "+",
                );

                xtc_db_perform(TABLE_ORDERS_PRODUCTS_ATTRIBUTES, $productAttributeData);
            }
        }

        $coupons = $order->getExternalCoupons();
        if (!empty($coupons)) {
            foreach ($coupons as $coupon) {
                $couponModel->redeemCoupon($coupon, $order->getExternalCustomerId(), $dbOrderId);
            }
        }

        $this->log('method: updateItemsStock', ShopgateLogger::LOGTYPE_DEBUG);
        $this->updateItemsStock($order);

        if (!empty($errors)) {
            $this->log('db: save errors in history', ShopgateLogger::LOGTYPE_DEBUG);

            $comments = $this->stringFromUtf8(
                'Es sind Fehler beim Importieren der Bestellung aufgetreten: ',
                $this->config->getEncoding()
            );

            $comments .= $errors;

            $history = array(
                "orders_id"         => $dbOrderId,
                "orders_status_id"  => $currentOrderStatus,
                // "-5" Damit diese Meldung als erstes oben angezeigt wird
                "date_added"        => date("Y-m-d H:i:s", time() - 5),
                "customer_notified" => false,
                "comments"          => xtc_db_prepare_input($comments),
            );

            xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $history);
        }
    }

    /**
     * update the stock quantity for ordered products in the database
     *
     * @param ShopgateOrder $order
     */
    private function updateItemsStock(ShopgateOrder $order)
    {
        foreach ($order->getItems() as $item) {
            // Skip "coupon" and "payment_fee" items
            if ($item->getItemNumber() == 'COUPON' || $item->getItemNumber() == 'PAYMENT_FEE') {
                continue;
            }

            // Attribute ids are set inside the internal order info
            $internalOrderInfo = $this->jsonDecode($item->getInternalOrderInfo(), true);

            $usesProductsAttributes = false;

            // Get id (parent id for child products)
            $productId = $item->getItemNumber();
            if (!empty($internalOrderInfo['base_item_number'])) {
                $productId              = $internalOrderInfo['base_item_number'];
                $usesProductsAttributes = true;
            }

            $itemOptions = $item->getOptions();

            if (!empty($itemOptions)) {
                $usesProductsAttributes = true;
            }

            // Update products stock if reduction enabled
            if (STOCK_LIMITED == 'true') {
                $qry = "
                    UPDATE `" . TABLE_PRODUCTS . "` AS `p`
                        SET `p`.`products_quantity` = `p`.`products_quantity` - {$item->getQuantity(
                    )}
                    WHERE `p`.`products_id` = '{$productId}'
                ;";

                xtc_db_query($qry);

                // Deactivate product if checkout is not allowed and the stock level reaches zero
                if (STOCK_ALLOW_CHECKOUT == 'false') {
                    // commerce:seo has an additional constant that tells if the product may be deactivated (STOCK_ALLOW_CHECKOUT_DEACTIVATE)
                    if (!defined('STOCK_ALLOW_CHECKOUT_DEACTIVATE') || STOCK_ALLOW_CHECKOUT_DEACTIVATE == 'true') {
                        // don't update if defined and not true
                        $qry = "
                            UPDATE `" . TABLE_PRODUCTS . "` AS `p`
                                SET `p`.`products_status` = 0
                            WHERE `p`.`products_id` = '{$productId}' AND `p`.`products_quantity` <= 0
                        ;";
                        xtc_db_query($qry);
                    }
                }
            }

            // Attribute items also need to be reduced in stock
            if ($usesProductsAttributes) {
                // Build additional SQL snippets to update the attributes stock (not using the products_attributes_id because they all change on each update of any attribute in the backend)
                $attributeSQLQueryParts = array();
                if (!empty($internalOrderInfo['base_item_number'])) {
                    for ($i = 1; $i <= 10; $i++) {
                        if (!empty($internalOrderInfo["attribute_{$i}"])) {
                            $tmpAttr = $internalOrderInfo["attribute_{$i}"];
                            if (!is_array($tmpAttr)) {
                                $attributeSQLQueryParts[] =
                                    " ATTRIBUTES_ID='{$tmpAttr}'";
                            } else {
                                // Only the first element is relevant since there can only be one per attribute-number
                                reset($tmpAttr);
                                $attributeSQLQueryParts[] =
                                    'OPTIONS_ID=\'' . $tmpAttr[key(
                                        $tmpAttr
                                    )]['options_id']
                                    . '\' AND OPTIONS_VALUES_ID=\''
                                    . $tmpAttr[key(
                                        $tmpAttr
                                    )]['options_values_id'] . '\'';
                            }
                        }
                    }
                } else {
                    // Attributes was exported as options
                    foreach ($itemOptions as $itemOption) {
                        $attributeSQLQueryParts[] =
                            'OPTIONS_ID=\'' . $itemOption->getOptionNumber()
                            . '\' AND OPTIONS_VALUES_ID=\''
                            . $itemOption->getValueNumber() . '\'';
                    }
                }
                if (!empty($attributeSQLQueryParts)) {
                    // Attribute stock is ALWAYS reduced (no matter what is set as STOCK_LIMITED or the other constants)!
                    $attributeSQLConditionSnippet = '(' . str_replace(
                            array(
                                'OPTIONS_ID', 'OPTIONS_VALUES_ID',
                                'ATTRIBUTES_ID'
                            ), array(
                                '`pa`.`options_id`',
                                '`pa`.`options_values_id`',
                                '`pa`.`products_attributes_id`'
                            ),
                            implode(') OR (', $attributeSQLQueryParts)
                        ) . ')';

                    // Update attributes stock
                    $qry = "
                        UPDATE `" . TABLE_PRODUCTS_ATTRIBUTES . "` AS `pa`
                            SET `pa`.`attributes_stock` = `pa`.`attributes_stock` - {$item->getQuantity(
                        )}
                        WHERE `pa`.`products_id` = '{$productId}'
                            AND ({$attributeSQLConditionSnippet})
                    ;";
                    xtc_db_query($qry);
                }
            }

            // Specials stock and active status
            if (!empty($internalOrderInfo['is_special_price'])) {
                // Always update specials quantity if it is a special
                $qry = "
                    UPDATE `" . TABLE_SPECIALS . "` AS `s`
                        SET `s`.`specials_quantity` = `s`.`specials_quantity` - {$item->getQuantity(
                    )}
                    WHERE `s`.`products_id` = '{$productId}'
                ;";
                xtc_db_query($qry);

                $reduceQuantitySqlSnippet = '';
                if (STOCK_CHECK == 'true') {
                    // only if stock check is active we have to deactivate specials
                    $reduceQuantitySqlSnippet =
                        " OR `s`.specials_quantity <= 0 AND `s`.`products_id` = '{$productId}'";
                }

                // Always deactivate specials that have turned to a value equal to or less than zero and deactivate all specials that are expired
                $qry = "
                    UPDATE `" . TABLE_SPECIALS . "` AS `s`
                        SET `s`.`status` = 0
                    WHERE
                        `s`.`status` != 0
                        AND
                            (`s`.`expires_date` < NOW() AND `s`.`expires_date` != '0000-00-00 00:00:00' AND `s`.`expires_date` IS NOT NULL
                        " . $reduceQuantitySqlSnippet . ")
                ;";
                xtc_db_query($qry);
            }
        }
    }

    /**
     * insert total data to an order into the database
     *
     * @param ShopgateOrder $order
     * @param               $dbOrderId
     */
    private function insertOrderTotal(ShopgateOrder $order, $dbOrderId)
    {
        $amountWithTax   = $order->getAmountComplete();
        $shippingTaxRate = $this->getOrderShippingTaxRate($order);
        $taxes           = $this->getOrderTaxes($order, $dbOrderId, $shippingTaxRate);
        $xtPrice         = new xtcPrice($this->currency["code"], 1);
        $shippingCosts   = $order->getAmountShipping();
        $couponModel     = new Shopgate_Models_Coupon($this->currency['code'], $this->config, $this->languageId, $this->language, $this->countryId);


        $this->log(
            'insertOrderTotal(): add subtotal', ShopgateLogger::LOGTYPE_DEBUG
        );

        $sort = 10;

        $ordersTotal               = array();
        $ordersTotal["orders_id"]  = $dbOrderId;
        $ordersTotal["title"]      = xtc_db_prepare_input("Zwischensumme:");
        $ordersTotal["text"]       =
            $xtPrice->xtcFormat($order->getAmountItems(), true);
        $ordersTotal["value"]      = $order->getAmountItems();
        $ordersTotal["class"]      = "ot_subtotal";
        $ordersTotal["sort_order"] = $sort++;
        xtc_db_perform(TABLE_ORDERS_TOTAL, $ordersTotal);

        $this->log(
            'insertOrderTotal(): add shipping costs total',
            ShopgateLogger::LOGTYPE_DEBUG
        );

        $shippingDisplayName       = $order->getShippingInfos()->getDisplayName();
        $ordersTotal               = array();
        $ordersTotal["orders_id"]  = $dbOrderId;
        $ordersTotal["title"]      = xtc_db_prepare_input(
            "Versand"
            . (!empty($shippingDisplayName) ?
                ' (' . $shippingDisplayName . '):' : '')
        );
        $ordersTotal["text"]       = $xtPrice->xtcFormat($shippingCosts, true);
        $ordersTotal["value"]      = $shippingCosts;
        $ordersTotal["class"]      = "ot_shipping";
        $ordersTotal["sort_order"] = $sort++;
        xtc_db_perform(TABLE_ORDERS_TOTAL, $ordersTotal);

        // insert payment costs.
        //
        //WARNING: On modify: Change the taxes calculation too!
        if ($order->getAmountShopPayment() != 0) {
            $this->log('db: save payment fee', ShopgateLogger::LOGTYPE_DEBUG);

            $paymentInfos = $order->getPaymentInfos();

            $ordersTotal               = array();
            $ordersTotal["orders_id"]  = $dbOrderId;
            $ordersTotal["title"]      = xtc_db_prepare_input(
                'Zahlungsartkosten'
                . (!empty($paymentInfos['shopgate_payment_name']) ?
                    ' (' . $paymentInfos['shopgate_payment_name'] . '):' : '')
            );
            $ordersTotal["text"]       =
                $xtPrice->xtcFormat($order->getAmountShopPayment(), true);
            $ordersTotal["value"]      = $order->getAmountShopPayment();
            $ordersTotal["class"]      = "ot_shipping";
            $ordersTotal["sort_order"] = $sort++;
            xtc_db_perform(TABLE_ORDERS_TOTAL, $ordersTotal);

        }

        $this->log(
            'insertOrderTotal(): add tax totals', ShopgateLogger::LOGTYPE_DEBUG
        );

        foreach ($taxes as $percent => $tax_value) {
            $ordersTotal               = array();
            $ordersTotal["orders_id"]  = $dbOrderId;
            $ordersTotal["title"]      = "inkl. UST {$percent} %";
            $ordersTotal["text"]       = $xtPrice->xtcFormat($tax_value, true);
            $ordersTotal["value"]      = $tax_value;
            $ordersTotal["class"]      = "ot_tax";
            $ordersTotal["sort_order"] = $sort++;
            xtc_db_perform(TABLE_ORDERS_TOTAL, $ordersTotal);
        }

        $this->log(
            'insertOrderTotal(): add coupon order total',
            ShopgateLogger::LOGTYPE_DEBUG
        );

        $couponAmount = 0;
        $coupons      = $order->getExternalCoupons();
        if (!empty($coupons)) {
            foreach ($coupons as $coupon) {
                $couponAmount += $couponModel->insertOrderTotal($dbOrderId, $coupon, $sort++);
            }
        }

        $this->log(
            'insertOrderTotal(): add order total',
            ShopgateLogger::LOGTYPE_DEBUG
        );

        $ordersTotal               = array();
        $ordersTotal["orders_id"]  = $dbOrderId;
        $ordersTotal["title"]      = "<b>Summe:</b>";
        $ordersTotal["text"]       =
            "<b>" . $xtPrice->xtcFormat($amountWithTax, true) . "</b>";
        $ordersTotal["value"]      = $amountWithTax;
        $ordersTotal["class"]      = "ot_total";
        $ordersTotal["sort_order"] = $sort++;
        xtc_db_perform(TABLE_ORDERS_TOTAL, $ordersTotal);

    }

    /**
     * @param ShopgateOrder $order
     *
     * @return float|int
     */
    private function getOrderShippingTaxRate(ShopgateOrder $order)
    {
        $this->log('getOrderShippingTaxRate(): start', ShopgateLogger::LOGTYPE_DEBUG);

        $shippingTaxRate = 0;

        // Check if a shipping method is set in config
        $shippingMethod    = $this->config->getShipping();
        $orderCountryCode2 = $order->getInvoiceAddress()->getCountry();

        if (!empty($shippingMethod)) {
            $this->log('db: get configuration value ', ShopgateLogger::LOGTYPE_DEBUG);

            // Get tax value from shipping module
            $qry =
                "SELECT `c`.`configuration_value`, `tr`.`tax_rate` " . "FROM `"
                . TABLE_CONFIGURATION . "` AS `c` " . "INNER JOIN `"
                . TABLE_TAX_RATES
                . "` AS `tr` ON(`c`.`configuration_value`=`tr`.`tax_class_id`) "
                . "INNER JOIN `" . TABLE_ZONES_TO_GEO_ZONES
                . "` AS `geozones` ON(`tr`.`tax_zone_id`=`geozones`.`geo_zone_id`) "
                . "INNER JOIN `" . TABLE_COUNTRIES
                . "` AS `co` ON(`geozones`.`zone_country_id`=`co`.`countries_id`) "
                . "WHERE " . "`c`.`configuration_key` = 'MODULE_SHIPPING_"
                . strtoupper($shippingMethod) . "_TAX_CLASS' " . "AND "
                . "`co`.`countries_iso_code_2`='$orderCountryCode2';";

            $result           = xtc_db_query($qry);
            $moduleTaxSetting = xtc_db_fetch_array($result);

            if (!empty($moduleTaxSetting)
                && !empty($moduleTaxSetting['configuration_value'])
                && !empty($moduleTaxSetting['tax_rate'])
            ) {
                // get the tax rate for the shipping costs
                $shippingTaxRate = intval($moduleTaxSetting['tax_rate'] * 100) / 100;
            }
        }

        $this->log('getOrderShippingTaxRate(): end', ShopgateLogger::LOGTYPE_DEBUG);

        return $shippingTaxRate;
    }

    /**
     * calculate the tax value to the order
     *
     * @param ShopgateOrder $order
     * @param               $dbOrderId
     * @param int           $shippingTaxRate
     *
     * @return array
     */
    private function getOrderTaxes(ShopgateOrder $order, $dbOrderId, $shippingTaxRate = 0)
    {
        $this->log('getOrderTaxes(): start', ShopgateLogger::LOGTYPE_DEBUG);

        $taxes = array();

        foreach ($order->getItems() as $orderItem) {
            $tax       = $orderItem->getTaxPercent();
            $tax       = intval($tax * 100) / 100;
            $tax_value = $orderItem->getUnitAmountWithTax() - $orderItem->getUnitAmount();

            if (!isset($taxes[$tax])) {
                $taxes[$tax] = 0;
            }

            $taxes[$tax] += $tax_value * $orderItem->getQuantity();
        }

        if (!empty($shippingTaxRate)) {

            $shippingTaxRate = intval($shippingTaxRate * 100) / 100;

            if (!isset($taxes[$shippingTaxRate])) {
                $taxes[$shippingTaxRate] = 0;
            }

            $taxes[$shippingTaxRate] +=
                $order->getAmountShipping() - $this->getOrderShippingAmountWithoutTax($order, $shippingTaxRate);
        }

        // set taxes for payment method
        if ($order->getAmountShopPayment() != 0) {
            $tax       = $order->getPaymentTaxPercent();
            $tax       = intval($tax * 100) / 100;
            $tax_value =
                $order->getAmountShopPayment() - round(
                    ($order->getAmountShopPayment() * 100) / ($order->getPaymentTaxPercent() + 100), 2
                );

            if (!isset($taxes[$tax])) {
                $taxes[$tax] = 0;
            }

            $taxes[$tax] += $tax_value;
        }

        $this->log('getOrderTaxes(): end', ShopgateLogger::LOGTYPE_DEBUG);

        return $taxes;
    }

    /**
     * calculate the shipping cost value without tax
     *
     * @param ShopgateOrder $order
     * @param int           $shippingTaxRate
     *
     * @return float|int
     */
    private function getOrderShippingAmountWithoutTax(ShopgateOrder $order, $shippingTaxRate = 0)
    {
        $shippingAmountWithoutTax = $order->getAmountShipping();

        // Check if a shipping method is set in config
        $shippingMethod = $this->config->getShipping();
        if (!empty($shippingTaxRate)) {
            // remove tax from shipping costs
            $shippingAmountWithoutTax /= 1 + $shippingTaxRate / 100;
        }

        return $shippingAmountWithoutTax;
    }

    /**
     * push the order data to afterbuy
     *
     * @param               $iOrderId
     * @param ShopgateOrder $order
     */
    private function pushOrderToAfterbuy($iOrderId, ShopgateOrder $order)
    {
        if (!$order->getIsShippingBlocked() && defined('AFTERBUY_ACTIVATED') && AFTERBUY_ACTIVATED == 'true') {

            $this->log("START TO SEND ORDER TO AFTERBUY", ShopgateLogger::LOGTYPE_ACCESS);

            require_once(DIR_WS_CLASSES . 'afterbuy.php');

            $aBUY = new xtc_afterbuy_functions($iOrderId);

            if ($aBUY->order_send()) {
                $aBUY->process_order();
                $this->log("SUCCESSFUL ORDER SEND TO AFTERBUY", ShopgateLogger::LOGTYPE_ACCESS);
            } else {
                $this->log("ORDER ALREADY SEND TO AFTERBUY", ShopgateLogger::LOGTYPE_ACCESS);
            }

            $this->log("FINISH SEND ORDER TO AFTERBUY", ShopgateLogger::LOGTYPE_ACCESS);
        }
    }

    /**
     * push the order data to dreambot
     *
     * @param               $dbOrderId
     * @param ShopgateOrder $shopgateOrder
     */
    private function pushOrderToDreamRobot($dbOrderId, ShopgateOrder $shopgateOrder)
    {
        if (!$shopgateOrder->getIsShippingBlocked() && file_exists(DIR_FS_CATALOG . 'dreamrobot_checkout.inc.php')) {

            require_once(rtrim(DIR_FS_CATALOG, "/") . '/includes/classes/order.php');

            $this->log("START TO SEND ORDER TO DREAMROBOT", ShopgateLogger::LOGTYPE_ACCESS);

            $order                        = new order($dbOrderId);
            $_SESSION['tmp_oID']          = $dbOrderId;
            $order->info['shipping_cost'] = $shopgateOrder->getAmountShipping();

            include_once('./dreamrobot_checkout.inc.php');

            $this->log("FINISH SEND ORDER TO DREAMROBOT", ShopgateLogger::LOGTYPE_ACCESS);
        }
    }

    /**
     * send order confirmation mail to customer
     * Source was taken from send_order xtc class
     * also xtc_php_mailer class was taken from xtc3, changed and used in our plugin
     *
     * @param int $insert_id
     * @param int $userId
     *
     * @throws ShopgateLibraryException
     */
    private function sendOrderEmail($insert_id, $userId)
    {

        $_SESSION['customer_id'] = $userId;

        require_once(DIR_FS_INC . 'xtc_get_order_data.inc.php');
        require_once(DIR_FS_INC . 'xtc_get_attributes_model.inc.php');
        require_once(DIR_WS_INCLUDES . "classes/order.php");
        require_once(DIR_FS_CATALOG . "shopgate/base/inc/xtc_php_mail.inc.php");
        define(SEND_EMAILS, 'true');

        $smarty = new Smarty;

        $order_query_check = xtc_db_query(
            "SELECT
                      customers_id
                      FROM " . TABLE_ORDERS . "
                      WHERE orders_id='" . $insert_id . "'"
        );

        $order_check = xtc_db_fetch_array($order_query_check);
        if ($_SESSION['customer_id'] == $order_check['customers_id']) {

            $order = new order($insert_id);

            $smarty->assign(
                'address_label_customer', xtc_address_format(
                                            $order->customer['format_id'], $order->customer, 1, '',
                                            '<br />'
                                        )
            );
            $smarty->assign(
                'address_label_shipping', xtc_address_format(
                                            $order->delivery['format_id'], $order->delivery, 1, '',
                                            '<br />'
                                        )
            );

            if ($_SESSION['credit_covers'] != '1') {
                $smarty->assign(
                    'address_label_payment', xtc_address_format(
                                               $order->billing['format_id'], $order->billing, 1, '',
                                               '<br />'
                                           )
                );
            }

            $smarty->assign('csID', $order->customer['csID']);

            $order_total = $order->getTotalData($insert_id);

            $smarty->assign('order_data', $order->getOrderData($insert_id));
            $smarty->assign('order_total', $order_total['data']);

            // assign language to template for caching
            $smarty->assign('language', $_SESSION['language']);
            $smarty->assign('tpl_path', 'templates/' . CURRENT_TEMPLATE . '/');
            $smarty->assign(
                'logo_path',
                HTTP_SERVER . DIR_WS_CATALOG . 'templates/' . CURRENT_TEMPLATE
                . '/img/'
            );
            $smarty->assign('oID', $insert_id);

            if ($order->info['payment_method'] != ''
                && $order->info['payment_method'] != 'no_payment'
            ) {
                include(rtrim(DIR_WS_LANGUAGES, "/") . "/"
                    . $_SESSION['language'] . '/modules/payment/'
                    . $order->info['payment_method'] . '.php');
                $payment_method = constant(
                    strtoupper(
                        'MODULE_PAYMENT_' . $order->info['payment_method']
                        . '_TEXT_TITLE'
                    )
                );
            }

            $smarty->assign('PAYMENT_METHOD', $payment_method);
            $smarty->assign(
                'DATE', xtc_date_long($order->info['date_purchased'])
            );

            $smarty->assign('NAME', $order->customer['name']);
            $smarty->assign('COMMENTS', $order->info['comments']);
            $smarty->assign('EMAIL', $order->customer['email_address']);
            $smarty->assign('PHONE', $order->customer['telephone']);

            // PAYMENT MODUL TEXTS
            // EU Bank Transfer
            if ($order->info['payment_method'] == 'eustandardtransfer') {
                $smarty->assign(
                    'PAYMENT_INFO_HTML',
                    MODULE_PAYMENT_EUTRANSFER_TEXT_DESCRIPTION
                );
                $smarty->assign(
                    'PAYMENT_INFO_TXT', str_replace(
                                          "<br />", "\n",
                                          MODULE_PAYMENT_EUTRANSFER_TEXT_DESCRIPTION
                                      )
                );
            }

            // MONEYORDER
            if ($order->info['payment_method'] == 'moneyorder') {
                $smarty->assign(
                    'PAYMENT_INFO_HTML',
                    MODULE_PAYMENT_MONEYORDER_TEXT_DESCRIPTION
                );
                $smarty->assign(
                    'PAYMENT_INFO_TXT', str_replace(
                                          "<br />", "\n",
                                          MODULE_PAYMENT_MONEYORDER_TEXT_DESCRIPTION
                                      )
                );
            }

            // dont allow cache
            $smarty->caching = false;

            $html_mail = $smarty->fetch(
                CURRENT_TEMPLATE . '/mail/' . $_SESSION['language']
                . '/order_mail.html'
            );
            $txt_mail  = $smarty->fetch(
                CURRENT_TEMPLATE . '/mail/' . $_SESSION['language']
                . '/order_mail.txt'
            );

            // create subject
            $order_subject =
                str_replace('{$nr}', $insert_id, EMAIL_BILLING_SUBJECT_ORDER);
            $order_subject = str_replace(
                '{$date}', strftime(DATE_FORMAT_LONG), $order_subject
            );
            $order_subject = str_replace(
                '{$lastname}', $order->customer['lastname'], $order_subject
            );
            $order_subject = str_replace(
                '{$firstname}', $order->customer['firstname'], $order_subject
            );

            // send mail to admin
            if (!shopgate_php_mail(
                EMAIL_BILLING_ADDRESS, EMAIL_BILLING_NAME,
                EMAIL_BILLING_ADDRESS, STORE_NAME,
                EMAIL_BILLING_FORWARDING_STRING,
                $order->customer['email_address'],
                $order->customer['firstname'], '', '', $order_subject,
                $html_mail, $txt_mail
            )
            ) {
                throw new ShopgateLibraryException(
                    ShopgateLibraryException::PLUGIN_EMAIL_SEND_ERROR,
                    "Error while sending order confirmation mail to admin"
                );
            }

            // send mail to customer
            if (!shopgate_php_mail(
                EMAIL_BILLING_ADDRESS, EMAIL_BILLING_NAME,
                $order->customer['email_address'],
                $order->customer['firstname'] . ' '
                . $order->customer['lastname'], '', EMAIL_BILLING_REPLY_ADDRESS,
                EMAIL_BILLING_REPLY_ADDRESS_NAME, '', '', $order_subject,
                $html_mail, $txt_mail
            )
            ) {
                throw new ShopgateLibraryException(
                    ShopgateLibraryException::PLUGIN_EMAIL_SEND_ERROR,
                    "Error while sending order confirmation mail to customer"
                );
            }
        }
    }

    /**
     * read all tax classes from the database
     *
     * @return array
     * @throws ShopgateLibraryException
     */
    protected function getTaxClasses()
    {
        $sqlQuery    =
            "SELECT `tbl_tc`.`tax_class_id` 'id', `tbl_tc`.`tax_class_title` 'key' FROM `"
            . TABLE_TAX_CLASS . "` as tbl_tc";
        $queryResult = xtc_db_query($sqlQuery);
        $result      = array();
        if (!$queryResult) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_DATABASE_ERROR,
                "Shopgate Plugin - Error selecting.", true
            );
        } else {
            while ($row = xtc_db_fetch_array($queryResult)) {
                foreach ($row AS &$value) {
                    $value = utf8_encode($value);
                }
                array_push($result, $row);
            }
        }

        return $result;
    }

    /**
     * read all tax rates from the database
     *
     * @return array<string, mixed>[]
     * @throws ShopgateLibraryException
     */
    protected function getTaxRates()
    {
        $query =
            "SELECT tr.tax_rates_id, tr.tax_description, tr.tax_rate, tr.tax_priority, "
            . "c.countries_iso_code_2, z.zone_code, tc.tax_class_id, tc.tax_class_title "
            . "FROM `" . TABLE_TAX_RATES . "` AS tr " .

            "JOIN `" . TABLE_GEO_ZONES
            . "` AS gz ON tr.tax_zone_id = gz.geo_zone_id " . "JOIN `"
            . TABLE_ZONES_TO_GEO_ZONES
            . "` AS ztgz ON gz.geo_zone_id = ztgz.geo_zone_id " .

            "JOIN `" . TABLE_COUNTRIES
            . "` AS c ON ztgz.zone_country_id = c.countries_id "
            . "LEFT OUTER JOIN `" . TABLE_ZONES
            . "` AS z ON ztgz.zone_id = z.zone_id "
            . // zone (aka state) might not be mapped, rate applies for whole country in that case
            "JOIN `" . TABLE_TAX_CLASS
            . "` tc ON tr.tax_class_id = tc.tax_class_id;";

        $result = xtc_db_query($query);

        if (!$result) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_DATABASE_ERROR,
                "Shopgate Plugin - Error selecting.", true
            );
        }

        $taxRates = array();
        while ($row = xtc_db_fetch_array($result)) {
            foreach ($row AS &$value) {
                $value = utf8_encode($value);
            }
            $taxRates[] = $row;
        }

        return $taxRates;
    }

    /**
     * return an array with all valid shipping methods to an order
     *
     * @param ShopgateCart $sgShoppingcart
     *
     * @return array
     */
    private function getShipping(ShopgateCart $sgShoppingcart)
    {
        $resultShippingMethods = array();

        if (!defined('MODULE_SHIPPING_INSTALLED') || MODULE_SHIPPING_INSTALLED == "") {
            return array();
        }

        /* INCLUDES BOF */
        include_once(rtrim(DIR_WS_CLASSES, "/") . "/shopping_cart.php");
        global $cart;
        $cart = new shoppingCart();
        if (method_exists($cart,'add_cart')) {
            $addCartMethod = 'add_cart';
        } elseif (method_exists($cart,'add')) {
            $addCartMethod = 'add';
        } else {
            return array();
        }

        global $total_count, $shipping_weight, $total_weight, $shipping_num_boxes;
        $cartItemModel      = new Shopgate_Models_Item_CartItem($this->languageId);
        $locationModel      = new Shopgate_Models_Location();
        $taxModel           = new Shopgate_Models_Tax();
        $total_count        = count($sgShoppingcart->getItems());
        $total_weight       = $cartItemModel->getProductsWeight($sgShoppingcart->getItems());
        $shipping_num_boxes = 1;

        $cartContents = array();
        foreach ($sgShoppingcart->getItems() as $product) {
            $options = $product->getOptions();

            if (!empty($options)) {
                $sgOptions = array();

                foreach ($options as $option) {
                    $sgOptions[$option->getOptionNumber()] =
                        $option->getValueNumber();
                }
            } else {
                $sgOptions = ''; // add_cart checks for is_array()
            }

            $cart->$addCartMethod($product->getItemNumber(), $product->getQuantity(), $sgOptions);
            $cartContents[$product->getItemNumber()] = array('qty' => $product->getQuantity());
        }
        $_SESSION['actual_content'] = $cartContents;
        $cart->contents             = $cartContents;
        $_SESSION['cart']           = $cart;

        $sgDeliverAddress = $sgShoppingcart->getDeliveryAddress();

        if (!empty($sgDeliverAddress)) {
            $country = $locationModel->getCountryByIso2Name($sgDeliverAddress->getCountry());
            $zone    = $locationModel->getZoneByCountryId($country["countries_id"]);

            $sendto = array(
                "firstname"         => $sgDeliverAddress->getFirstName(),
                "lastname"          => $sgDeliverAddress->getLastName(),
                "company"           => $sgDeliverAddress->getCompany(),
                "street_address"    => $sgDeliverAddress->getStreet1(),
                "suburb"            => "",
                "postcode"          => $sgDeliverAddress->getZipcode(),
                "city"              => $sgDeliverAddress->getCity(),
                "zone_id"           => $zone["zone_id"],
                "zone_name"         => $zone["zone_name"],
                "country"           => array(
                    "id"         => $country["countries_id"],
                    "title"      => $country['countries_name'],
                    "iso_code_2" => $country["countries_iso_code_2"],
                    "iso_code_3" => $country["countries_iso_code_3"],
                ),
                "country_id"        => $country["countries_id"],
                "address_format_id" => "",
            );

            $_SESSION['delivery_zone'] = $country['countries_iso_code_2'];
        }

        $sgInvoiceAddress = $sgShoppingcart->getInvoiceAddress();

        if (empty($sgInvoiceAddress)) {
            $billto = $sendto;
        } else {
            $country = $locationModel->getCountryByIso2Name(
                $sgInvoiceAddress->getCountry()
            );
            $zone    = $locationModel->getZoneByCountryId(
                $country["countries_id"]
            );

            $billto = array(
                "firstname"         => $sgInvoiceAddress->getFirstName(),
                "lastname"          => $sgInvoiceAddress->getLastName(),
                "company"           => $sgInvoiceAddress->getCompany(),
                "street_address"    => $sgInvoiceAddress->getStreet1(), "suburb" => "",
                "postcode"          => $sgInvoiceAddress->getZipcode(), "city" => $sgInvoiceAddress->getCity(),
                "zone_id"           => $zone["zone_id"],
                "zone_name"         => $zone["zone_name"],
                "country"           => array(
                    "id"         => $country["countries_id"],
                    "title"      => $country['countries_name'],
                    "iso_code_2" => $country["countries_iso_code_2"],
                    "iso_code_3" => $country["countries_iso_code_3"],
                ),
                "country_id"        => $country["countries_id"],
                "address_format_id" => "",
            );
        }

        include_once(rtrim(DIR_WS_CLASSES, "/") . "/order.php");
        global $order;
        $order           = new order();
        $order->customer = $sendto;
        $order->delivery = $sendto;
        $order->billing  = $billto;

        include_once(rtrim(DIR_WS_MODULES, "/") . "/order_total/ot_shipping.php");

        global $ot_shipping;
        $ot_shipping = new ot_shipping();
        $ot_shipping->process();

        require(DIR_WS_CLASSES . 'shipping.php');
        $shipping_modules = new shipping;

        if (defined('MODULE_ORDER_TOTAL_SHIPPING_FREE_SHIPPING')
            && (MODULE_ORDER_TOTAL_SHIPPING_FREE_SHIPPING == 'true')
        ) {
            $pass = false;

            switch (MODULE_ORDER_TOTAL_SHIPPING_DESTINATION) {
                case 'national':
                    if ($order->delivery['country_id'] == STORE_COUNTRY) {
                        $pass = true;
                    }
                    break;
                case 'international':
                    if ($order->delivery['country_id'] != STORE_COUNTRY) {
                        $pass = true;
                    }
                    break;
                case 'both':
                    $pass = true;
                    break;
            }

            $free_shipping = false;
            if (($pass == true)
                && ($order->info['total']
                    >= MODULE_ORDER_TOTAL_SHIPPING_FREE_SHIPPING_OVER)
            ) {
                $free_shipping = true;
                include(DIR_WS_LANGUAGES . $this->language . '/modules/order_total/ot_shipping.php');
            }
        } else {
            $free_shipping = false;
        }

        // if shipping is free all other shipping methods will be ignored
        if ($free_shipping) {
            global $xtPrice;
            $sgShippingMethod = new ShopgateShippingMethod();
            $sgShippingMethod->setDescription(
                html_entity_decode(
                    sprintf(
                        FREE_SHIPPING_DESCRIPTION,
                        $xtPrice->xtcFormat(MODULE_ORDER_TOTAL_SHIPPING_FREE_SHIPPING_OVER, true, 0, true)
                    )
                )
            );

            $sgShippingMethod->setTitle(FREE_SHIPPING_TITLE);
            $sgShippingMethod->setAmount(0);

            return array($sgShippingMethod);
        }

        $quotes = $shipping_modules->quote();

        foreach ($quotes as $shippingModule) {
            if (isset($shippingModule['error'])) {
                continue;
            }

            //we dont support usps as shopgate plugin shipping method
            //also on error continue
            if (strpos(
                    $shippingModule['module'], "United States Postal Service"
                ) !== false
                || !empty($shippingModule['error'])
            ) {
                continue;
            }

            $sgShippingMethod = new ShopgateShippingMethod();
            $sgShippingMethod->setId($shippingModule["id"]);
            $sgShippingMethod->setTitle($shippingModule["module"]);
            $sgShippingMethod->setTaxPercent($shippingModule["tax"]);

            if (isset($shippindModule["tax"]) && !empty($shippindModule["tax"])) {
                $sgShippingMethod->setTaxClass($taxModel->getTaxClassByValue($shippindModule["tax"]));
            }

            if (!empty($shippingModule["methods"]) && is_array($shippingModule["methods"])) {
                foreach ($shippingModule["methods"] as $method) {

                    $tmp_shipping = $sgShippingMethod;
                    $cost         = $method["cost"];

                    if (isset($shippingModule["tax"]) && !empty($shippingModule["tax"])) {
                        $costWithTax = $this->formatPriceNumber($cost * (1 + ($shippingModule["tax"] / 100)), 2);
                        $tmp_shipping->setAmountWithTax($costWithTax);
                        $tmp_shipping->setAmount($cost);
                    } else {
                        $tmp_shipping->setAmountWithTax($cost);
                    }

                    $resultShippingMethods[] = $tmp_shipping;
                }
            }
        }

        return $resultShippingMethods;
    }

    /**
     * validate all products the ShopgateCart object has included
     *
     * @param ShopgateCart $cart
     *
     * @return array
     */
    private function checkCartItems(ShopgateCart $cart)
    {

        $includeXtcFiles = array(
            "/inc/xtc_get_tax_class_id.inc.php",
            "/inc/xtc_get_products_stock.inc.php",
        );

        foreach ($includeXtcFiles as $xtcFile) {
            $file = rtrim(DIR_FS_CATALOG, "/") . $xtcFile;
            if (file_exists($file)) {
                include_once $file;
            }
        }

        $returnProducts = array();
        $itemModel      = new Shopgate_Models_Item($this->config, $this->languageId, $this->exchangeRate);
        $cartItemModel  = new Shopgate_Models_Item_CartItem($this->languageId);

        foreach ($cart->getItems() AS $product) {

            $sgProduct    = new ShopgateCartItem();
            $sgOrderInfos = $this->jsonDecode($product->getInternalOrderInfo(), true);

            $productId = $cartItemModel->getProductIdFromCartItem($product);
            $dbproduct = $itemModel->getProductById($productId);

            if (empty($dbproduct)) {
                $sgProduct->setIsBuyable(false);
                $sgProduct->setError(ShopgateLibraryException::CART_ITEM_PRODUCT_NOT_FOUND);
                $returnProducts[] = $sgProduct;
                continue;
            }

            //tax
            $productTaxClassId = xtc_get_tax_class_id($productId);
            $productTaxRate    = xtc_get_tax_rate($productTaxClassId, $this->countryId, $this->zoneId);

            //price
            $xtcPrice     = new xtcPrice($this->currency["code"], $this->config->getCustomerPriceGroup());
            $priceWithTax = $xtcPrice->xtcGetPrice(
                $productId,
                $format = false,
                1,
                $productTaxClassId,
                $product->getUnitAmount(),
                1
            );
            $price        = $xtcPrice->getPprice($productId);

            $sgProduct->setItemNumber($productId);
            $sgProduct->setIsBuyable(true);

            if (!empty($price)) {
                $sgProduct->setUnitAmount($price);
                $sgProduct->setUnitAmountWithTax($priceWithTax);
            }

            $quantitiy = xtc_get_products_stock($productId);

            if (!empty($dbproduct["specials_new_products_price"])
                && (STOCK_CHECK == 'true' && STOCK_ALLOW_CHECKOUT == 'false')
                && ($dbproduct["specials_quantity"] > 0)
            ) {
                // Nur wenn die quantity > 0 ist dann specialprice setzen, ansonsten normalen Preis mit normalem Stock
                $quantitiy =
                    $dbproduct["specials_quantity"] > $quantitiy ? $quantitiy : $dbproduct["specials_quantity"];
            }

            $sgProduct->setStockQuantity($quantitiy);
            $sgProduct->setQtyBuyable($quantitiy);

            if (($product->getQuantity() > $quantitiy || $quantitiy == 0)
                && (STOCK_CHECK == 'true' && STOCK_ALLOW_CHECKOUT == 'false')
            ) {
                $sgProduct->setIsBuyable(false);
                $sgProduct->setError(
                    ShopgateLibraryException::CART_ITEM_OUT_OF_STOCK
                );
            }

            $attributeIds = array();
            foreach ($sgOrderInfos as $infoName => $infoValue) {
                if (strpos($infoName, "attribute_") === 0) {
                    if (is_array($infoValue)) {
                        foreach ($infoValue as $attributeKey => $attributeArray) {
                            $attributeIds[] = array_merge(
                                array("products_attributes_id" => $attributeKey),
                                $attributeArray
                            );
                        }
                    }
                }
            }

            $sgProduct->setOptions(
                $cartItemModel->getOptionsToProduct(
                    $productId, $attributeIds, $productTaxRate
                )
            );
            $sgProduct->setAttributes(
                $cartItemModel->getAttributesToProduct(
                    $product, $sgProduct
                )
            );
            //not supported by xtcommerce3
            $sgProduct->setInputs(array());
            $returnProducts[] = $sgProduct;
        }

        return $returnProducts;
    }

    /**
     * Marks shipped orders as "shipped" at Shopgate.
     *
     * This will find all orders that are marked "shipped" in the shop system but not at Shopgate yet and marks them "shipped" at Shopgate via
     * Shopgate Merchant API.
     *
     * @param string $message    Process log will be appended to this reference.
     * @param int    $errorcount This reference gets incremented on errors.
     */
    protected function cronSetOrdersShippingCompleted(&$message, &$errorcount)
    {
        $query  = "SELECT `sgo`.`orders_id`, `sgo`.`shopgate_order_number` "
            . "FROM `" . TABLE_SHOPGATE_ORDERS . "` sgo " . "INNER JOIN `"
            . TABLE_ORDERS . "` xto ON (`xto`.`orders_id` = `sgo`.`orders_id`) "
            . "INNER JOIN `" . TABLE_LANGUAGES
            . "` xtl ON (`xtl`.`directory` = `xto`.`language`) "
            . "WHERE `sgo`.`is_sent_to_shopgate` = 0 "
            . "AND `xto`.`orders_status` = " . xtc_db_input($this->config->getOrderStatusShipped()) . " "
            . "AND `xtl`.`code` = '" . xtc_db_input($this->config->getLanguage()) . "';";
        $result = xtc_db_query($query);

        if (empty($result)) {
            return;
        }

        while ($shopgateOrder = xtc_db_fetch_array($result)) {
            if (!$this->setOrderShippingCompleted($shopgateOrder, $this->merchantApi, $this->config)) {
                $errorcount++;
                $message .= SHOPGATE_ORDER_ORDER_NUMBER . '"' . $shopgateOrder['shopgate_order_number'] . "\" error\n";
            }
        }

        if (!empty($errorcount)) {
            $this->log($message, ShopgateLogger::LOGTYPE_ERROR);
        }
    }

    /**
     * Sets the order status of a Shopgate order to "shipped" via Shopgate Merchant API
     *
     * @param                     array of string     $shopgateOrder
     * @param ShopgateMerchantApi $merchantApi
     * @param ShopgateConfigXtc   $config
     *
     * @return bool true on success, false on failure.
     */
    protected function setOrderShippingCompleted(
        $shopgateOrder, ShopgateMerchantApi &$merchantApi, ShopgateConfigXtc &$config
    ) {
        $success = false;
        // These are expected and should not be added to error count:
        $ignoreCodes = array(
            ShopgateMerchantApiException::ORDER_ALREADY_COMPLETED,
            ShopgateMerchantApiException::ORDER_SHIPPING_STATUS_ALREADY_COMPLETED
        );

        try {
            $sendMail = ($this->config->getSendShopgateShippingConfirmationMail() == 0) ? false : true;
            $merchantApi->setOrderShippingCompleted($shopgateOrder["shopgate_order_number"], $sendMail);
            $this->saveSingleOrdersStatusHistory(
                $shopgateOrder["orders_id"], $config->getOrderStatusShipped(), SHOPGATE_ORDER_MARKED_AS_SENT, 1
            );
            $success = true;
        } catch (ShopgateLibraryException $e) {
            $msg = SHOPGATE_DEFAULT_MODULE_EXCEPTION_TEXT . "{$e->getCode()}): {$e->getAdditionalInformation()} )";
            $this->saveSingleOrdersStatusHistory($shopgateOrder["orders_id"], $config->getOrderStatusShipped(), $msg);
        } catch (ShopgateMerchantApiException $e) {
            $msg = SHOPGATE_DEFAULT_MODULE_EXCEPTION_TEXT . "{$e->getCode()}): {$e->getMessage()} )";
            $this->saveSingleOrdersStatusHistory($shopgateOrder["orders_id"], $config->getOrderStatusShipped(), $msg);
            $success = (in_array($e->getCode(), $ignoreCodes)) ? true : false;
        } catch (Exception $e) {
            $msg = SHOPGATE_DEFAULT_MODULE_EXCEPTION_TEXT . "{$e->getCode()}): {$e->getMessage()} )";
            $this->saveSingleOrdersStatusHistory($shopgateOrder["orders_id"], $config->getOrderStatusShipped(), $msg);
        }

        // Update shopgate order on success
        if ($success) {
            xtc_db_query(
                'UPDATE `' . TABLE_SHOPGATE_ORDERS . '` SET `is_sent_to_shopgate` = 1 WHERE `shopgate_order_number` = '
                . $shopgateOrder["shopgate_order_number"] . ';'
            );
        }

        return $success;
    }

    /**
     * request Shopgate to cancel/cancel parts(shipping) of an order
     *
     * @param                     array of string     $shopgateOrder
     * @param ShopgateMerchantApi $merchantApi
     * @param ShopgateConfigXtc   $config
     *
     * @return bool
     * @throws ShopgateLibraryException
     */
    protected function setOrderCanceled($shopgateOrder, ShopgateMerchantApi &$merchantApi, ShopgateConfigXtc &$config)
    {
        $success     = false;
        $ignoreCodes =
            array(222);// These are expected and should not be added to error count: //Order already cancelled

        try {

            $orderStatus       = $config->getOrderStatusOpen();
            $cancellationItems = array();
            // fallback when order data was not inserted into db.
            if (empty($shopgateOrder) || empty($shopgateOrder["order_data"])) {

                $params = array(
                    'shop_number'   => $this->config->getShopNumber(),
                    'with_items'    => 1,
                    'order_numbers' => array(0 => "{$shopgateOrder['shopgate_order_number']}")
                );

                /* @var ShopgateMerchantApiResponse $order */
                $merchantOrderResponse = $merchantApi->getOrders($params);
                $sgRequestedOrders     = $merchantOrderResponse->getData();
                if (count($sgRequestedOrders) == 0) {
                    return false;
                }

                /* @var ShopgateOrder $shopgateOrderObject */
                $shopgateOrderObject =
                    reset($sgRequestedOrders);// there can be only one order. cause only one was requested

            } else {
                $shopgateOrderObject = unserialize($shopgateOrder["order_data"]);
            }
            /* @var ShopgateOrder $shopgateOrderObject */
            if (!$config->getAlwaysCancelShipping()) {
                foreach ($shopgateOrderObject->getItems() as $item) {
                    $cancellationItems[] =
                        array(
                            'order_item_id' => $item->getOrderItemId(),
                            'item_number'   => (!is_null($item->getParentItemNumber()) ? $item->getParentItemNumber()
                                : $item->getItemNumber()),
                            'quantity'      => $item->getQuantity()
                        );
                }
            }

            // if cancelShipping is true, cancelCompleteOrder must be true too
            $merchantApi->cancelOrder(
                $shopgateOrder['shopgate_order_number'],
                $config->getAlwaysCancelShipping(), $cancellationItems,
                $config->getAlwaysCancelShipping()
            );

            $this->saveSingleOrdersStatusHistory(
                $shopgateOrder["orders_id"], $orderStatus,
                "[Shopgate] " . ENTRY_ORDER_MARKED_AS_CANCELED
            );

            $success = true;

        } catch (ShopgateLibraryException $e) {
            $msg = SHOPGATE_DEFAULT_MODULE_EXCEPTION_TEXT . "{$e->getCode()}): {$e->getAdditionalInformation()} )";
            $this->saveSingleOrdersStatusHistory($shopgateOrder["orders_id"], $config->getOrderStatusCancled(), $msg);
        } catch (ShopgateMerchantApiException $e) {
            $msg = SHOPGATE_DEFAULT_MODULE_EXCEPTION_TEXT . "{$e->getCode()}): {$e->getMessage()} )";
            $this->saveSingleOrdersStatusHistory($shopgateOrder["orders_id"], $config->getOrderStatusCancled(), $msg);
            $success = (in_array($e->getCode(), $ignoreCodes)) ? true : false;
        } catch (Exception $e) {
            $msg = SHOPGATE_DEFAULT_MODULE_EXCEPTION_TEXT . "{$e->getCode()}): {$e->getMessage()} )";
            $this->saveSingleOrdersStatusHistory($shopgateOrder["orders_id"], $config->getOrderStatusCancled(), $msg);
        }

        // Update shopgate order on success
        if ($success) {
            xtc_db_query(
                'UPDATE `' . TABLE_ORDERS_SHOPGATE_ORDER
                . '` SET `is_cancellation_sent_to_shopgate` = 1 WHERE `shopgate_order_number` = '
                . $shopgateOrder['shopgate_order_number'] . ';'
            );
        }

        return $success;
    }

    /**
     * logic for cron job CANCEL_ORDERS.
     * Takes all as "cancel" marked orders and apply this status to the equivalent Shopgate order.
     *
     * @param $jobname
     * @param $errCount
     * @param $message
     *
     * @throws ShopgateLibraryException
     */
    protected function cronCancelOrder($jobname, &$errCount, &$message)
    {
        if (empty($message)) {
            $message = '';
        }

        // Get shopgate orders
        if (!$this->tableExists(TABLE_ORDERS_SHOPGATE_ORDER)) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_DATABASE_ERROR,
                "Shopgate Plugin - Error: \"" . TABLE_ORDERS_SHOPGATE_ORDER
                . "\" table is missing.", true
            );
        }

        $query  =
            "SELECT `sgo`.`orders_id`, `sgo`.`shopgate_order_number`, `sgo`.`order_data` "
            . "FROM `" . TABLE_ORDERS_SHOPGATE_ORDER . "` sgo " . "INNER JOIN `"
            . TABLE_ORDERS
            . "` osco ON (`osco`.`orders_id` = `sgo`.`orders_id`) "
            . "WHERE `sgo`.`is_cancellation_sent_to_shopgate` = 0 " . "AND ("
            . "`sgo`.`shopgate_shop_number` = '" . xtc_db_input(
                $this->config->getShopNumber()
            ) . "' " . "OR `sgo`.`shopgate_shop_number` IS NULL"
            . // To get the "pre-saving-the-shop-number" ones. These may fail when the API call comes with the wrong shop number.
            ") " . "AND `osco`.`orders_status` = '" . xtc_db_input(
                $this->config->getOrderStatusCancled()
            ) . "';";
        $result = xtc_db_query($query);

        if (empty($result)) {
            return;
        }

        $errorcount = 0;
        while ($shopgateOrder = xtc_db_fetch_array($result)) {
            if (!$this->setOrderCanceled($shopgateOrder, $this->merchantApi, $this->config)) {
                $errorcount++;
                $message .= SHOPGATE_ORDER_ORDER_NUMBER . '"' . $shopgateOrder['shopgate_order_number'] . "\" error\n";
            }
        }

        if (!empty($errorcount)) {
            $this->log($message, ShopgateLogger::LOGTYPE_ERROR);
        }
    }

    /**
     * Creates a status history entry from one single comment text
     *
     * @param        $orderId
     * @param int    $ordersStatusId
     * @param string $commentText
     * @param string $forcedDate
     *
     * @return bool
     * @throws ShopgateLibraryException
     */
    private function saveSingleOrdersStatusHistory(
        $orderId, $ordersStatusId = 1, $commentText = '', $customerNotified = 0, $forcedDate = ''
    ) {

        if (empty($forcedDate)) {
            $forcedDate = date('Y-m-d H:i:s');//'now()';
        }

        $ordersStatusHistory = array(
            'orders_id'         => $orderId,
            'orders_status_id'  => $ordersStatusId,
            'date_added'        => $forcedDate,
            'customer_notified' => 0,
            'comments'          => $this->stringFromUtf8($commentText, $this->config->getEncoding()),
        );

        xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $ordersStatusHistory);

        return true;
    }

    /**
     * Set the shipping status for a list of order IDs.
     *
     * @param int[] $orderIds The IDs of the orders in the shop system.
     * @param int   $status   The ID of the order status that has been set in the shopping system.
     */
    public function updateOrdersStatus($orderIds, $status)
    {
        if (empty($orderIds) || !is_array($orderIds)) {
            return;
        }

        $query  = xtc_db_input(
            "SELECT `sgo`.`orders_id`,`sgo`.`shopgate_order_number`,`xtl`.`code`, "
            . "`sgo`.`is_cancellation_sent_to_shopgate`,`sgo`.`is_sent_to_shopgate`, `sgo`.`order_data` "
            . "FROM `" . TABLE_SHOPGATE_ORDERS . "` sgo " . "INNER JOIN `"
            . TABLE_ORDERS . "` xto ON (`xto`.`orders_id` = `sgo`.`orders_id`) "
            . "INNER JOIN `" . TABLE_LANGUAGES
            . "` xtl ON (`xtl`.`directory` = `xto`.`language`) "
            . "WHERE `sgo`.`orders_id` IN (" . xtc_db_input(implode(", ", $orderIds)) . ")"
        );
        $result = xtc_db_query($query);

        if (empty($result)) {
            return;
        }

        /** @var ShopgateConfigXtc[] $configurations */
        $configurations = array();
        $merchantApis   = array();

        while ($shopgateOrder = xtc_db_fetch_array($result)) {
            $language = $shopgateOrder['code']; // convenience

            if (empty($merchantApis[$language])) {
                try {
                    $config = new ShopgateConfigXtc();
                    $config->loadByLanguage($language);
                    $builder                   = new ShopgateBuilder($config);
                    $merchantApi               = $builder->buildMerchantApi();
                    $merchantApis[$language]   = $merchantApi;
                    $configurations[$language] = $config;
                } catch (ShopgateLibraryException $e) {
                    // do not abort. the error will be logged
                }
            }

            $shippedStatus   = $configurations[$language]->getOrderStatusShipped();
            $cancelledStatus = $configurations[$language]->getOrderStatusCancled();

            if (!empty($shippedStatus) && $status == $shippedStatus) {
                $this->setOrderShippingCompleted($shopgateOrder, $merchantApis[$language], $configurations[$language]);
            } elseif (!empty($cancelledStatus) && $status == $cancelledStatus) {
                $this->setOrderCanceled($shopgateOrder, $merchantApis[$language], $configurations[$language]);
            }
        }
    }

    /**
     * check if a database table exist by the table name
     *
     * @param $tableName
     *
     * @return bool
     */
    private function tableExists($tableName)
    {
        $tableResult = xtc_db_query("SHOW TABLES LIKE '%{$tableName}%';");
        $result      = xtc_db_fetch_array($tableResult);

        return empty($result) ? false : true;
    }

    /**
     * overrides the "xtc_get_products_stock" function from the shop system
     *
     * @param $products_id
     *
     * @return mixed
     */
    private function xtc_get_products_stock($products_id)
    {
        $products_id  = xtc_get_prid($products_id);
        $stock_query  = xtc_db_query(
            "select products_quantity from " . TABLE_PRODUCTS
            . " where products_id = '" . $products_id . "'"
        );
        $stock_values = xtc_db_fetch_array($stock_query);

        return $stock_values['products_quantity'];
    }

    /**
     * Requires all files the plugin depends on with require_once.
     */
    private function requireFiles()
    {
        // xtcommerce files
        $inc_path = $this->getPath('inc');
        require_once($inc_path . 'xtc_validate_password.inc.php');
        require_once($inc_path . 'xtc_format_price_order.inc.php');
        require_once($inc_path . 'xtc_get_tax_class_id.inc.php');
        require_once($this->getPath('includes/classes') . 'xtcPrice.php');

        if (file_exists($inc_path . 'xtc_db_prepare_input.inc.php')) {
            require_once($inc_path . 'xtc_db_prepare_input.inc.php');
        } else if (file_exists($inc_path . 'cseo_dbv.inc.php')) {
            // this one's for commerce:SEO > V3
            require_once($inc_path . 'cseo_dbv.inc.php');
        } else {
            // this one's for commerce:SEO
            require_once($inc_path . 'cseo_db.inc.php');
        }

        // language files
        if (!defined('DIR_FS_LANGUAGES')) {
            define('DIR_FS_LANGUAGES', rtrim(DIR_FS_CATALOG, '/') . '/lang/');
        }

        if (file_exists($this->getPath('lang') . $this->language . '/admin/categories.php')) {
            // XTCOMMERCE-217: In commerce:seo v2next 2.5.14 there is no such file
            require_once($this->getPath('lang') . $this->language . '/admin/categories.php');
        }

        if (file_exists($this->getPath('lang') . $this->language . '/admin/content_manager.php')) {
            // XTCOMMERCE-217: In commerce:seo v2next 2.5.14 there is no such file
            require_once($this->getPath('lang') . $this->language . '/admin/content_manager.php');
        }

        require_once($this->getPath('shopgate/base/lang') . $this->language . '/admin/' . $this->language . '.php');

        // Shopgate plugin configuration
        require_once($this->getPath('shopgate/base'). 'shopgate_config.php');
    }

    /**
     * Gives you the full path if root constant exits,
     * else gives you a relative path
     *
     * @param $relPath - folder path relative to the root of the site
     *
     * @return string
     */
    private function getPath($relPath)
    {
        $relPath = trim($relPath, DS);

        if (defined('DIR_FS_CATALOG')) {
            $path = DIR_FS_CATALOG . $relPath;
        } else {
            $path = dirname(__FILE__) . "/../{$relPath}";
        }
        return $path . DS;
    }
}

/**
 * Class ShopgateXtcMapper
 */
class ShopgateXtcMapper
{

    /**
     * The countries with non-ISO-3166-2 state codes in xt:Commerce 3 are mapped here.
     *
     * @var string[][]
     */
    protected static $stateCodesByCountryCode = array(
        'DE' => array(
            "BW" => "BAW",
            "BY" => "BAY",
            "BE" => "BER",
            "BB" => "BRG",
            "HB" => "BRE",
            "HH" => "HAM",
            "HE" => "HES",
            "MV" => "MEC",
            "NI" => "NDS",
            "NW" => "NRW",
            "RP" => "RHE",
            "SL" => "SAR",
            "SN" => "SAS",
            "ST" => "SAC",
            "SH" => "SCN",
            "TH" => "THE",
        ),
        "AT" => array(
            "1" => "BL",
            "2" => "KN",
            "3" => "NO",
            "4" => "OO",
            "5" => "SB",
            "6" => "ST",
            "7" => "TI",
            "8" => "VB",
            "9" => "WI",
        ),
        //"CH" => ist in xt:commerce bereits korrekt
        //"US" => ist in xt:commerce bereits korrekt
    );

    /**
     * Finds the corresponding Shopgate state code for a given xt:Commerce 3 state code (zone_code).
     *
     * @param string $countryCode  The code of the country to which the state belongs
     * @param string $xtcStateCode The code of the state / zone as found in the default "zones" table of xt:Commerce 3
     *
     * @return string The state code as defined at Shopgate Wiki
     *
     * @throws ShopgateLibraryException if one of the given codes is unknown
     */
    public static function getShopgateStateCode($countryCode, $xtcStateCode)
    {
        $countryCode  = strtoupper($countryCode);
        $xtcStateCode = strtoupper($xtcStateCode);

        if (!isset(self::$stateCodesByCountryCode[$countryCode])) {
            return $countryCode . '-' . $xtcStateCode;
        }

        $codes = array_flip(self::$stateCodesByCountryCode[$countryCode]);
        if (!isset($codes[$xtcStateCode])) {
            return $countryCode . '-' . $xtcStateCode;
        }

        $stateCode = $codes[$xtcStateCode];

        return $countryCode . '-' . $stateCode;
    }

    /**
     * Finds the corresponding xt:Commerce 3 state code (zone_code) for a given Shopgate state code
     *
     * @param string $shopgateStateCode The Shopgate state code as defined at Shopgate Wiki
     *
     * @return string The zone code for xt:Commerce 3
     *
     * @throws ShopgateLibraryException if the given code is unknown
     */
    public static function getXtcStateCode($shopgateStateCode)
    {
        $splitCodes = null;
        preg_match(
            '/^([A-Z]{2})\-([A-Z]{2})$/', $shopgateStateCode, $splitCodes
        );

        if (empty($splitCodes) || empty($splitCodes[1])
            || empty($splitCodes[2])
        ) {
            return null;
        }

        if (!isset(self::$stateCodesByCountryCode[$splitCodes[1]])
            || !isset(self::$stateCodesByCountryCode[$splitCodes[1]][$splitCodes[2]])
        ) {
            //throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_UNKNOWN_STATE_CODE, 'Code: '.$shopgateStateCode);
            return $splitCodes[2];
        } else {
            return self::$stateCodesByCountryCode[$splitCodes[1]][$splitCodes[2]];
        }
    }
}
