<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author             Shopgate GmbH <interfaces@shopgate.com>
 * @coversDefaultClass ShopgateCartHelper
 */

namespace Shopgate\Xtcommerce3\Test\Tracking\Entity;

use Shopgate\Xtcommerce3\Tracking\Entity\Cart;
use Shopgate\Xtcommerce3\Tracking\ReadModel\CartDiff;
use Shopgate\Xtcommerce3\Tracking\ReadModel\CartItem;

class CartTest extends \PHPUnit_Framework_TestCase
{
    /** @var Cart */
    protected $class;

    /**
     * Initializing main test class
     */
    public function setUp()
    {
        $this->class = new Cart();
    }

    /**
     * @param array $cartProducts
     * @param array $expectedItems
     *
     * @covers       Cart::update()
     * @dataProvider updateProvider
     */
    public function testUpdate(
        $cartProducts,
        $expectedItems
    ) {
        $cartMock =
            $this->getMockBuilder('shoppingCart')
                 ->disableOriginalConstructor()
                 ->setMethods(array('get_products'))
                 ->getMock();
        $cartMock->method('get_products')->willReturn($cartProducts);
        $this->class->update($cartMock);

        $this->assertEquals($expectedItems, $this->class->getItems());
    }

    /**
     * @param array $cartProductsOld
     * @param array $cartProductsNew
     * @param array $expectedDifferences
     *
     * @covers       Cart::getDifferences()
     * @dataProvider getDifferencesProvider
     */
    public function testGetDifferences(
        $cartProductsOld,
        $cartProductsNew,
        $expectedDifferences
    ) {
        $cartMockOld =
            $this->getMockBuilder('shoppingCart')
                 ->disableOriginalConstructor()
                 ->setMethods(array('get_products'))
                 ->getMock();
        $cartMockOld->method('get_products')->willReturn($cartProductsOld);
        $this->class->update($cartMockOld);

        $cartMockNew =
            $this->getMockBuilder('shoppingCart')
                 ->disableOriginalConstructor()
                 ->setMethods(array('get_products'))
                 ->getMock();
        $cartMockNew->method('get_products')->willReturn($cartProductsNew);

        $this->assertEquals($expectedDifferences, $this->class->getDifferences($cartMockNew));
    }

    /**
     * @param array $cartProducts
     * @param int   $itemId
     * @param array $expectedItem
     *
     * @covers       Cart::getItem()
     * @dataProvider getItemProvider
     */
    public function testGetItem(
        $cartProducts,
        $itemId,
        $expectedItem
    ) {
        $cartMockOld =
            $this->getMockBuilder('shoppingCart')
                 ->disableOriginalConstructor()
                 ->setMethods(array('get_products'))
                 ->getMock();
        $cartMockOld->method('get_products')->willReturn($cartProducts);
        $this->class->update($cartMockOld);

        $this->assertEquals($expectedItem, $this->class->getItem($itemId));
    }

    /**
     * @param array $cartProducts
     * @param int   $expectedAmount
     *
     * @covers       Cart::getAmountItems()
     * @dataProvider getAmountItemsProvider
     */
    public function testGetAmountItems(
        $cartProducts,
        $expectedAmount
    ) {
        $cartMockOld =
            $this->getMockBuilder('shoppingCart')
                 ->disableOriginalConstructor()
                 ->setMethods(array('get_products'))
                 ->getMock();
        $cartMockOld->method('get_products')->willReturn($cartProducts);
        $this->class->update($cartMockOld);

        $this->assertEquals($expectedAmount, $this->class->getAmountItems());
    }

    /**
     * @return array
     */
    public function updateProvider()
    {
        return array(
            'Cart without items'         => array(
                false,
                array(),
            ),
            'Cart with one item'         => array(
                array(array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50)),
                array(1 => new CartItem(1, 'name', 5, 25000)),
            ),
            'Cart with two items'        => array(
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                ),
                array(
                    1 => new CartItem(1, 'name', 5, 25000),
                    2 => new CartItem(2, 'another name', 20, 60140),
                ),
            ),
            'Cart with a duplicate item' => array(
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                    array('id' => 1, 'name' => 'name', 'quantity' => 10, 'price' => 100),
                ),
                array(
                    1 => new CartItem(1, 'name', 15, 125000),
                    2 => new CartItem(2, 'another name', 20, 60140),
                ),
            ),
        );
    }

    /**
     * Set up fake items
     *
     * @return array
     */
    public function getDifferencesProvider()
    {
        return array(
            'Cart without items, add one item'             => array(
                false,
                array(array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50)),
                array(1 => new CartDiff(1, 5, 25000)),
            ),
            'Cart without items, no changes'               => array(
                array(array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50)),
                array(array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50)),
                array(),
            ),
            'Cart with one item, add one item'             => array(
                array(array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50)),
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                ),
                array(2 => new CartDiff(2, 20, 60140)),
            ),
            'Cart with two items, remove one item'         => array(
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                ),
                array(array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50)),
                array(2 => new CartDiff(2, -20, -60140)),
            ),
            'Cart with two items, change both quantities'  => array(
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                ),
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 10, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 10, 'price' => 30.07),
                ),
                array(
                    1 => new CartDiff(1, 5, 25000),
                    2 => new CartDiff(2, -10, -30070),
                ),
            ),
            'Cart with two items, add a duplicate item'    => array(
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                ),
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                    array('id' => 1, 'name' => 'name', 'quantity' => 10, 'price' => 100),
                ),
                array(
                    1 => new CartDiff(1, 10, 100000),
                ),
            ),
            'Cart with two items, remove a duplicate item' => array(
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                    array('id' => 1, 'name' => 'name', 'quantity' => 10, 'price' => 100),
                ),
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                ),
                array(
                    1 => new CartDiff(1, -10, -100000),
                ),
            ),
        );
    }

    /**
     * Set up fake items
     *
     * @return array
     */
    public function getItemProvider()
    {
        return array(
            'first item of two'        => array(
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                ),
                1,
                new CartItem(1, 'name', 5, 25000),
            ),
            'second item of two'       => array(
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                ),
                2,
                new CartItem(2, 'another name', 20, 60140),
            ),
            'non existing item of two' => array(
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                ),
                3,
                null,
            ),
        );
    }

    /**
     * Set up fake items
     *
     * @return array
     */
    public function getAmountItemsProvider()
    {
        return array(
            'Cart without items'         => array(
                false,
                0,
            ),
            'Cart with one item'         => array(
                array(array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50)),
                25000,
            ),
            'Cart with two items'        => array(
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                ),
                85140,
            ),
            'Cart with a duplicate item' => array(
                array(
                    array('id' => 1, 'name' => 'name', 'quantity' => 5, 'price' => 50),
                    array('id' => 2, 'name' => 'another name', 'quantity' => 20, 'price' => 30.07),
                    array('id' => 1, 'name' => 'name', 'quantity' => 10, 'price' => 100),
                ),
                185140,
            ),
        );
    }
}