<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Xtcommerce3\Test\Tracking\Repository\Globals;

use Shopgate\FacebookPixelLibrary\Event\ValueObject\ContentType;
use Shopgate\Xtcommerce3\Tracking\Entity\Cart;
use Shopgate\Xtcommerce3\Tracking\Repository\Globals\Datasource\ShadowCart;
use Shopgate\Xtcommerce3\Tracking\Repository\Globals\Datasource\Xtcommerce3;
use Shopgate\Xtcommerce3\Tracking\Repository\Globals\EventData;

class EventDataTest extends \PHPUnit_Framework_TestCase
{
    /** @var EventData */
    private $subjectUnderTest;

    /** @var Xtcommerce3 | \PHPUnit_Framework_MockObject_MockObject */
    private $xtc3DatasourceMock;

    /** @var ShadowCart | \PHPUnit_Framework_MockObject_MockObject */
    private $shadowCartDatasourceMock;

    protected function setUp()
    {
        $this->xtc3DatasourceMock = $this
            ->getMockBuilder('\Shopgate\Xtcommerce3\Tracking\Repository\Globals\Datasource\Xtcommerce3')
            ->disableOriginalConstructor()
            ->getMock()
        ;

        $this->shadowCartDatasourceMock = $this
            ->getMockBuilder('\Shopgate\Xtcommerce3\Tracking\Repository\Globals\Datasource\ShadowCart')
            ->disableOriginalConstructor()
            ->getMock()
        ;

        $this->subjectUnderTest = new EventData($this->xtc3DatasourceMock, $this->shadowCartDatasourceMock);
    }

    /**
     * @param string      $expectedScriptName
     * @param string|null $scriptName
     *
     * @dataProvider provideScriptNameFixtures
     */
    public function testGetScriptName($expectedScriptName, $scriptName)
    {
        $this
            ->xtc3DatasourceMock
            ->expects($this->once())
            ->method('getScriptName')
            ->willReturn($scriptName)
        ;

        $this->xtc3DatasourceMock->getTitle();

        $this->assertEquals($expectedScriptName, $this->subjectUnderTest->getScriptName());
    }

    /**
     * @return array
     */
    public function provideScriptNameFixtures()
    {
        return array(
            'not set'   => array(
                '',
                null,
            ),
            'empty'     => array(
                '',
                '',
            ),
            'file name' => array(
                'stuff.php',
                'stuff.php',
            ),
            'file path' => array(
                'stuff.php',
                'path/to/stuff.php',
            )
        );
    }

    /**
     * @param string      $expectedPageName
     * @param string|null $title
     * @param string|null $productMetaTitle
     * @param string|null $productName
     * @param string|null $productSku
     *
     * @dataProvider providePageNameFixtures
     */
    public function testGetPageName(
        $expectedPageName,
        $title = null,
        $productMetaTitle = null,
        $productName = null,
        $productSku = null
    ) {
        $this
            ->xtc3DatasourceMock
            ->expects($this->any())
            ->method('getTitle')
            ->willReturn($title)
        ;

        $this
            ->xtc3DatasourceMock
            ->expects($this->once())
            ->method('getProductMetaTitle')
            ->willReturn($productMetaTitle)
        ;

        $this
            ->xtc3DatasourceMock
            ->expects($this->once())
            ->method('getProductName')
            ->willReturn($productName)
        ;

        $this
            ->xtc3DatasourceMock
            ->expects($this->once())
            ->method('getProductSku')
            ->willReturn($productSku)
        ;

        $this->assertEquals($expectedPageName, $this->subjectUnderTest->getPageName());
    }

    /**
     * @return array
     */
    public function providePageNameFixtures()
    {
        $titleConstant    = '  Store title  ';
        $productMetaTitle = '  Meta title  ';
        $productName      = '  Product name  ';
        $productSku       = '  Product SKU  ';

        $trimmedTitleConstant    = trim($titleConstant);
        $trimmedProductMetaTitle = trim($productMetaTitle);
        $trimmedProductName      = trim($productName);
        $trimmedProductSku       = trim($productSku);

        return array(
            'no title set'                                                                            => array(
                '',
            ),
            'regular page, not a product'                                                             => array(
                $trimmedTitleConstant,
                $titleConstant,
            ),
            'product detail page - 1 title, 0 product meta title, 0 product meta name, 1 product SKU' => array(
                "{$trimmedTitleConstant} {$trimmedProductSku}",
                $titleConstant,
                null,
                null,
                $productSku,
            ),
            'product detail page - 1 title, 0 product meta title, 1 product meta name, 0 product SKU' => array(
                "{$trimmedTitleConstant} {$trimmedProductName}",
                $titleConstant,
                null,
                $productName,
            ),
            'product detail page - 1 title, 0 product meta title, 1 product meta name, 1 product SKU' => array(
                "{$trimmedTitleConstant} {$trimmedProductName} {$trimmedProductSku}",
                $titleConstant,
                null,
                $productName,
                $productSku,
            ),
            'product detail page - 1 title, 1 product meta title, 0 product meta name, 0 product SKU' => array(
                "{$trimmedTitleConstant} {$trimmedProductMetaTitle}",
                $titleConstant,
                $productMetaTitle,
            ),
            'product detail page - 1 title, 1 product meta title, 0 product meta name, 1 product SKU' => array(
                "{$trimmedTitleConstant} {$trimmedProductMetaTitle} {$trimmedProductSku}",
                $titleConstant,
                $productMetaTitle,
                null,
                $productSku,
            ),
            'product detail page - 1 title, 1 product meta title, 1 product meta name, 0 product SKU' => array(
                "{$trimmedTitleConstant} {$trimmedProductMetaTitle} {$trimmedProductName}",
                $titleConstant,
                $productMetaTitle,
                $productName,
            ),
            'product detail page - 1 title, 1 product meta title, 1 product meta name, 1 product SKU' => array(
                "{$trimmedTitleConstant} {$trimmedProductMetaTitle} {$trimmedProductName} {$trimmedProductSku}",
                $titleConstant,
                $productMetaTitle,
                $productName,
                $productSku
            ),
        );
    }

    /**
     * @param string      $expectedCurrencyCode
     * @param string|null $currencyCode
     *
     * @dataProvider provideCurrencyFixtures
     */
    public function testGetCurrencyCode($expectedCurrencyCode, $currencyCode)
    {
        $this
            ->xtc3DatasourceMock
            ->expects($this->once())
            ->method('getCurrencyCode')
            ->willReturn($currencyCode)
        ;

        $this->assertEquals($expectedCurrencyCode, $this->subjectUnderTest->getCurrencyCode());
    }

    /**
     * @return array
     */
    public function provideCurrencyFixtures()
    {
        return array(
            'none set' => array(
                null,
                ''
            ),
            'empty'    => array(
                '',
                ''
            ),
            'set'      => array(
                'USD',
                'USD'
            )
        );
    }

    /**
     * @param \shoppingCart      $expectedCart
     * @param \shoppingCart|null $nativeCart
     * @param \shoppingCart|null $newCart
     *
     * @dataProvider provideNativeCartFixtures
     */
    public function testGetNativeCart(
        \shoppingCart $expectedCart,
        \shoppingCart $nativeCart = null,
        \shoppingCart $newCart = null
    ) {
        $this
            ->xtc3DatasourceMock
            ->expects($this->once())
            ->method('getNativeCart')
            ->willReturn($nativeCart)
        ;

        $this
            ->xtc3DatasourceMock
            ->expects(null === $nativeCart ? $this->once() : $this->never())
            ->method('createNativeCart')
            ->willReturn($newCart)
        ;

        $this->assertSame($expectedCart, $this->subjectUnderTest->getNativeCart());
    }

    /**
     * @return array
     */
    public function provideNativeCartFixtures()
    {
        $newCart     = $this->getMockBuilder('\shoppingCart')->getMock();
        $defaultCart = $this->getMockBuilder('\shoppingCart')->getMock();

        return array(
            'none set' => array(
                $newCart,
                null,
                $newCart,
            ),
            'set'      => array(
                $defaultCart,
                $defaultCart
            )
        );
    }

    /**
     * @param Cart      $expectedCart
     * @param Cart|null $sessionCart
     * @param Cart|null $newCart
     *
     * @dataProvider provideShadowCartFixtures
     */
    public function testGetShadowCart(Cart $expectedCart, Cart $sessionCart = null, Cart $newCart = null)
    {
        $this
            ->shadowCartDatasourceMock
            ->expects($this->once())
            ->method('getCart')
            ->willReturn($sessionCart)
        ;

        $this
            ->shadowCartDatasourceMock
            ->expects(null === $sessionCart ? $this->once() : $this->never())
            ->method('createCart')
            ->willReturn($newCart)
        ;

        $this->assertSame($expectedCart, $this->subjectUnderTest->getShadowCart());
    }

    /**
     * @return array
     */
    public function provideShadowCartFixtures()
    {
        $sessionCart = new Cart();
        $newCart     = new Cart();

        return array(
            'not set' => array(
                $newCart,
                null,
                $newCart
            ),
            'set'     => array(
                $sessionCart,
                $sessionCart
            )
        );
    }

    public function testPersistShadowCart()
    {
        $cartToBePersisted = new Cart();

        $this
            ->shadowCartDatasourceMock
            ->expects($this->once())
            ->method('persistCart')
            ->with($cartToBePersisted)
        ;

        $this->subjectUnderTest->persistShadowCart($cartToBePersisted);
    }

    public function testGetCurrentProductContentType()
    {
        $this->assertEquals($this->subjectUnderTest->getCurrentProductContentType()->getString(), ContentType::PRODUCT);
    }

    /**
     * @param string      $expectedSku
     * @param string|null $sku
     *
     * @dataProvider provideCurrentProductSkuFixtures
     */
    public function testGetCurrentProductSku($expectedSku, $sku = null)
    {
        $this
            ->xtc3DatasourceMock
            ->expects($this->once())
            ->method('getProductSku')
            ->willReturn($sku)
        ;

        $this->assertSame($expectedSku, $this->subjectUnderTest->getCurrentProductSku());
    }

    /**
     * @return array
     */
    public function provideCurrentProductSkuFixtures()
    {
        return array(
            'not a product'       => array(
                '',
                null,
            ),
            'product without SKU' => array(
                '',
                ''
            ),
            'product with SKU'    => array(
                '12345ab',
                '12345ab'
            )
        );
    }

    /**
     * @param int      $expectedId
     * @param int|null $id
     *
     * @dataProvider provideCurrentProductIdFixtures
     */
    public function testGetCurrentProductId($expectedId, $id = null)
    {
        $this
            ->xtc3DatasourceMock
            ->expects($this->once())
            ->method('getProductId')
            ->willReturn($id)
        ;

        $this->assertSame($expectedId, $this->subjectUnderTest->getCurrentProductId());
    }

    /**
     * @return array
     */
    public function provideCurrentProductIdFixtures()
    {
        return array(
            'not a product'      => array(
                '',
                null,
            ),
            'product without ID' => array(
                '0',
                0,
            ),
            'product with ID'    => array(
                '123',
                123,
            )
        );
    }

    /**
     * @param string      $expectedProductName
     * @param string|null $productMetaTitle
     * @param string|null $productName
     * @param string|null $productSku
     *
     * @dataProvider provideCurrentProductNameFixtures
     */
    public function estGetCurrentProductName(
        $expectedProductName,
        $productMetaTitle = null,
        $productName = null,
        $productSku = null
    ) {
        $this
            ->xtc3DatasourceMock
            ->expects($this->once())
            ->method('getProductMetaTitle')
            ->willReturn($productMetaTitle)
        ;

        $this
            ->xtc3DatasourceMock
            ->expects($this->once())
            ->method('getProductName')
            ->willReturn($productName)
        ;

        $this
            ->xtc3DatasourceMock
            ->expects($this->once())
            ->method('getProductSku')
            ->willReturn($productSku)
        ;

        $this->assertSame($expectedProductName, $this->subjectUnderTest->getCurrentProductName());
    }

    /**
     * @return array
     */
    public function provideCurrentProductNameFixtures()
    {
        $productMetaTitle = '  Meta title  ';
        $productName      = '  Product name  ';
        $productSku       = '  Product SKU  ';

        $trimmedProductMetaTitle = trim($productMetaTitle);
        $trimmedProductName      = trim($productName);
        $trimmedProductSku       = trim($productSku);

        return array(
            'regular page, not a product'                                                    => array(
                '',
            ),
            'all empty'                                                                      => array(
                '',
                '',
                '',
                '',
            ),
            'product detail page - 0 product meta title, 0 product meta name, 1 product SKU' => array(
                "{$trimmedProductSku}",
                null,
                null,
                $productSku,
            ),
            'product detail page - 0 product meta title, 1 product meta name, 0 product SKU' => array(
                "{$trimmedProductName}",
                null,
                $productName,
            ),
            'product detail page - 0 product meta title, 1 product meta name, 1 product SKU' => array(
                "{$trimmedProductName} {$trimmedProductSku}",
                null,
                $productName,
                $productSku,
            ),
            'product detail page - 1 product meta title, 0 product meta name, 0 product SKU' => array(
                "{$trimmedProductMetaTitle}",
                $productMetaTitle,
            ),
            'product detail page - 1 product meta title, 0 product meta name, 1 product SKU' => array(
                "{$trimmedProductMetaTitle} {$trimmedProductSku}",
                $productMetaTitle,
                null,
                $productSku,
            ),
            'product detail page - 1 product meta title, 1 product meta name, 0 product SKU' => array(
                "{$trimmedProductMetaTitle} {$trimmedProductName}",
                $productMetaTitle,
                $productName,
            ),
            'product detail page - 1 product meta title, 1 product meta name, 1 product SKU' => array(
                "{$trimmedProductMetaTitle} {$trimmedProductName} {$trimmedProductSku}",
                $productMetaTitle,
                $productName,
                $productSku
            ),
        );
    }

    /**
     * @param int        $expectedPrice
     * @param float|null $productPrice
     *
     * @dataProvider provideCurrentProductPriceFixtures
     */
    public function testGetCurrentProductPrice($expectedPrice, $productPrice = null)
    {
        $this
            ->xtc3DatasourceMock
            ->expects($this->once())
            ->method('getProductPrice')
            ->willReturn($productPrice)
        ;

        $this->assertSame($expectedPrice, $this->subjectUnderTest->getCurrentProductPrice());
    }

    /**
     * @return array
     */
    public function provideCurrentProductPriceFixtures()
    {
        return array(
            'not a product'                       => array(
                0,
                null
            ),
            'product with price empty string'     => array(
                0,
                ''
            ),
            'product with price'                  => array(
                1322,
                13.22
            ),
            'product with price string'           => array(
                1344,
                '13.44'
            ),
            'product with roundable price'        => array(
                1345,
                13.445
            ),
            'product with roundable price string' => array(
                1355,
                '13.545'
            ),
        );
    }
}