<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Xtcommerce3\Test\Tracking\Service;


use Shopgate\FacebookPixelLibrary\Event\Event\AddToCart;
use Shopgate\FacebookPixelLibrary\Event\Event\FacebookTrackingEvent;
use Shopgate\FacebookPixelLibrary\Event\Event\InitiateCheckout;
use Shopgate\FacebookPixelLibrary\Event\Event\Purchase;
use Shopgate\FacebookPixelLibrary\Event\Event\ViewContent;
use Shopgate\FacebookPixelLibrary\Event\Service\EventRenderer as FacebookEventRenderer;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\ContentId;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\ContentName;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\ContentType;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\Currency;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\NumItems;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\Value;
use Shopgate\Xtcommerce3\Tracking\Entity\Cart;
use Shopgate\Xtcommerce3\Tracking\ReadModel\CartDiff;
use Shopgate\Xtcommerce3\Tracking\ReadModel\CartItem;
use Shopgate\Xtcommerce3\Tracking\Repository\EventData;
use Shopgate\Xtcommerce3\Tracking\Service\EventRenderer;

class EventRendererTest extends \PHPUnit_Framework_TestCase
{
    /** @var EventRenderer */
    private $subjectUnderTest;

    /** @var FacebookEventRenderer | \PHPUnit_Framework_MockObject_MockObject */
    private $rendererMock;

    /** @var EventData | \PHPUnit_Framework_MockObject_MockObject */
    private $eventDataRepositoryMock;

    public function setUp()
    {
        $this->rendererMock = $this
            ->getMockBuilder("Shopgate\FacebookPixelLibrary\Event\Service\EventRenderer")
            ->getMockForAbstractClass()
        ;

        $this->eventDataRepositoryMock = $this
            ->getMockBuilder("Shopgate\Xtcommerce3\Tracking\Repository\EventData")
            ->getMockForAbstractClass()
        ;

        $this->subjectUnderTest = new EventRenderer($this->rendererMock, $this->eventDataRepositoryMock);
    }

    /**
     * @param string                  $scriptName
     * @param string                  $pageName
     * @param string[][]              $currencyCodes
     * @param string[][]              $productNames
     * @param int[][]                 $productPrices
     * @param ContentType[][]         $productContentTypes
     * @param int[][]                 $productIds
     * @param FacebookTrackingEvent[] $expectedEvents
     * @param \shoppingCart           $nativeCart
     * @param Cart                    $shadowCartMock
     *
     * @dataProvider provideTestGetSnippetFixtures
     */
    public function testGetSnippet(
        $scriptName,
        $pageName,
        array $currencyCodes,
        array $productNames,
        array $productPrices,
        array $productContentTypes,
        array $productIds,
        array $expectedEvents,
        \shoppingCart $nativeCart,
        Cart $shadowCartMock
    ) {
        // set up the event data repository mock
        $this
            ->eventDataRepositoryMock
            ->expects($this->exactly(count($currencyCodes)))
            ->method("getCurrencyCode")
            ->willReturnMap(array_reverse($currencyCodes))
        ;

        $this
            ->eventDataRepositoryMock
            ->expects($this->once())
            ->method('getPageName')
            ->willReturn($pageName)
        ;

        $this
            ->eventDataRepositoryMock
            ->expects($this->any())
            ->method("getShadowCart")
            ->willReturn($shadowCartMock)
        ;

        $this
            ->eventDataRepositoryMock
            ->expects($this->any())
            ->method('getNativeCart')
            ->willReturn($nativeCart)
        ;

        $this
            ->eventDataRepositoryMock
            ->expects($this->once())
            ->method("getScriptName")
            ->willReturn($scriptName)
        ;

        $this
            ->eventDataRepositoryMock
            ->expects($this->exactly(count($productPrices)))
            ->method('getCurrentProductPrice')
            ->willReturnMap(array_reverse($productPrices))
        ;

        $this
            ->eventDataRepositoryMock
            ->expects($this->exactly(count($productNames)))
            ->method('getCurrentProductName')
            ->willReturnMap(array_reverse($productNames))
        ;

        $this
            ->eventDataRepositoryMock
            ->expects($this->exactly(count($productContentTypes)))
            ->method('getCurrentProductContentType')
            ->willReturnMap(array_reverse($productContentTypes))
        ;

        $this
            ->eventDataRepositoryMock
            ->expects($this->exactly(count($productIds)))
            ->method('getCurrentProductId')
            ->willReturnMap(array_reverse($productIds))
        ;

        // set up the FB pixel renderer mock
        $this
            ->rendererMock
            ->expects($this->once())
            ->method("render")
            ->with($expectedEvents)
        ;

        $this->subjectUnderTest->getSnippet();
    }

    /**
     * @return array
     */
    public function provideTestGetSnippetFixtures()
    {
        $defaultCurrencyCode       = "USD";
        $defaultPageName           = 'page title';
        $defaultProductName        = 'product name';
        $defaultProductContentType = new ContentType(ContentType::PRODUCT);
        $defaultProductPrice       = 2345;
        $defaultProductPriceString = '23.45';

        return array(
            'default page'                     => array(
                'index.php',
                $defaultPageName,
                array(array($defaultCurrencyCode)),
                array(),
                array(),
                array(),
                array(),
                array(
                    new ViewContent(
                        null,
                        new Currency($defaultCurrencyCode),
                        new ContentName($defaultPageName),
                        null,
                        array()
                    )
                ),
                $this->getDefaultNativeCartMock(),
                $this->getDefaultShadowCartMock()
            ),
            'product detail page'              => array(
                'product_info.php',
                '',
                array(array($defaultCurrencyCode), array($defaultCurrencyCode)),
                array(array($defaultProductName)),
                array(array($defaultProductPrice)),
                array(array($defaultProductContentType)),
                array(array('123')),
                array(
                    new ViewContent(
                        new Value($defaultProductPriceString),
                        new Currency($defaultCurrencyCode),
                        new ContentName($defaultProductName),
                        $defaultProductContentType,
                        array(new ContentId('123'))
                    )
                ),
                $this->getDefaultNativeCartMock(),
                $this->getDefaultShadowCartMock()
            ),
            'checkout initiated page'          => array(
                'checkout_shipping.php',
                $defaultPageName,
                array(array($defaultCurrencyCode), array($defaultCurrencyCode)),
                array(),
                array(),
                array(),
                array(),
                array(
                    new ViewContent(
                        null,
                        new Currency($defaultCurrencyCode),
                        new ContentName($defaultPageName),
                        null,
                        array()
                    ),
                    new InitiateCheckout(
                        new Value('23.33'),
                        new Currency($defaultCurrencyCode),
                        null,
                        null,
                        array(new ContentId('123'), new ContentId('789')),
                        new NumItems(999)
                    )
                ),
                $this->getCheckoutInitiatedNativeCartMock(),
                $this->getDefaultShadowCartMock()
            ),
            'purchase / checkout success page' => array(
                'checkout_success.php',
                $defaultPageName,
                array(array($defaultCurrencyCode), array($defaultCurrencyCode)),
                array(),
                array(),
                array(),
                array(),
                array(
                    new ViewContent(
                        null,
                        new Currency($defaultCurrencyCode),
                        new ContentName($defaultPageName),
                        null,
                        array()
                    ),
                    new Purchase(
                        new Value('135.82'),
                        new Currency($defaultCurrencyCode),
                        null,
                        null,
                        array(new ContentId('123'), new ContentId('127')),
                        new NumItems(5)
                    )
                ),
                $this->getDefaultNativeCartMock(),
                $this->getPurchaseShadowCartMock()
            ),
            'add to cart'                      => array(
                'index.php',
                $defaultPageName,
                array(array($defaultCurrencyCode), array($defaultCurrencyCode)),
                array(),
                array(),
                array(array($defaultProductContentType)),
                array(),
                array(
                    new ViewContent(
                        null,
                        new Currency($defaultCurrencyCode),
                        new ContentName($defaultPageName),
                        null,
                        array()
                    ),
                    new AddToCart(
                        new Value('14.54'),
                        new Currency($defaultCurrencyCode),
                        null,
                        $defaultProductContentType,
                        array(new ContentId('123'), new ContentId('357'))
                    )
                ),
                $this->getDefaultNativeCartMock(),
                $this->getAddToCartShadowCartMock()
            ),
            'add to cart with no differences'  => array(
                'index.php',
                $defaultPageName,
                array(array($defaultCurrencyCode)),
                array(),
                array(),
                array(),
                array(),
                array(
                    new ViewContent(
                        null,
                        new Currency($defaultCurrencyCode),
                        new ContentName($defaultPageName),
                        null,
                        array()
                    ),
                ),
                $this->getDefaultNativeCartMock(),
                $this->getAddToCartZeroQuantityChangesShadowCartMock()
            ),
        );
    }

    /**
     * @return \shoppingCart | \PHPUnit_Framework_MockObject_MockObject
     */
    private function getDefaultNativeCartMock()
    {
        return $this->getMockBuilder('\shoppingCart')->getMock();
    }

    /**
     * @return \shoppingCart | \PHPUnit_Framework_MockObject_MockObject
     */
    private function getCheckoutInitiatedNativeCartMock()
    {
        $nativeCartMock = $this
            ->getMockBuilder('\shoppingCart')
            ->setMethods(array('show_total', 'get_products', 'count_contents'))
            ->getMock()
        ;
        $nativeCartMock
            ->expects($this->once())
            ->method('show_total')
            ->willReturn(23.33)
        ;
        $nativeCartMock
            ->expects($this->once())
            ->method('get_products')
            ->willReturn(array(
                array(
                    'id' => 123
                ),
                array(
                    'id' => 789
                )
            ))
        ;
        $nativeCartMock
            ->expects($this->once())
            ->method('count_contents')
            ->willReturn(999)
        ;

        return $nativeCartMock;
    }

    /**
     * @return Cart | \PHPUnit_Framework_MockObject_MockObject
     */
    private function getDefaultShadowCartMock()
    {
        $shadowCartMock = $this->getMockBuilder("\Shopgate\Xtcommerce3\Tracking\Entity\Cart")->getMock();

        $shadowCartMock
            ->expects($this->once())
            ->method("getDifferences")
            ->willReturn(array())
        ;

        $shadowCartMock
            ->expects($this->once())
            ->method("update")
        ;

        return $shadowCartMock;
    }

    /**
     * @return Cart | \PHPUnit_Framework_MockObject_MockObject
     */
    private function getPurchaseShadowCartMock()
    {
        $shadowCartMock = $this->getDefaultShadowCartMock();

        $shadowCartMock
            ->expects($this->once())
            ->method("getAmountItems")
            ->willReturn(13582)
        ;

        $shadowCartMock
            ->expects($this->exactly(2))
            ->method("getItems")
            ->willReturn(
                array(
                    new CartItem(
                        123,
                        'product 1',
                        3,
                        1178
                    ),
                    new CartItem(
                        127,
                        'product 4',
                        2,
                        1278
                    ),
                )
            )
        ;

        return $shadowCartMock;
    }

    /**
     * @return Cart | \PHPUnit_Framework_MockObject_MockObject
     */
    private function getAddToCartShadowCartMock()
    {
        $shadowCartMock = $this->getMockBuilder("\Shopgate\Xtcommerce3\Tracking\Entity\Cart")->getMock();

        $shadowCartMock
            ->expects($this->once())
            ->method("update")
        ;

        $shadowCartMock
            ->expects($this->once())
            ->method("getDifferences")
            ->willReturn(
                array(
                    new CartDiff(
                        123,
                        2,
                        1299
                    ),
                    new CartDiff(
                        234,
                        -1,
                        -199
                    ),
                    new CartDiff(
                        789,
                        0,
                        0
                    ),
                    new CartDiff(
                        357,
                        1,
                        155
                    )
                )
            )
        ;

        return $shadowCartMock;
    }


    /**
     * @return Cart | \PHPUnit_Framework_MockObject_MockObject
     */
    private function getAddToCartZeroQuantityChangesShadowCartMock()
    {
        $shadowCartMock = $this->getMockBuilder("\Shopgate\Xtcommerce3\Tracking\Entity\Cart")->getMock();

        $shadowCartMock
            ->expects($this->once())
            ->method("update")
        ;

        $shadowCartMock
            ->expects($this->once())
            ->method("getDifferences")
            ->willReturn(
                array(
                    new CartDiff(
                        123,
                        0,
                        0
                    ),
                    new CartDiff(
                        234,
                        0,
                        0
                    ),
                    new CartDiff(
                        789,
                        0,
                        0
                    ),
                    new CartDiff(
                        357,
                        0,
                        0
                    )
                )
            )
        ;

        return $shadowCartMock;
    }
}