# Merchant Facebook pixel library

## Installation
Enter dependency "shopgate/facebook-pixel-library" in your composer.json and perform a:

    composer install

or

    composer update shopgate/facebook-pixel-library

## Getting Started
### Rendering the Configuration Section
The configuration section ideally consists of a title, a sub-title and a bunch of "on/off" settings for the events that
should be listened for. The sub title has a label in case you have to provide one in the cart's configuration.

To get the settings, create an instance of the factory for the language you want the settings for and get the object:

    <?php
    $factoryEnglish = new Shopgate\FacebookPixelLibrary\Settings\Factory\SettingsSectionEnglishFactory();
    $factoryGerman = new Shopgate\FacebookPixelLibrary\Settings\Factory\SettingsSectionGermanFactory();
    
    $settingsEnglish = $factoryEnglish->getSettingsSectionForAllEvents();
    $settingsGerman = $factoryGerman->getSettingsSectionForAllEvents();

This will return an instance of Shopgate\FacebookPixelLibrary\Settings\Entity\SettingsSection. You should implement
Shopgate\FacebookPixelLibrary\Settings\Service\SettingsSectionRenderer, which defines a method render() that takes
a SettingsSection object and returns a string.

Ideally, the consumer should check whether a Facebook Pixel ID is actually set before displaying the menu. The check
should be done using an implementation of the Shopgate\FacebookPixel\Base\Repository\Settings interface.

### Rendering the Facebook Pixel
The Facebook Pixel should be rendered as Javascript snippet inside the &lt;head&gt;&lt;/head&gt; section of your store.
Contents of the snippet depend on the various events that can be reported to Facebook by using the pixel. Event classes
are located in Shopgate\FacebookPixel\Event\Event.

To have the event rendered, create an instance of the factory, get the renderer and call the render() method. The
renderer requires an object that implements the Shopgate\FacebookPixel\Base\Repository\Settings interface.

Example:

    <?php
    use Shopgate\FacebookPixelLibrary\Base\Repository\Settings;
    use Shopgate\FacebookPixel\Event\Event\ViewContent;
    use Shopgate\FacebookPixel\Event\Factory\JavaScriptTagRendererFactory
    use Shopgate\FacebookPixel\Event\ValueObject\Value;
    use Shopgate\FacebookPixel\Event\ValueObject\Currency;
    use Shopgate\FacebookPixel\Event\ValueObject\ContentName;
    use Shopgate\FacebookPixel\Event\ValueObject\ContentType;
    use Shopgate\FacebookPixel\Event\ValueObject\ContentId;
    
    $facebookPixelEvent = new Shopgate\FacebookPixel\Event\Event\ViewContent(
        new Value('32.99'), new Currency('EUR'),
        new ContentName('Quite an awesome product'),
        new ContentType(ContentType::PRODUCT),
        [new ContentId('product-code-abc')]
    );
    
    $pixelRendererFactory = new JavaScriptTagRendererFactory(
        // this will only work in PHP >= 7.1; but just to demonstrate the concept:
        new class implements Settings {
            public function getFacebookPixelId() { return 1337; }
            public function getFacebookPixelViewContentEventActive() { return true; }
            // ...
        }
    );
    
    $facebookPixelEvents            = array($facebookPixelEvent);
    $facebookPixelJavaScriptSnippet = $pixelRendererFactory->getRenderer()->render($facebookPixelEvents);
    ?>
    
    <html>
      <head>
        <meta charset="utf-8">
        <?php echo $facebookPixelJavaScriptSnippet; ?>
        <!-- more HTML headers ... -->
      </head>
      <body>
        <!-- page content ... -->
      </body>
    </html>

## Changelog

See [CHANGELOG.md](CHANGELOG.md) file for more information.