<?php

namespace Shopgate\FacebookPixelLibrary\Event\Service;

use Shopgate\FacebookPixelLibrary\Base\Repository\Settings;
use Shopgate\FacebookPixelLibrary\Event\Event\FacebookTrackingEvent;
use Shopgate\FacebookPixelLibrary\Event\Event\AddPaymentInfo;
use Shopgate\FacebookPixelLibrary\Event\Event\AddToCart;
use Shopgate\FacebookPixelLibrary\Event\Event\AddToWishlist;
use Shopgate\FacebookPixelLibrary\Event\Event\CompleteRegistration;
use Shopgate\FacebookPixelLibrary\Event\Event\InitiateCheckout;
use Shopgate\FacebookPixelLibrary\Event\Event\Lead;
use Shopgate\FacebookPixelLibrary\Event\Event\Purchase;
use Shopgate\FacebookPixelLibrary\Event\Event\Search;
use Shopgate\FacebookPixelLibrary\Event\Event\ViewContent;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\ContentId;

class ImageTag implements EventRenderer
{
    /** @var Settings */
    private $settings;

    /**
     * @param Settings $settings
     */
    public function __construct(Settings $settings)
    {
        $this->settings = $settings;
    }

    public function render(array $events)
    {
        $facebookPixelId = $this->settings->getFacebookPixelId();
        if (empty($facebookPixelId)) {
            return '';
        }

        $imageTags = array();
        /** @var FacebookTrackingEvent $event */
        foreach ($events as $event) {
            $imageTag =  $this->renderEvent($event, $facebookPixelId);
            if (!empty($imageTag)) {
                $imageTags[] = $imageTag;
            }
        }

        return implode("\n", $imageTags);
    }


    /**
     * @param FacebookTrackingEvent $event
     * @param int                   $facebookPixelId
     *
     * @return string|null
     */
    protected function renderEvent(FacebookTrackingEvent $event, $facebookPixelId)
    {
        $fullClassName     = get_class($event);
        $explodedClassName = explode('\\', $fullClassName);
        $className         = array_pop($explodedClassName);
        $renderFunction    = 'render' . $className;
        $isActiveFunction  = 'getFacebookPixel' . $className . 'EventActive';

        return ($this->settings->$isActiveFunction())
            ? '<img height="1" width="1" border="0" alt="" style="display:none" src="https://www.facebook.com/tr?id='
            . $facebookPixelId . '&ev=' . $className . $this->$renderFunction($event) . '&noscript=1"/>'
            : null;
    }

    /**
     * @param AddPaymentInfo $event
     *
     * @return string
     */
    protected function renderAddPaymentInfo(AddPaymentInfo $event)
    {
        $content = '';

        if (!empty($event->value)) {
            $content .= '&cd[value]=' . $event->value->getStringRawUrlEncoded();
        }
        if (!empty($event->currency)) {
            $content .= '&cd[currency]=' . $event->currency->getStringRawUrlEncoded();
        }
        if (!empty($event->content_category)) {
            $content .= '&cd[content_category]=' . $event->content_category->getStringRawUrlEncoded();
        }
        if (!empty($event->content_ids)) {
            $content .= '&cd[content_ids]=' . $this->getContentIdsCommaSeparated($event->content_ids);
        }

        return $content;
    }

    /**
     * @param AddToCart $event
     *
     * @return string
     */
    protected function renderAddToCart(AddToCart $event)
    {
        $content = '';

        if (!empty($event->value)) {
            $content .= '&cd[value]=' . $event->value->getStringRawUrlEncoded();
        }
        if (!empty($event->currency)) {
            $content .= '&cd[currency]=' . $event->currency->getStringRawUrlEncoded();
        }
        if (!empty($event->content_name)) {
            $content .= '&cd[content_name]=' . $event->content_name->getStringRawUrlEncoded();
        }
        if (!empty($event->content_type)) {
            $content .= '&cd[content_type]=' . $event->content_type->getStringRawUrlEncoded();
        }
        if (!empty($event->content_ids)) {
            $content .= '&cd[content_ids]=' . $this->getContentIdsCommaSeparated($event->content_ids);
        }

        return $content;
    }

    /**
     * @param AddToWishlist $event
     *
     * @return string
     */
    protected function renderAddToWishlist(AddToWishlist $event)
    {
        $content = '';

        if (!empty($event->value)) {
            $content .= '&cd[value]=' . $event->value->getStringRawUrlEncoded();
        }
        if (!empty($event->currency)) {
            $content .= '&cd[currency]=' . $event->currency->getStringRawUrlEncoded();
        }
        if (!empty($event->content_name)) {
            $content .= '&cd[content_name]=' . $event->content_name->getStringRawUrlEncoded();
        }
        if (!empty($event->content_category)) {
            $content .= '&cd[content_category]=' . $event->content_category->getStringRawUrlEncoded();
        }
        if (!empty($event->content_ids)) {
            $content .= '&cd[content_ids]=' . $this->getContentIdsCommaSeparated($event->content_ids);
        }

        return $content;
    }

    /**
     * @param CompleteRegistration $event
     *
     * @return string
     */
    protected function renderCompleteRegistration(CompleteRegistration $event)
    {
        $content = '';

        if (!empty($event->value)) {
            $content .= '&cd[value]=' . $event->value->getStringRawUrlEncoded();
        }
        if (!empty($event->currency)) {
            $content .= '&cd[currency]=' . $event->currency->getStringRawUrlEncoded();
        }
        if (!empty($event->content_name)) {
            $content .= '&cd[content_name]=' . $event->content_name->getStringRawUrlEncoded();
        }
        if (!empty($event->status)) {
            $content .= '&cd[status]=' . $event->status->getStringRawUrlEncoded();
        }

        return $content;
    }

    /**
     * @param InitiateCheckout $event
     *
     * @return string
     */
    protected function renderInitiateCheckout(InitiateCheckout $event)
    {
        $content = '';

        if (!empty($event->value)) {
            $content .= '&cd[value]=' . $event->value->getStringRawUrlEncoded();
        }
        if (!empty($event->currency)) {
            $content .= '&cd[currency]=' . $event->currency->getStringRawUrlEncoded();
        }
        if (!empty($event->content_name)) {
            $content .= '&cd[content_name]=' . $event->content_name->getStringRawUrlEncoded();
        }
        if (!empty($event->content_category)) {
            $content .= '&cd[content_category]=' . $event->content_category->getStringRawUrlEncoded();
        }
        if (!empty($event->content_ids)) {
            $content .= '&cd[content_ids]=' . $this->getContentIdsCommaSeparated($event->content_ids);
        }
        if (!empty($event->num_items)) {
            $content .= '&cd[num_items]=' . $event->num_items->getInt();
        }

        return $content;
    }

    /**
     * @param Lead $event
     *
     * @return string
     */
    protected function renderLead(Lead $event)
    {
        $content = '';

        if (!empty($event->value)) {
            $content .= '&cd[value]=' . $event->value->getStringRawUrlEncoded();
        }
        if (!empty($event->currency)) {
            $content .= '&cd[currency]=' . $event->currency->getStringRawUrlEncoded();
        }
        if (!empty($event->content_name)) {
            $content .= '&cd[content_name]=' . $event->content_name->getStringRawUrlEncoded();
        }
        if (!empty($event->content_category)) {
            $content .= '&cd[content_category]=' . $event->content_category->getStringRawUrlEncoded();
        }

        return $content;
    }

    /**
     * @param Purchase $event
     *
     * @return string
     */
    protected function renderPurchase(Purchase $event)
    {
        $content = '';

        if (!empty($event->value)) {
            $content .= '&cd[value]=' . $event->value->getStringRawUrlEncoded();
        }
        if (!empty($event->currency)) {
            $content .= '&cd[currency]=' . $event->currency->getStringRawUrlEncoded();
        }
        if (!empty($event->content_name)) {
            $content .= '&cd[content_name]=' . $event->content_name->getStringRawUrlEncoded();
        }
        if (!empty($event->content_type)) {
            $content .= '&cd[content_type]=' . $event->content_type->getStringRawUrlEncoded();
        }
        if (!empty($event->content_ids)) {
            $content .= '&cd[content_ids]=' . $this->getContentIdsCommaSeparated($event->content_ids);
        }
        if (!empty($event->num_items)) {
            $content .= '&cd[num_items]=' . $event->num_items->getInt();
        }

        return $content;
    }

    /**
     * @param Search $event
     *
     * @return string
     */
    protected function renderSearch(Search $event)
    {
        $content = '';

        if (!empty($event->value)) {
            $content .= '&cd[value]=' . $event->value->getStringRawUrlEncoded();
        }
        if (!empty($event->currency)) {
            $content .= '&cd[currency]=' . $event->currency->getStringRawUrlEncoded();
        }
        if (!empty($event->content_category)) {
            $content .= '&cd[content_category]=' . $event->content_category->getStringRawUrlEncoded();
        }
        if (!empty($event->content_ids)) {
            $content .= '&cd[content_ids]=' . $this->getContentIdsCommaSeparated($event->content_ids);
        }
        if (!empty($event->search_string)) {
            $content .= '&cd[search_string]=' . $event->search_string->getStringRawUrlEncoded();
        }

        return $content;
    }

    /**
     * @param ViewContent $event
     *
     * @return string
     */
    protected function renderViewContent(ViewContent $event)
    {
        $content = '';

        if (!empty($event->value)) {
            $content .= '&cd[value]=' . $event->value->getStringRawUrlEncoded();
        }
        if (!empty($event->currency)) {
            $content .= '&cd[currency]=' . $event->currency->getStringRawUrlEncoded();
        }
        if (!empty($event->content_name)) {
            $content .= '&cd[content_name]=' . $event->content_name->getStringRawUrlEncoded();
        }
        if (!empty($event->content_type)) {
            $content .= '&cd[content_type]=' . $event->content_type->getStringRawUrlEncoded();
        }
        if (!empty($event->content_ids)) {
            $content .= '&cd[content_ids]=' . $this->getContentIdsCommaSeparated($event->content_ids);
        }

        return $content;
    }

    /**
     * @param ContentId[] $contentIds
     *
     * @return string
     */
    public function getContentIdsCommaSeparated(array $contentIds)
    {
        $contentIdsArray = array();
        foreach ($contentIds as $contentId) {
            $contentIdsArray[] = $contentId->getStringRawUrlEncoded();
        }

        return implode(',', $contentIdsArray);
    }
}