<?php

namespace Shopgate\FacebookPixelLibrary\Setting\Factory;

use Shopgate\FacebookPixelLibrary\Setting\Entity\Details;
use Shopgate\FacebookPixelLibrary\Setting\Entity\EventSetting;
use Shopgate\FacebookPixelLibrary\Setting\Entity\SettingsSection;
use Shopgate\FacebookPixelLibrary\Setting\Repository\Provider\Translation;
use Shopgate\FacebookPixelLibrary\Setting\ValueObject\EventName;
use Shopgate\FacebookPixelLibrary\Setting\ValueObject\EventSettingFieldName;

class SettingsSectionDefaultFactory implements SettingsSectionFactory
{
    /** @var Translation */
    private $translationRepository;

    /**
     * @param Translation $translationRepository
     */
    public function __construct(Translation $translationRepository)
    {
        $this->translationRepository = $translationRepository;
    }

    public function getSettingsSection(array $supportedEvents = array())
    {
        $settings = array();
        foreach ($supportedEvents as $supportedEvent) {
            $settings[] = $this->createEventSetting($supportedEvent);
        }

        return new SettingsSection(
            $this->translationRepository->getSettingsSectionTitle(),
            new Details(
                $this->translationRepository->getDetailsLabel(),
                $this->translationRepository->getDetailsLink(),
                $this->translationRepository->getDetailsDescription()
            ),
            $settings
        );
    }

    public function getSettingsSectionForAllEvents()
    {
        return $this->getSettingsSection(array(
            new EventName(EventName::VIEW_CONTENT),
            new EventName(EventName::ADD_TO_CART),
            new EventName(EventName::INITIATE_CHECKOUT),
            new EventName(EventName::PURCHASE),
            new EventName(EventName::SEARCH),
            new EventName(EventName::ADD_TO_WISHLIST),
            new EventName(EventName::ADD_PAYMENT_INFO),
            new EventName(EventName::LEAD),
            new EventName(EventName::COMPLETE_REGISTRATION),
        ));
    }

    /**
     * @param EventName $eventName
     *
     * @return EventSetting
     */
    private function createEventSetting(EventName $eventName)
    {
        $labelTranslationMethodName = "getLabel{$eventName->getString()}Setting";

        return new EventSetting(
            new EventSettingFieldName("facebook_pixel_{$eventName->getSettingsFieldName()}_event_active"),
            $this->translationRepository->{$labelTranslationMethodName}(),
            $this->translationRepository->getEventSettingActiveLabel(),
            $this->translationRepository->getEventSettingInactiveLabel()
        );
    }
}