<?php

namespace Shopgate\FacebookPixelLibrary\Tests\Unit\Base\ValueObject;

use Shopgate\FacebookPixelLibrary\Base\ValueObject\PositiveIntegerValueObject;

class PositiveIntegerValueObjectTest extends \PHPUnit_Framework_TestCase
{
    /**
     * @param mixed $inputParameter
     *
     * @dataProvider provideInvalidParameter
     */
    public function testInvalidParameter($inputParameter)
    {
        $this->setExpectedException('\InvalidArgumentException');
        new PositiveIntegerValueObject($inputParameter);
    }

    /**
     * @return array
     */
    public function provideInvalidParameter()
    {
        return array(
            'string'       => array(
                'test',
            ),
            'empty string' => array(
                '',
            ),
            'null'         => array(
                null,
            ),
            'float'        => array(
                1.0,
            ),
            'stdClass'     => array(
                new \stdClass(),
            ),
            'array'        => array(
                array(),
            ),
            'negative int' => array(
                -3,
            ),
        );
    }

    /**
     * @param mixed $inputParameter
     *
     * @dataProvider provideValidParameter
     */
    public function testValidParameter($inputParameter)
    {
        new PositiveIntegerValueObject($inputParameter);
    }

    /**
     * @return array
     */
    public function provideValidParameter()
    {
        return array(
            'positive int' => array(
                4,
            ),
            'zero'         => array(
                0,
            ),
        );
    }
}