<?php

namespace Shopgate\FacebookPixelLibrary\Tests\Unit\Event\Service;

use Shopgate\FacebookPixelLibrary\Base\Repository\Settings;
use Shopgate\FacebookPixelLibrary\Event\Event\FacebookTrackingEvent;
use Shopgate\FacebookPixelLibrary\Event\Service\ImageTag;
use Shopgate\FacebookPixelLibrary\Tests\Fixtures\Unit\Event\Service\FixtureProvider;

class ImageTagTest extends \PHPUnit_Framework_TestCase
{
    /** @var ImageTag */
    private $subjectUnderTest;

    /** @var FixtureProvider */
    private $fixtureProvider;

    /** @var Settings|\PHPUnit_Framework_MockObject_MockObject */
    private $settingsMock;

    public function __construct($name = null, array $data = array(), $dataName = '')
    {
        $this->fixtureProvider = new FixtureProvider();

        parent::__construct($name, $data, $dataName);
    }

    public function setUp()
    {
        $this->settingsMock     = $this->getSettingsMock();
        $this->subjectUnderTest = new ImageTag($this->settingsMock);
    }

    /**
     * @param string                  $expectedImageCode
     * @param FacebookTrackingEvent[] $facebookTrackingEvents
     *
     * @dataProvider provideRenderCases
     */
    public function testRender($expectedImageCode, array $facebookTrackingEvents)
    {
        $this->settingsMock->method('getFacebookPixelAddPaymentInfoEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelAddToCartEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelAddToWishlistEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelCompleteRegistrationEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelInitiateCheckoutEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelLeadEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelPurchaseEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelSearchEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelViewContentEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelId')->willReturn('1234567');
        $this->assertEquals(trim($expectedImageCode), $this->subjectUnderTest->render($facebookTrackingEvents));
    }

    /**
     * @return array
     */
    public function provideRenderCases()
    {
        return $this->fixtureProvider->provideImageRenderCases();
    }

    /**
     * @param FacebookTrackingEvent[] $facebookTrackingEvents
     *
     * @dataProvider provideAllEvents
     */
    public function testEmptyFacebookId(array $facebookTrackingEvents)
    {
        $this->settingsMock->method('getFacebookPixelAddPaymentInfoEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelAddToCartEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelAddToWishlistEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelCompleteRegistrationEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelInitiateCheckoutEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelLeadEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelPurchaseEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelSearchEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelViewContentEventActive')->willReturn(true);
        $this->settingsMock->method('getFacebookPixelId')->willReturn('');
        $this->assertEquals('', $this->subjectUnderTest->render($facebookTrackingEvents));
    }

    /**
     * @return array
     */
    public function provideAllEvents()
    {
        return $this->fixtureProvider->provideAllEvents();
    }

    /**
     * @param FacebookTrackingEvent[] $facebookTrackingEvents
     *
     * @dataProvider provideAllEvents
     */
    public function testInactiveSettings(array $facebookTrackingEvents)
    {
        $this->settingsMock->method('getFacebookPixelAddPaymentInfoEventActive')->willReturn(false);
        $this->settingsMock->method('getFacebookPixelAddToCartEventActive')->willReturn(false);
        $this->settingsMock->method('getFacebookPixelAddToWishlistEventActive')->willReturn(false);
        $this->settingsMock->method('getFacebookPixelCompleteRegistrationEventActive')->willReturn(false);
        $this->settingsMock->method('getFacebookPixelInitiateCheckoutEventActive')->willReturn(false);
        $this->settingsMock->method('getFacebookPixelLeadEventActive')->willReturn(false);
        $this->settingsMock->method('getFacebookPixelPurchaseEventActive')->willReturn(false);
        $this->settingsMock->method('getFacebookPixelSearchEventActive')->willReturn(false);
        $this->settingsMock->method('getFacebookPixelViewContentEventActive')->willReturn(false);
        $this->settingsMock->method('getFacebookPixelId')->willReturn('1234567');

        $this->assertEquals('', $this->subjectUnderTest->render($facebookTrackingEvents));
    }

    /**
     * @return \PHPUnit_Framework_MockObject_MockObject | Settings
     */
    private function getSettingsMock()
    {
        return $this->getMockBuilder('\Shopgate\FacebookPixelLibrary\Base\Repository\Settings')
                    ->disableOriginalConstructor()
                    ->getMock();
    }
}