<?php

namespace Shopgate\FacebookPixelLibrary\Tests\Unit\Event\Service;

use Shopgate\FacebookPixelLibrary\Base\Repository\Settings;
use Shopgate\FacebookPixelLibrary\Event\Event\FacebookTrackingEvent;
use Shopgate\FacebookPixelLibrary\Event\Service\ImageTag;
use Shopgate\FacebookPixelLibrary\Event\Service\JavaScriptTag;
use Shopgate\FacebookPixelLibrary\Event\ValueObject\TrackingType;
use Shopgate\FacebookPixelLibrary\Tests\Fixtures\Unit\Event\Service\FixtureProvider;

class JavaScriptTagTest extends \PHPUnit_Framework_TestCase
{
    /** @var JavaScriptTag */
    private $subjectUnderTest;

    /** @var FixtureProvider */
    private $fixtureProvider;

    /** @var ImageTag|\PHPUnit_Framework_MockObject_MockObject */
    private $imageTagRenderer;

    /** @var Settings|\PHPUnit_Framework_MockObject_MockObject */
    private $settings;

    public function __construct($name = null, array $data = array(), $dataName = '')
    {
        $this->fixtureProvider = new FixtureProvider();

        parent::__construct($name, $data, $dataName);
    }

    public function setUp()
    {
        $this->settings = $this
            ->getMockBuilder('\Shopgate\FacebookPixelLibrary\Base\Repository\Settings')
            ->getMockForAbstractClass();

        $this->imageTagRenderer = $this
            ->getMockBuilder('Shopgate\FacebookPixelLibrary\Event\Service\EventRenderer')
            ->disableOriginalConstructor()
            ->getMock();

        $this->subjectUnderTest = new JavaScriptTag(
            $this->imageTagRenderer,
            $this->settings,
            new TrackingType(TrackingType::TRACK)
        );
    }

    /**
     * @return array
     */
    public function provideRenderCases()
    {
        return $this->fixtureProvider->provideJavaScriptRenderCases();
    }

    /**
     * @param string                  $expectedJavaScriptCode
     * @param FacebookTrackingEvent[] $facebookTrackingEvents
     *
     * @dataProvider provideRenderCases
     */
    public function testRender($expectedJavaScriptCode, array $facebookTrackingEvents)
    {
        $this->imageTagRenderer
            ->expects($this->once())
            ->method('render')
            ->with($facebookTrackingEvents)
            ->willReturn('<img width="0" height="0" src="test" />');

        $this->settings->method('getFacebookPixelAddPaymentInfoEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelAddToCartEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelAddToWishlistEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelCompleteRegistrationEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelInitiateCheckoutEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelLeadEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelPurchaseEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelSearchEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelViewContentEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelId')->willReturn('1234567');
        $this->assertEquals(trim($expectedJavaScriptCode), $this->subjectUnderTest->render($facebookTrackingEvents));
    }

    /**
     * @param FacebookTrackingEvent[] $facebookTrackingEvents
     *
     * @dataProvider provideAllEvents
     */
    public function testEmptyFacebookId(array $facebookTrackingEvents)
    {
        $this->settings->method('getFacebookPixelAddPaymentInfoEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelAddToCartEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelAddToWishlistEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelCompleteRegistrationEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelInitiateCheckoutEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelLeadEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelPurchaseEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelSearchEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelViewContentEventActive')->willReturn(true);
        $this->settings->method('getFacebookPixelId')->willReturn('');
        $this->assertEquals('', $this->subjectUnderTest->render($facebookTrackingEvents));
    }

    /**
     * @return array
     */
    public function provideAllEvents()
    {
        return $this->fixtureProvider->provideAllEvents();
    }

    /**
     * @param FacebookTrackingEvent[] $facebookTrackingEvents
     *
     * @dataProvider provideAllEvents
     */
    public function testInactiveSettings(array $facebookTrackingEvents)
    {
        $this->imageTagRenderer
            ->expects($this->never())
            ->method('render');

        $this->settings->method('getFacebookPixelAddPaymentInfoEventActive')->willReturn(false);
        $this->settings->method('getFacebookPixelAddToCartEventActive')->willReturn(false);
        $this->settings->method('getFacebookPixelAddToWishlistEventActive')->willReturn(false);
        $this->settings->method('getFacebookPixelCompleteRegistrationEventActive')->willReturn(false);
        $this->settings->method('getFacebookPixelInitiateCheckoutEventActive')->willReturn(false);
        $this->settings->method('getFacebookPixelLeadEventActive')->willReturn(false);
        $this->settings->method('getFacebookPixelPurchaseEventActive')->willReturn(false);
        $this->settings->method('getFacebookPixelSearchEventActive')->willReturn(false);
        $this->settings->method('getFacebookPixelViewContentEventActive')->willReturn(false);
        $this->settings->method('getFacebookPixelId')->willReturn('1234567');

        $this->assertEquals('', $this->subjectUnderTest->render($facebookTrackingEvents));
    }
}