<?php

namespace Shopgate\FacebookPixelLibrary\Tests\Unit\Base\ValueObject;

use Shopgate\FacebookPixelLibrary\Event\ValueObject\Currency;

class CurrencyTest extends \PHPUnit_Framework_TestCase
{
    /**
     * @param mixed $inputParameter
     *
     * @dataProvider provideInvalidParameter
     */
    public function testInvalidParameter($inputParameter)
    {
        $this->setExpectedException('\InvalidArgumentException');
        new Currency($inputParameter);
    }

    /**
     * @return array
     */
    public function provideInvalidParameter()
    {
        return array(
            'less than 3 characters' => array(
                'US',
            ),
            'More than 3 characters' => array(
                'USDD',
            ),
            'lower case currency'    => array(
                'usd',
            ),
            'null'                   => array(
                null,
            ),
            'float'                  => array(
                1.0,
            ),
            'stdClass'               => array(
                new \stdClass(),
            ),
            'array'                  => array(
                array(),
            ),
            'negative int'           => array(
                -3,
            ),
            'positive int'           => array(
                4,
            ),
            'zero'                   => array(
                0,
            ),
        );
    }

    /**
     * @param mixed $inputParameter
     *
     * @dataProvider provideValidParameter
     */
    public function testValidParameter($inputParameter)
    {
        new Currency($inputParameter);
    }

    /**
     * @return array
     */
    public function provideValidParameter()
    {
        return array(
            'upper case currency USD' => array(
                'USD',
            ),
            'upper case currency EUR' => array(
                'EUR',
            ),
        );
    }
}