<?php

namespace Shopgate\FacebookPixelLibrary\Tests\Unit\Base\ValueObject;

use Shopgate\FacebookPixelLibrary\Event\ValueObject\TrackingType;

class TrackingTypeTest extends \PHPUnit_Framework_TestCase
{
    /**
     * @param mixed $inputParameter
     *
     * @dataProvider provideInvalidParameter
     */
    public function testInvalidParameter($inputParameter)
    {
        $this->setExpectedException('\InvalidArgumentException');
        new TrackingType($inputParameter);
    }

    /**
     * @return array
     */
    public function provideInvalidParameter()
    {
        return array(
            'null'         => array(
                null,
            ),
            'float'        => array(
                1.0,
            ),
            'stdClass'     => array(
                new \stdClass(),
            ),
            'array'        => array(
                array(),
            ),
            'negative int' => array(
                -3,
            ),
            'positive int' => array(
                4,
            ),
            'zero'         => array(
                0,
            ),
        );
    }

    /**
     * @param mixed $inputParameter
     *
     * @dataProvider provideValidParameter
     */
    public function testValidParameter($inputParameter)
    {
        new TrackingType($inputParameter);
    }

    /**
     * @return array
     */
    public function provideValidParameter()
    {
        return array(
            'track'    => array(
                TrackingType::TRACK,
            ),
            'track custom' => array(
                TrackingType::TRACK_CUSTOM,
            ),
        );
    }
}